@@ -1,6 +1,6 @@
 /*
     ac.c - aircraft trace
-    Copyright (C) 2013-2015 Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2013-2016 Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -53,7 +53,7 @@ struct acs *init_acs(void){
 
 	if (cfg->ac_url && *cfg->ac_url){
 		acs->url = g_strdup(cfg->ac_url);
-	}else{
+	/*}else{
 		double myw, myh, minw, maxw, minh, maxh;
 		int qtf;
 		char *wwl = cfg->pwwlo;
@@ -71,7 +71,7 @@ struct acs *init_acs(void){
 			maxw = Z_MAX(maxw, w2);
 		}
 
-		acs->url = g_strdup_printf("arn.data.fr24.com/zones/fcgi/feed.js?bounds=%f,%f,%f,%f&faa=1&mlat=1&flarm=1&adsb=1&gnd=1&air=1", maxw * 180.0/M_PI, minw * 180.0/M_PI, minh * 180.0/M_PI, maxh * 180.0/M_PI);
+		acs->url = g_strdup_printf("data-live.flightradar24.com/zones/fcgi/feed.js?bounds=%f,%f,%f,%f&faa=1&mlat=1&flarm=1&adsb=1&gnd=1&air=1", maxw * 180.0/M_PI, minw * 180.0/M_PI, minh * 180.0/M_PI, maxh * 180.0/M_PI);*/
 	}
 
 	
@@ -111,6 +111,8 @@ void free_acs(struct acs *acs){
 }
 
 // feed since 1/2015 http://arn.data.fr24.com/zones/fcgi/feed.js?bounds=55,44,4,24&faa=1&mlat=1&flarm=1&adsb=1&gnd=1&air=1
+// feed since 4/2016 http://data-live.flightradar24.com/zones/fcgi/feed.js?bounds=55,44,4,24&faa=1&mlat=1&flarm=1&adsb=1&gnd=1&air=1
+
 // thread
 void acs_load(struct acs *acs, const char *data){
 	char *c, *token_ptr = NULL;
@@ -150,7 +152,10 @@ void acs_load(struct acs *acs, const char *data){
 		c = strtok_r(NULL, AC_SEP, &token_ptr);	 // asl in feets
         if (!c) goto x;
 		ac->asl = atoi(c) * 0.3048;
-		if (ac->asl < cfg->ac_minalt) goto x; // unusable altitude
+		if (ac->asl < cfg->ac_minalt){ // unusable altitude
+			g_free(ac);
+			continue;
+		}
 
 		//if (ac->asl < 2000) goto abrt;
 		if (ac->asl > maxasl) maxasl = ac->asl;
@@ -459,6 +464,7 @@ void plot_ac(struct subwin *sw, struct ac *ac){
     //todo outline
     km2px_d(sw, ac->kx, ac->ky, &px, &py);
     r = 16;
+	if (sw->zoom < 10000) r = (r * sw->zoom) / 10000;
     
     //dbg("qtf=%f %f \n", ac->qtf, ac->qtf * 180.0 / M_PI);
 
@@ -664,18 +670,25 @@ void acs_http_timer(void *arg){
 	}
 	
 	acs->http = zhttp_init();
-	zhttp_get(acs->http, zsel, acs->url, acs_downloaded_callback, acs);
+	if (acs->url == NULL){
+		char *url = g_strdup_printf("tucnak.vaiz.cz/acurl.php?mycall=%s&mywwl=%s", ctest ? ctest->pcall : cfg->pcall, ctest ? ctest->pwwlo : cfg->pwwlo);
+		zhttp_get(acs->http, zsel, url, acs_goturl, acs);
+		g_free(url);
+	}else{
+		zhttp_get(acs->http, zsel, acs->url, acs_downloaded_callback, acs);
+	}
 	acs->http_timer = zselect_timer_new(zsel, 50*1000, acs_http_timer, acs); // restart when freezes
 
 	dbg("acs_timer: zhttp_get\n");
 }
 
-void acs_downloaded_callback(struct zhttp *http){
-	char *data;
-	struct timeval tv;
-	int remains;
+void acs_goturl(struct zhttp *http){
+	char *data, *status, *url;
+	int remains = 10000;
 	struct acs *acs = (struct acs *)http->arg;
 
+	dbg("acs_goturl\n");
+
 	if (acs->http_timer){
 		zselect_timer_kill(zsel, acs->http_timer);
 		acs->http_timer = 0;
@@ -683,7 +696,38 @@ void acs_downloaded_callback(struct zhttp *http){
 
 	data = g_new(char, http->response->len + 1);
 	zbinbuf_getstr(http->response, http->dataofs, data, http->response->len + 1);
+
+	if (acs->http->errorstr == NULL){
+		z_split2(data, ';', &status, &url, 0);
+		if (status != NULL && status[0] == 'A'){
+			remains = 1;
+			acs->url = g_strdup(url);
+			g_free(status);
+			g_free(url);
+		}
+	}
+	g_free(data);
+	zhttp_free(http);
+	acs->http = NULL;
+
+	acs->http_timer = zselect_timer_new(zsel, remains, acs_http_timer, acs);
+}
+
+void acs_downloaded_callback(struct zhttp *http){
+	char *data;
+	struct timeval tv;
+	int remains;
+	struct acs *acs = (struct acs *)http->arg;
+
 	dbg("acs_downloaded_callback\n");
+
+	if (acs->http_timer){
+		zselect_timer_kill(zsel, acs->http_timer);
+		acs->http_timer = 0;
+	}
+
+	data = g_new(char, http->response->len + 1);
+	zbinbuf_getstr(http->response, http->dataofs, data, http->response->len + 1);
 	zhttp_free(http);
 	acs->http = NULL;
 
