@@ -1,16 +1,26 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
+
+
 #include "header.h"
 
+#include "bfu.h"
+#include "fifo.h"
+#include "language2.h"
+#include "net.h"
+#include "qsodb.h"
+#include "stats.h"
+#include "tsdl.h"
+
 #define ADIF_STR(iname,item) if (strlen(item)) g_string_sprintfa(gs,"<%s:%d>%s  ",iname, (int)strlen(item),safe_strncpy0(s,item,1024))
-#define ADIF_STR_UC(iname,item) if (strlen(item)) g_string_sprintfa(gs,"<%s:%d>%s  ",iname,(int)strlen(item),uc(safe_strncpy0(s,item,1024)))
+#define ADIF_STR_UC(iname,item) if (strlen(item)) g_string_sprintfa(gs,"<%s:%d>%s  ",iname,(int)strlen(item),z_str_uc(safe_strncpy0(s,item,1024)))
 #define ADIF_DATE(iname,item) if (strlen(item)) g_string_sprintfa(gs,"<%s:%d>%s  ",iname, (int)strlen(item),safe_strncpy0(s,item,1024))
 #define ADIF_INT(iname,item) sprintf(s,"%d",item); g_string_sprintfa(gs,"<%s:%d>%s  ",iname, (int)strlen(s),s)
 
@@ -35,22 +45,22 @@ static int write_adif_header(struct band *b, struct config_band *confb, FILE *f)
     }
     
     gs = g_string_sized_new(1024);
-    g_string_sprintfa(gs, "Created by TUCNAK ver. %s\r\n", VERSION_STRING);
+    g_string_sprintfa(gs, "Created by TUCNAK-%s ver. %s\r\n", Z_PLATFORM, VERSION_STRING);
     g_string_sprintfa(gs, "TName=%s\r\n", safe_strncpy0(s, ctest->tname, 69));
     if (!b->qsos->len)
         g_string_sprintfa(gs, "TDate=%s;%s\r\n", ctest->cdate, ctest->cdate); 
     else
         g_string_sprintfa(gs, "TDate=%d;%d\r\n", b->stats->first_date, b->stats->last_date); 
-    g_string_sprintfa(gs, "PCall=%s\r\n", uc(safe_strncpy0(s, ctest->pcall, 69)));
-    g_string_sprintfa(gs, "PWWLo=%s\r\n", uc(safe_strncpy0(s, ctest->pwwlo, 69)));
-    g_string_sprintfa(gs, "PExch=%s\r\n", uc(safe_strncpy0(s, ctest->pexch, 69)));
+    g_string_sprintfa(gs, "PCall=%s\r\n", z_str_uc(safe_strncpy0(s, ctest->pcall, 69)));
+    g_string_sprintfa(gs, "PWWLo=%s\r\n", z_str_uc(safe_strncpy0(s, ctest->pwwlo, 69)));
+    g_string_sprintfa(gs, "PExch=%s\r\n", z_str_uc(safe_strncpy0(s, ctest->pexch, 69)));
     g_string_sprintfa(gs, "PAdr1=%s\r\n",    safe_strncpy0(s, ctest->padr1, 69));
     g_string_sprintfa(gs, "PAdr2=%s\r\n",    safe_strncpy0(s, ctest->padr2, 69));
     g_string_sprintfa(gs, "PSect=%s\r\n", psect);
 /*    g_string_sprintfa(gs, "PBand=%s\r\n", b->pband);*/
-    g_string_sprintfa(gs, "PClub=%s\r\n", uc(safe_strncpy0(s, ctest->pclub, 69)));
+    g_string_sprintfa(gs, "PClub=%s\r\n", z_str_uc(safe_strncpy0(s, ctest->pclub, 69)));
     g_string_sprintfa(gs, "RName=%s\r\n",    safe_strncpy0(s, ctest->rname, 69));
-    g_string_sprintfa(gs, "RCall=%s\r\n", uc(safe_strncpy0(s, ctest->rcall, 69)));
+    g_string_sprintfa(gs, "RCall=%s\r\n", z_str_uc(safe_strncpy0(s, ctest->rcall, 69)));
     g_string_sprintfa(gs, "RAdr1=%s\r\n",    safe_strncpy0(s, ctest->radr1, 69));
     g_string_sprintfa(gs, "RAdr2=%s\r\n",    safe_strncpy0(s, ctest->radr2, 69));
     g_string_sprintfa(gs, "RPoCo=%s\r\n",    safe_strncpy0(s, ctest->rpoco, 69));
@@ -58,8 +68,8 @@ static int write_adif_header(struct band *b, struct config_band *confb, FILE *f)
     g_string_sprintfa(gs, "RCoun=%s\r\n",    safe_strncpy0(s, ctest->rcoun, 69));
     g_string_sprintfa(gs, "RPhon=%s\r\n",    safe_strncpy0(s, ctest->rphon, 69));
     g_string_sprintfa(gs, "RHBBS=%s\r\n",    safe_strncpy0(s, ctest->rhbbs, 69));
-/*    g_string_sprintfa(gs, "MOpe1=%s\r\n", uc(safe_strncpy0(s, b->mope1, 69)));
-    g_string_sprintfa(gs, "MOpe2=%s\r\n", uc(safe_strncpy0(s, b->mope2, 69)));
+/*    g_string_sprintfa(gs, "MOpe1=%s\r\n", z_str_uc(safe_strncpy0(s, b->mope1, 69)));
+    g_string_sprintfa(gs, "MOpe2=%s\r\n", z_str_uc(safe_strncpy0(s, b->mope2, 69)));
     g_string_sprintfa(gs, "STXEq=%s\r\n",    safe_strncpy0(s, b->stxeq, 69));
     g_string_sprintfa(gs, "SPowe=%s\r\n",    safe_strncpy0(s, b->spowe, 69));
     g_string_sprintfa(gs, "SRXEq=%s\r\n",    safe_strncpy0(s, b->srxeq, 69));
@@ -74,8 +84,8 @@ static int write_adif_header(struct band *b, struct config_band *confb, FILE *f)
     g_string_sprintfa(gs, "CDXCs=%d;%d;%d\r\n", g_hash_table_size(b->stats->dxcs),  ctest->dxcbonu, ctest->dxcmult==0 ? 1 : ctest->dxcmult); 
     g_string_sprintfa(gs, "CDXCB=%d\r\n",       g_hash_table_size(b->stats->dxcs) * ctest->dxcbonu);
     g_string_sprintfa(gs, "CToSc=%d\r\n",       b->stats->ntotal); 
-    g_string_sprintfa(gs, "CODXC=%s;",  uc(safe_strncpy0(s, b->stats->odxcall, 20)));
-    g_string_sprintfa(gs, "%s;%d\r\n",    uc(safe_strncpy0(s, b->stats->odxwwl,  20)), b->stats->odxqrb_int);  */
+    g_string_sprintfa(gs, "CODXC=%s;",  z_str_uc(safe_strncpy0(s, b->stats->odxcall, 20)));
+    g_string_sprintfa(gs, "%s;%d\r\n",    z_str_uc(safe_strncpy0(s, b->stats->odxwwl,  20)), b->stats->odxqrb_int);  */
     convert_lt(gs->str);
     strcpy(dummy,"1.00");
     ADIF_STR("adif_ver",dummy);
@@ -187,8 +197,8 @@ int export_all_bands_adif(void){
     filename = g_strdup_printf("%s/%s_%s.adif",
                     ctest->directory,
                     ctest->cdate,
-                    uc(get_raw_call(callbuf,ctest->pcall)));
-        
+                    z_get_raw_call(callbuf,ctest->pcall));
+    z_wokna(filename);    
     f=fopen(filename,"wb"); /* must be b for windoze */
     if (!f) {
         if (!ignoreerror) { errbox(VTEXT(T_CANT_WRITE), errno); ignoreerror=1;}
@@ -202,6 +212,8 @@ int export_all_bands_adif(void){
         band = g_ptr_array_index(ctest->bands, i);
         confb = get_config_band_by_bandchar(band->bandchar);
 
+		progress(VTEXT(T_EXPORTING_S), band->bandname);
+
         stats_thread_join(band);
         if (band->stats->nqsos <=0) continue;
 
@@ -216,10 +228,12 @@ int export_all_bands_adif(void){
     if (err) {
         if (!ignoreerror) { errbox(VTEXT(T_CANT_WRITE), 0); ignoreerror=1; }
         g_free(filename);
+		progress(NULL);
         return -1;
     }
     log_addf(VTEXT(T_SAVED_S), filename);
     g_free(filename);
+	progress(NULL);
     return 0;
 }
 
@@ -247,7 +261,7 @@ static int adif_read_tag(FILE *f, GString *gs, int *taglen){
                 else
                     *taglen=atoi(items[1]);
                 gs->str[strlen(items[0])]='\0';
-                g_strup(gs->str);
+                z_str_uc(gs->str);
             }
             g_strfreev(items);
             return 0;
@@ -336,7 +350,7 @@ static int adif_add_qso(struct band *band, GHashTable *hash){
         q->error=1;
         q->callsign=g_strdup("");
     }else{
-        q->callsign=uc(g_strdup(call));
+        q->callsign=z_str_uc(g_strdup(call));
     }
     
     mode=SAFE_ADIF_ITEM("MODE");
@@ -368,7 +382,7 @@ static int adif_add_qso(struct band *band, GHashTable *hash){
 
     q->locator=g_strdup(SAFE_ADIF_ITEM("GRIDSQUARE"));
 
-    q->source=g_strdup(net->myid);
+    q->source=g_strdup(gnet->myid);
 
     q->operator_=fixsemi(g_strdup(SAFE_ADIF_ITEM("OPERATOR")));
     
@@ -420,11 +434,11 @@ void import_adif(void *xxx, char *filename){
 /*      dbg("  tag='%s'  \tvalue: '%s'\n", gs->str, gs2->str);*/
         if (strcasecmp(gs->str, "EOR")==0){
 /*            dbg("-------------------------\n");
-            dbg_str_hash(hash);*/
+            dbg_str_hash(hash);*/                       
             adif_add_qso(aband, hash);
-            g_hash_table_foreach_remove(hash, free_gstr, NULL);
+            g_hash_table_foreach_remove(hash, zg_hash_free_item, NULL);
         }else{
-            hash_safe_insert(hash, g_strdup(gs->str), g_strdup(gs2->str));
+            zg_hash_safe_insert(hash, g_strdup(gs->str), g_strdup(gs2->str));
         }
         
         
@@ -435,11 +449,11 @@ void import_adif(void *xxx, char *filename){
     
     g_string_free(gs, TRUE);
     g_string_free(gs2, TRUE);
-    g_hash_table_foreach_remove(hash, free_gstr, NULL);
+    g_hash_table_foreach_remove(hash, zg_hash_free_item, NULL);
     
     
 	g_hash_table_foreach(ctest->bystamp, foreach_source_qsort_by_stamp, NULL);
-    g_index_array_qsort(ctest->allqsos, compare_date_time_qsonrs);
+    z_ptr_array_qsort(ctest->allqsos, compare_date_time_qsonrs);
 
 /*   don't needed   DIRTY_BAND(); */
 
