@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2012  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -10,7 +10,11 @@
 #ifdef HAVE_CONFIG_H
 #include "../config.h"
 #endif
+#include <zconfig.h>
+
+#ifdef Z_HAVE_SDL
 #include "cor.h"
 #include "cordata.h"
 #include "cordata.inc"
+#endif
 
