@@ -598,7 +598,7 @@ void cwdaemon_read_handler(struct cwdaemon *cwda, char *s, char *played){
             if (gses->last_cq->cw_repeat || gses->extcq > 0) 
                 cq_cw_wait(gses->last_cq);
             else{
-                gses->last_cq->type=MOD_NONE;
+                gses->last_cq->mode=MOD_NONE;
                 cq_abort(SSBDRECORDING);
             }
             peer_tx(aband, 0);
@@ -999,7 +999,7 @@ int cq_run_cw(struct cq *cq){
     ssbd_abort(gssbd,1); /*aborts playing or recording */
 #endif
 
-    cq->type=MOD_CW_CW;
+    cq->mode=MOD_CW_CW;
     cwdaemon_ssbway(cwda, 1); /* soundcard but here to allow CQ TX on both bands */
     redraw_later();
 
@@ -1065,7 +1065,7 @@ int cq_run_ssb(struct cq *cq){
     int ret = 0;
 
     dbg("cq_run_ssb\n");
-    cq->type=MOD_SSB_SSB;
+    cq->mode=MOD_SSB_SSB;
     cwdaemon_ptt(cwda, 1, 0);
     cwdaemon_ssbway(cwda, 1); /* soundcard */
     
@@ -1142,12 +1142,10 @@ int cq_run_by_number(int no){
     if (get_mode() == MOD_CW_CW){
         /*dbg(" speed=%d repeat=%d ts=%d '%s'\n", cq->cw_speed, cq->cw_repeat, cq->cw_ts, cq->cw_allowifundef, cq->cw_str);*/
         ret=cq_run_cw(cq);
-    }else if (get_mode() == MOD_SSB_SSB){
+    }else{
         /*dbg(" repeat=%d ts=%d '%s'\n", cq->ssb_repeat, cq->ssb_ts, cq->ssb_file);*/
         ret=cq_run_ssb(cq);
-    }else
-        ret=-1;        
-
+	}
     if (ret!=0) cq_abort(SSBDRECORDING);
 
     return ret;
