@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -8,9 +8,20 @@
 
 */
 
+
 #include "header.h"
 
-#include <string.h>
+#include "bfu.h"
+#include "cwdb.h"
+#include "fifo.h"
+#include "main.h"
+#include "masterdb.h"
+#include "qsodb.h"
+#include "tsdl.h"
+#include "session.h"
+#include "stats.h"
+#include "subwin.h"
+
 struct cw *cw;
 static gchar *cw_next;
 static gchar *wc_next;
@@ -20,9 +31,10 @@ struct cw *init_cw(void){
     time_t now;
     struct tm tm;
 
+	progress(VTEXT(T_INIT_CWDB));		
     cw = g_new0(struct cw, 1);
     
-    cw->cw = t_hash_table_new(g_str_hash, g_str_equal);
+    cw->cw = z_hash_table_new(g_str_hash, g_str_equal);
     cw->wc = g_hash_table_new(g_str_hash, g_str_equal);
     
     cw_next = NULL;
@@ -62,16 +74,18 @@ static gboolean free_wc_item(gpointer key, gpointer value, gpointer user_data){
 }
 
 void free_cw(struct cw *cw){
+	progress(VTEXT(T_TERM_CWDB));		
+
     g_hash_table_foreach_remove(cw->wc, free_wc_item, NULL);
     g_hash_table_destroy(cw->wc);
     
-    t_hash_table_foreach_remove(cw->cw, free_cw_item, NULL);
-    t_hash_table_destroy(cw->cw);
+    z_hash_table_foreach_remove(cw->cw, free_cw_item, NULL);
+    z_hash_table_destroy(cw->cw);
     g_free(cw);
 }
 
 gint get_cw_size(struct cw *cw){
-    return t_hash_table_size(cw->cw);
+    return z_hash_table_size(cw->cw);
 }
 
 gint get_wc_size(struct cw *cw){
@@ -84,9 +98,10 @@ void load_one_cw(struct cw *cw, gchar *s){
     gchar *call, *wwl, *stamp_str, *qrv_str;
     char *token_ptr;
     
-    uc(s);
+    z_str_uc(s);
     call = strtok_r(s, CW_DELIM, &token_ptr);
     if (!call) return;
+	z_strip_from(call, '-');  // before 3.15 in db can be calls with SSID from KST
 
     wwl = strtok_r(NULL, CW_DELIM, &token_ptr);
     if (!wwl) return;
@@ -119,15 +134,36 @@ int load_cw_from_file(struct cw *cw, gchar *filename){
     return 0;
 }
 
+int load_cw_from_mem(struct cw *cw, const char *file, size_t len){
+    GString *gs; 
+	long int pos;
+    
+    gs = g_string_sized_new(100);
+    
+    pos=0;
+    while(1){
+        if (!zfile_mgets(gs, file, &pos, len, 1)) break;
+		load_one_cw(cw, gs->str);
+    }
+    g_string_free(gs, 1);
+    return 0;
+
+}
 
 void read_cw_files(struct cw *cw){
     gchar *s;
+	int ret = 0;
     
-#ifdef UNIX
-    load_cw_from_file(cw, "/etc/tucnakcw");
-#endif
-    s = g_strconcat(getenv("HOME"), "/tucnak/tucnakcw", NULL); 
-    load_cw_from_file(cw, s);
+	progress(VTEXT(T_LOADING_CWDB));		
+
+/*#ifdef UNIX
+    ret |= load_cw_from_file(cw, "/etc/tucnakcw");
+#endif*/
+    s = g_strconcat(tucnak_dir, "/tucnakcw", NULL); 
+	z_wokna(s);
+    ret |= load_cw_from_file(cw, s);
+	if (ret) load_cw_from_mem(cw, txt_tucnakcw, sizeof(txt_tucnakcw));
+	cw->dirty = 0;
     g_free(s);
 }
 
@@ -137,6 +173,7 @@ static void save_one_cw(gpointer key, gpointer value, gpointer user_data){
     struct cw_item *cwi;
     gchar qrv_str[33], *c;
     int i;
+	char s[4096];
 
     gs   = (GString *) user_data;
     call = (gchar *) key;
@@ -149,16 +186,23 @@ static void save_one_cw(gpointer key, gpointer value, gpointer user_data){
     }
     *c='\0';
 
-    if (cwi->wwl0) 
-        g_string_sprintfa(gs, "%-14s %-6s %08d %s\n", call, cwi->wwl0, cwi->stamp0, qrv_str);
+    if (cwi->wwl0){ 
+		sprintf(s, "%-14s %-6s %08d %s\n", call, cwi->wwl0, cwi->stamp0, qrv_str);
+		g_string_append(gs, s);
+		// g_string_sprintfa is incredibely slow under msvc
+        //g_string_sprintfa(gs, "%-14s %-6s %08d %s\n", call, cwi->wwl0, cwi->stamp0, qrv_str);
+	}
     
-    if (cwi->wwl1) 
-        g_string_sprintfa(gs, "%-14s %-6s %08d %s\n", call, cwi->wwl1, cwi->stamp1, qrv_str);
+    if (cwi->wwl1){
+		sprintf(s, "%-14s %-6s %08d %s\n", call, cwi->wwl1, cwi->stamp1, qrv_str);
+		g_string_append(gs, s);
+        //g_string_sprintfa(gs, "%-14s %-6s %08d %s\n", call, cwi->wwl1, cwi->stamp1, qrv_str);
+	}
     
 }
 
 int save_cw_string(struct cw *cw, GString *gs){
-    t_hash_table_foreach(cw->cw, save_one_cw, (gpointer) gs);
+    z_hash_table_foreach(cw->cw, save_one_cw, (gpointer) gs);
     return 0;
 }
 
@@ -180,6 +224,63 @@ int save_cw_into_file(struct cw *cw, gchar *filename){
     return ret;
 }
 
+int save_cw(struct cw *cw, int verbose){
+	char *s;
+	int ret;
+
+	if (!cw->dirty) return 0;
+	s = g_strconcat(tucnak_dir, "/tucnakcw", NULL);
+	z_wokna(s);
+    ret=save_cw_into_file(cw, s);        
+    if (ret){
+        errbox(VTEXT(T_CANT_WRITE), ret);
+    }else{
+		if (verbose) log_addf(VTEXT(T_SAVED_S), s);
+		cw->dirty = 0;
+    }
+    g_free(s);
+	return ret;
+}
+
+
+static void format_one_cw(gpointer key, gpointer value, gpointer user_data){
+    GString *gs;
+    gchar *call;
+    struct cw_item *cwi;
+    gchar qrv_str[33], *c;
+    int i;
+	char s[4096];
+
+    gs   = (GString *) user_data;
+    call = (gchar *) key;
+    cwi  = (struct cw_item *) value;
+    
+    for (i=0, c=qrv_str;i<32;i++){
+        if (!(cwi->qrv & (1<<i))) continue;
+        *c='A'+i;
+        c++;
+    }
+    *c='\0';
+
+    if (cwi->wwl0){ 
+		sprintf(s, "%s %s %d %s;", call, cwi->wwl0, cwi->stamp0, qrv_str);
+		g_string_append(gs, s);
+		// g_string_sprintfa is incredibely slow under msvc
+        //g_string_sprintfa(gs, "%-14s %-6s %08d %s\n", call, cwi->wwl0, cwi->stamp0, qrv_str);
+	}
+    
+    if (cwi->wwl1){
+		sprintf(s, "%4s %s %d %s;", call, cwi->wwl1, cwi->stamp1, qrv_str);
+		g_string_append(gs, s);
+        //g_string_sprintfa(gs, "%-14s %-6s %08d %s\n", call, cwi->wwl1, cwi->stamp1, qrv_str);
+	}
+    
+}
+
+int format_cw_string(struct cw *cw, GString *gs){
+    z_hash_table_foreach(cw->cw, format_one_cw, (gpointer) gs);
+    return 0;
+}
 
 void add_cw(struct cw *cw, gchar *call, gchar *wwl, gint stamp, gchar *qrv_str){
     struct cw_item *cwi;
@@ -188,15 +289,15 @@ void add_cw(struct cw *cw, gchar *call, gchar *wwl, gint stamp, gchar *qrv_str){
     if (stamp<0) return; 
     if (stamp>cw->latest) cw->latest=stamp;
     
-    cwi = t_hash_table_lookup(cw->cw, call);
+    cwi = (struct cw_item *)z_hash_table_lookup(cw->cw, call);
     if (!cwi){
         cwi = g_new0(struct cw_item, 1);
-        t_hash_table_insert(cw->cw, g_strdup(call), cwi);
+        z_hash_table_insert(cw->cw, g_strdup(call), cwi);
     }
     
     if (qrv_str){ /* can be NULL*/
         for (c=qrv_str;*c!='\0';c++){
-            *c=upcase(*c);
+            *c=z_char_uc(*c);
             cwi->qrv|=1<<(*c-'A');
         }
     }
@@ -209,11 +310,13 @@ void add_cw(struct cw *cw, gchar *call, gchar *wwl, gint stamp, gchar *qrv_str){
         if (cwi->wwl1) g_free(cwi->wwl1);
         cwi->wwl1   = g_strdup(wwl);
         cwi->stamp1 = stamp;
+		cw->dirty = 1;
         return;
     }
 
     if (cwi->wwl0 && strcasecmp(cwi->wwl0, wwl)==0) {
         cwi->stamp0 = stamp;
+		cw->dirty = 1;
         return;
     }
 
@@ -223,6 +326,7 @@ void add_cw(struct cw *cw, gchar *call, gchar *wwl, gint stamp, gchar *qrv_str){
     
     cwi->wwl0   = g_strdup(wwl);
     cwi->stamp0 = stamp;
+	cw->dirty = 1;
 }
 
 
@@ -232,7 +336,7 @@ void add_wc(struct cw *cw, gchar *wwl, gchar *call, gint stamp){
     if (stamp<0) return; 
     if (stamp>cw->latest) cw->latest=stamp;
     
-    wci = g_hash_table_lookup(cw->wc, wwl);
+    wci = (struct wc_item *)g_hash_table_lookup(cw->wc, wwl);
     if (!wci){
         wci = g_new0(struct wc_item, 1);
         g_hash_table_insert(cw->wc, g_strdup(wwl), wci);
@@ -270,7 +374,7 @@ gchar *find_wwl_by_call(struct cw *cw, gchar *call){
     if (!call) return cw_next;
     cw_next = NULL;
         
-    cwi = t_hash_table_lookup(cw->cw, call);
+    cwi = (struct cw_item *)z_hash_table_lookup(cw->cw, call);
     if (!cwi) return NULL;
     
     cw_next = cwi->wwl1;
@@ -280,7 +384,7 @@ gchar *find_wwl_by_call(struct cw *cw, gchar *call){
 gchar *find_wwl_by_call_newer(struct cw *cw, gchar *call, int maxstamp){
     struct cw_item *cwi;
 
-    cwi = t_hash_table_lookup(cw->cw, call);
+    cwi = (struct cw_item *)z_hash_table_lookup(cw->cw, call);
     if (!cwi) return NULL;
     if (cwi->stamp0 > maxstamp) return cwi->wwl0;
     if (cwi->stamp1 > maxstamp) return cwi->wwl1;
@@ -316,14 +420,14 @@ gchar *find_qrv_str_by_call(struct cw *cw, gchar *call){
 
     qrv=0;
     
-    cwi = t_hash_table_lookup(cw->cw, call);
+    cwi = (struct cw_item *)z_hash_table_lookup(cw->cw, call);
     if (cwi) qrv|=cwi->qrv;
 
-    cwi = t_hash_table_lookup(cw->cw, get_raw_call(raw, call));
+    cwi = (struct cw_item *)z_hash_table_lookup(cw->cw, z_get_raw_call(raw, call));
     if (cwi) qrv|=cwi->qrv;
 
     c=g_strdup_printf("%s/P", raw);
-    cwi = t_hash_table_lookup(cw->cw, c);
+    cwi = (struct cw_item *)z_hash_table_lookup(cw->cw, c);
     if (cwi) qrv|=cwi->qrv;
     g_free(c);
 
@@ -331,7 +435,7 @@ gchar *find_qrv_str_by_call(struct cw *cw, gchar *call){
     
     for (i='A',c=qrv_str; i<='Z'; i++){
         
-        if (i==toupper(aband->bandchar)) continue;          /* same band */
+        if (i==aband->bandchar) continue;          /* same band */
         if (!(qrv & ctest->qrv & (1<<(i-'A')))) continue;   /* we are not qrv on band */
 
         band = find_band_by_bandchar(i);
@@ -339,8 +443,16 @@ gchar *find_qrv_str_by_call(struct cw *cw, gchar *call){
             for (j=0; j<band->qsos->len; j++){
                 q = (struct qso *)g_ptr_array_index(band->qsos, j);
                 if (q->error || q->dupe) continue;
-                get_raw_call(raw2, q->callsign);
-                if (strcmp(raw2, raw)==0) goto next;
+                z_get_raw_call(raw2, q->callsign);
+                /*if (!cfg->daqrv){
+                    if (strcmp(raw2, raw)==0) goto next;
+                }else{ */
+                    if (strcmp(raw2, raw)==0) {
+                        *c = z_char_lc(i);
+                        c++;
+                        goto next;
+                    }
+                //}
             }
         }
         
@@ -377,7 +489,7 @@ int cmp_cw_qs(gchar *call, struct cw_item *cwi, GPtrArray *result){
     int qrvfilter;
     char degree[2];
 
-#ifdef HAVE_SDL    
+#ifdef Z_HAVE_SDL    
     if (sdl) 
         strcpy(degree, "\xb0");
     else
@@ -395,13 +507,13 @@ int cmp_cw_qs(gchar *call, struct cw_item *cwi, GPtrArray *result){
     *c='\0';
 
     
-    if (my_strstr(call, gses->qs_str)){
+    if (z_strstr(call, gses->qs_str)){
         if (cwi->wwl0) {
-            qrb_qtf_int(cwi->wwl0, &qrb_int, &qtf_int);
+            qrb_qtf_int(ctest ? ctest->pwwlo : cfg->pwwlo,  cwi->wwl0, &qrb_int, &qtf_int);
             c=g_strdup_printf("%-10s %-6s%5d/%3d%s %s", call, cwi->wwl0, qrb_int, qtf_int, degree, qrv_str);
             g_ptr_array_add(result, c);
             if (cwi->wwl1){ /* wwl1 is only together with wwl0 */
-                qrb_qtf_int(cwi->wwl1, &qrb_int, &qtf_int);
+                qrb_qtf_int(ctest ? ctest->pwwlo : cfg->pwwlo,  cwi->wwl1, &qrb_int, &qtf_int);
                 c=g_strdup_printf("%-10s %-6s%5d/%3d%s %s", call, cwi->wwl1, qrb_int, qtf_int, degree, qrv_str);
                 g_ptr_array_add(result, c);
             }
@@ -412,16 +524,16 @@ int cmp_cw_qs(gchar *call, struct cw_item *cwi, GPtrArray *result){
         goto x;
     }
     
-    if (cwi->wwl0 && my_strstr(cwi->wwl0, gses->qs_str)){
-        qrb_qtf_int(cwi->wwl0, &qrb_int, &qtf_int);
+    if (cwi->wwl0 && z_strstr(cwi->wwl0, gses->qs_str)){
+        qrb_qtf_int(ctest ? ctest->pwwlo : cfg->pwwlo,  cwi->wwl0, &qrb_int, &qtf_int);
         c=g_strdup_printf("%-10s %-6s%5d/%3d%s %s", call, cwi->wwl0, qrb_int, qtf_int, degree, qrv_str);
      /*   dbg(",%s", c);*/
         g_ptr_array_add(result, c);
         goto x;
     }
     
-    if (cwi->wwl1 && my_strstr(cwi->wwl1, gses->qs_str)){
-        qrb_qtf_int(cwi->wwl1, &qrb_int, &qtf_int);
+    if (cwi->wwl1 && z_strstr(cwi->wwl1, gses->qs_str)){
+        qrb_qtf_int(ctest ? ctest->pwwlo : cfg->pwwlo,  cwi->wwl1, &qrb_int, &qtf_int);
         c=g_strdup_printf("%-10s %-6s%5d/%3d%s %s", call, cwi->wwl1, qrb_int, qtf_int, degree, qrv_str);
     /*    dbg(",%s", c);  */
         g_ptr_array_add(result, c);
@@ -431,37 +543,28 @@ x:;
     return ret;
 }
 
-int compare_gstring(const void *a, const void *b){
-    gchar **sa, **sb;
-
-    sa = (gchar **)a;
-    sb = (gchar **)b;
-
-    return (strcmp(*sa, *sb));
-}
-
 void get_cw_qs(gchar *str){
     
     qs_thread_kill();
     
     
     if (strlen(str)<2) {
-//        CONDGFREE(gses->qs_master);
-        g_ptr_array_free_items(gses->qs);
+//        zg_free0(gses->qs_master);
+        zg_ptr_array_free_items(gses->qs);
         return;
     }
 
     if (gses && gses->ontop && gses->ontop->type == SWT_HF){
-        CONDGFREE(gses->qs_master);
+        zg_free0(gses->qs_master);
        // ST_START;
         gses->qs_master = masterdb_search(masterdb, str, gses->ontop->w);
        // ST_STOP;
         //log_adds(gses->qs_master);
     }else{
-        CONDGFREE(gses->qs_str);
+        zg_free0(gses->qs_str);
         gses->qs_str=g_strdup(str);
         gses->qs_max_matches = term->y - QSONR_HEIGHT - 4 - cfg->loglines - 1;
-        if (ctest) gses->qs_max_matches -= ctest->spypeers->len;
+		if (ctest && !ctest->oldcontest) gses->qs_max_matches -= ctest->spypeers->len;
         if (ctest && ctest->bands->len==1) gses->qs_max_matches*=2;
         if (!ctest) gses->qs_max_matches*=3;
         qs_thread_create();
@@ -469,55 +572,55 @@ void get_cw_qs(gchar *str){
 }
 
 static gpointer qs_thread_func(gpointer data){
-    char s[256];
     GPtrArray *result;
-    THashNode *node;
+    ZHashNode *node;
     gint i;
-    int count, ret;   
+
+	zg_thread_set_name("Tucnak quicksearch");
 
     result=g_ptr_array_new();
     
-    count=0;
     for (i = 0; i < cw->cw->size; i++){
         for (node = cw->cw->nodes[i]; node; node = node->next){
-            cmp_cw_qs(node->key, node->value, result);
+            cmp_cw_qs((char *)node->key, (struct cw_item*)node->value, result);
             
             if (result->len >= gses->qs_max_matches) goto x;
    
             if (gses->qs_thread_break){
-                g_ptr_array_free_all(result);
+                zg_ptr_array_free_all(result);
                 return NULL;
             }
 
         }
     }
 x:; 
-    g_index_array_qsort((GIndexArray *)result, compare_gstring); 
+    z_ptr_array_qsort((ZPtrArray *)result, z_compare_string); // FIXME GPtr vs. ZPtr ?
     
     MUTEX_LOCK(gses->qs);
-    g_ptr_array_free_all(gses->qs);
+    zg_ptr_array_free_all(gses->qs);
     gses->qs=result;
     MUTEX_UNLOCK(gses->qs);
     
-    sprintf(s, "CWQS\n");
-    ret = write(tpipe->threadpipe_write, s, strlen(s));
+    //sprintf(s, "CWQS\n");
+    //ret = write(tpipe->threadpipe_write, s, strlen(s));
+	zselect_msg_send(zsel, "CWQS");
 
 	return NULL;
 }
 
 void qs_thread_create(void){
-    if (gses->qs_thread) internal_("qs_thread_create gses->qs_thread");
+    if (gses->qs_thread) zinternal("qs_thread_create gses->qs_thread");
     gses->qs_thread_break=0;
-    gses->qs_thread=g_thread_create(qs_thread_func, (gpointer)NULL, TRUE, NULL);
-    if (!gses->qs_thread) internal_("qs_thread_create !gses->qs_thread");
+    gses->qs_thread=g_thread_try_new("quicksearch", qs_thread_func, (gpointer)NULL, NULL);
+    if (!gses->qs_thread) zinternal("qs_thread_create !gses->qs_thread");
 }
 
 
 void qs_thread_join(void){
     if (!gses->qs_thread) return;
-    dbg("join qs...\n");
+    //dbg("join qs...\n");
     g_thread_join(gses->qs_thread);
-    dbg("done\n");
+    //dbg("done\n");
     gses->qs_thread=NULL;
 }
 
@@ -573,43 +676,8 @@ void update_cw_from_ctest(struct cw *cw, struct contest *ctest){
 
 
 int similar_calls(const char *call1, const char *call2, int factor, int thr, int p){
-    factor = levenshtein(call1, call2);
+    factor = z_levenshtein(call1, call2);
     p=1;
-#if 0    
-    // this code gives bad results
-    const char *c1, *c2;
-    int f1, f2, f3;
-
-    if (factor>thr) return 9;
-    c1=call1;
-    c2=call2;
-
-    while(*c1!='\0' && *c2!='\0'){
-
-        if (*c1==*c2){
-            c1++;
-            c2++;
-            continue;
-        }
-        f1 = similar_calls(c1+1, c2+1, factor+1, thr, 0); 
-        f2 = similar_calls(c1, c2+1, factor+2, thr, 0);
-        f3 = similar_calls(c1+1, c2, factor+2, thr, 0);
-        if (f1<f2){
-            if (f1<f3){
-                factor=f1;
-            }else{
-                if (f2<f3) factor=f2;
-                else factor=f3;
-            }
-        }else{
-            if (f2<f3) factor=f2;
-            else factor=f3;
-        }
-        break;
-
-    }
-
-#endif    
     if (p) {
 //        dbg("similar %s %s = %d", call1, call2, factor);
 //        printf("  leven=%d", levenshtein_distance(call1, call2));
@@ -619,52 +687,12 @@ int similar_calls(const char *call1, const char *call2, int factor, int thr, int
     return factor;
 }
  
-static int min3(int a, int b, int c)
-{
-    int min = a;
-    if (b < min) min=b;
-    if (c < min) min=c;
-    return min;
-}
-
-int levenshtein(const char *s, const char *t){
-    int i, j, k, n, m, cost, *d, ret;
-    n = strlen(s); 
-    m = strlen(t);
-    if (!n) return -1;
-    if (!m) return -1;
-    
-    // d is a table with n+1 rows and m+1 columns
-    d = mem_alloc(sizeof(int) * (m + 1) * (n + 1));
-    m++;
-    n++;
-    // step 2  
-    for(k = 0; k < n; k++) d[k] = k;    // deletion
-    for(k = 0; k < m; k++) d[k*n] = k;  // insertion
-    // step 3 and 4    
-    for(i = 1; i<n; i++)
-    for(j=1; j<m; j++){
-        // step 5
-        if(s[i - 1] == t[j - 1])
-            cost = 0;
-        else
-            cost = 1;
-        // step 6             
-        d[j * n + i] = min3(
-            d[(j - 1) * n + i] + 1,
-            d[j * n + i - 1] + 1,
-            d[(j - 1) * n + i - 1] + cost);
-    }
-    ret = d[n * m - 1];
-    mem_free(d);
-    return ret;
-}
 
 
 
 int get_susp_ambiguous_call(struct cw *cw, struct band *band, char *call, char *wwl, GString *gs, int thr){
     int i;
-    THashNode *node;
+    ZHashNode *node;
     struct cw_item *cwi;
 
     for (i = 0; i < cw->cw->size; i++){
@@ -683,7 +711,7 @@ int get_susp_ambiguous_call(struct cw *cw, struct band *band, char *call, char *
                 if (factor<=thr){
                     if (!gs) return 1;
 
-                    g_string_append(gs, node->key);
+                    g_string_append(gs, (char *)node->key);
 //                    g_string_append_printf(gs, "(%d<=%d)", factor, thr);
                     g_string_append_c(gs, ' ');
                 }
@@ -693,21 +721,39 @@ int get_susp_ambiguous_call(struct cw *cw, struct band *band, char *call, char *
     return 0;
 }
 
-int cwdb_call_info(GString *gs, gchar *call, gchar *stroke){
+int cwdb_call_info(GString *gs, gchar *call, gchar *stroke, GHashTable *wwls){
     struct cw_item *cwi;
-    char c[40];
+    char c[40], *myqth;
     int i;
-
+	double qrb, qtf;
+    char degree[2];
+ 
     strcpy(c, call);
     strcat(c, stroke);
-    cwi = t_hash_table_lookup(cw->cw, c);
+    cwi = (struct cw_item *)z_hash_table_lookup(cw->cw, c);
     if (!cwi) return 0;
-    if (!cwi->wwl0) return 0;  /* wwl0 should be always defined */
-    g_string_sprintfa(gs, "C_W: %-9s %s %d", c, cwi->wwl0, cwi->stamp0);
-    if (cwi->wwl1) g_string_sprintfa(gs, ", %s %d", cwi->wwl1, cwi->stamp1);
-    g_string_append_c(gs, '\n');
+
+#ifdef Z_HAVE_SDL    
+    if (sdl) 
+        strcpy(degree, "\xb0");
+    else
+#endif
+        strcpy(degree, "");
+
+	myqth = ctest ? ctest->pwwlo : cfg->pwwlo;
+	 /* wwl0 should be always defined */
+    if (cwi->wwl0) {
+		qrbqtf(myqth, cwi->wwl0, &qrb, &qtf, NULL, 0);
+		g_string_sprintfa(gs, "C_W: %-10s %s %6dkm %3d%s  %d\n", c, cwi->wwl0, (int)qrb, (int)qtf, degree, cwi->stamp0);
+		if (wwls && g_hash_table_lookup(wwls, cwi->wwl0) == NULL) g_hash_table_insert(wwls, cwi->wwl0, NULL);
+	}
+    if (cwi->wwl1) {
+		qrbqtf(myqth, cwi->wwl1, &qrb, &qtf, NULL, 0);
+		g_string_sprintfa(gs, "C_W: %-10s %s %6dkm %3d%s  %d\n", c, cwi->wwl1, (int)qrb, (int)qtf, degree, cwi->stamp1);
+		if (wwls && g_hash_table_lookup(wwls, cwi->wwl1) == NULL) g_hash_table_insert(wwls, cwi->wwl1, NULL);
+	}
     if (cwi->qrv>0){
-        g_string_append(gs, "QRV on: ");
+        g_string_append(gs, VTEXT(T_QRV_ON));
         for (i=0;i<26;i++){
             if ((1<<i)&cwi->qrv) g_string_append_c(gs, i+'A');
         }
