@@ -25,6 +25,7 @@
 #include "subwin.h"
 #include "terminal.h"
 #include "tregex.h"
+#include "trig.h"
 #include "tsdl.h"
 #include "qrvdb.h"
 #include "qsodb.h"
@@ -457,6 +458,35 @@ int sw_dxc_kbd_func(struct subwin *sw, struct event *ev, int fw){
 		}
 	}
 
+	if (ev->y & KBD_CTRL){
+		struct spot *spot;
+		struct spotband *sband = get_actual_spotband();
+		if (sband == NULL) return 1;
+		if (sband->current < 0 || sband->current >= sband->freq->len) return 1;
+
+		switch (ev->x){
+			case KBD_UP:
+				if (sband->current + 1 >= sband->freq->len) return 1;
+				sband->current++;
+				spot = (struct spot *)z_ptr_array_index(sband->freq, sband->current);
+				if (spot == NULL) return 1;
+#ifdef HAVE_HAMLIB                
+				trigs_set_qrg(gtrigs, spot->qrg * 1000.0);
+#endif
+				redraw_later();
+				return 1;
+			case KBD_DOWN:
+				if (sband->current - 1 < 0) return 1;
+				sband->current--;
+				spot = (struct spot *)z_ptr_array_index(sband->freq, sband->current);
+				if (spot == NULL) return 1;
+#ifdef HAVE_HAMLIB                
+				trigs_set_qrg(gtrigs, spot->qrg * 1000.0);
+#endif
+				redraw_later();
+				return 1;
+		}
+	}
 
     switch(kbd_action(KM_MAIN,ev)){
         case ACT_ESC:
@@ -558,7 +588,7 @@ int sw_dxc_mouse_func(struct subwin *sw, struct event *ev, int fw){
                 return 1;
             }
 			g_free(sw->callunder);
-			sw->callunder = sw_dxc_call_under(sw, ev->x - sw->x + sw->ho, ev->y - sw->y);
+			sw->callunder = sw_dxc_call_under(sw, ev->x - sw->x + sw->ho, ev->y - sw->y, &sw->qrg);
 			if (!sw->callunder) break;
 			sw_dxc_toggle_highlight(sw, sw->callunder, 1);
             return 1;
@@ -571,7 +601,7 @@ int sw_dxc_mouse_func(struct subwin *sw, struct event *ev, int fw){
                 return 1;
             }
 			g_free(sw->callunder);
-			sw->callunder = sw_dxc_call_under(sw, ev->x - sw->x + sw->ho, ev->y - sw->y);
+			sw->callunder = sw_dxc_call_under(sw, ev->x - sw->x + sw->ho, ev->y - sw->y, &sw->qrg);
 			if (!sw->callunder) break;
 			mi = new_menu(1);
 			mi->rtext = sw->callunder;
@@ -938,14 +968,26 @@ void sw_dxc_disconnect(struct subwin *sw){
 	sw_printf(sw, VTEXT(T_ENTER_TO_DXC));
 }
 
+static void dxc_parse_qrg(char *s, double *out_qrg){
+	if (out_qrg != NULL){
+		double qrg;
+		char *qrgstr = s + 14;
+		if (strlen(qrgstr) > 11) qrgstr[10] = '\0';
+		qrg = atof(qrgstr);
+		if (qrg > 0) *out_qrg = qrg;
+	}
+}
+
+
 #define FREE_SW_CX if (c1) {g_free(c1); c1=NULL;} if (c2) {g_free(c2); c2=NULL;}
 
-char *sw_dxc_call_under(struct subwin *sw, int x, int y){
+char *sw_dxc_call_under(struct subwin *sw, int x, int y, double *out_qrg){
 	int index;
     int ret, len, i1, i2;
     char *c1 = NULL, *c2 = NULL, *s, call[25];
 
 	//dbg("sw_dxc_call_under(sw, %d, %d)\n", x, y);
+	if (out_qrg != NULL) *out_qrg = 0.0;
 
 	if (x >= sw->w){
 		struct spotband *sband = get_actual_spotband();
@@ -958,6 +1000,7 @@ char *sw_dxc_call_under(struct subwin *sw, int x, int y){
 			spot = (struct spot *)g_ptr_array_index(sband->freq, idx);
 			x = 0;						// call begins at zero offset
 			s = spot->callsign;
+			if (out_qrg != NULL) *out_qrg = spot->qrg;
 		}else{
 			return NULL;
 		}
@@ -1022,6 +1065,7 @@ char *sw_dxc_call_under(struct subwin *sw, int x, int y){
 		g_strlcpy(call, s + i1, Z_MIN(i2 - i1 + 1, 20));
 		z_strip_from(call, '-'); //rbn
 		if (z_can_be_call(call, ZCBC_SLASH | ZCBC_MINUS)){
+			dxc_parse_qrg(s, out_qrg);
 			g_free(s);
 			return g_strdup(call);
 		}
@@ -1046,6 +1090,7 @@ but here in upcase
 doit:;    
    /* dbg("ret=%d  c1='%s'  c2='%s'\n", ret, c1, c2);*/
     FREE_SW_CX;
+	dxc_parse_qrg(s, out_qrg);
     g_free(s);
     if (ret) return NULL;
 
@@ -1124,6 +1169,9 @@ void sw_dxc_use(void *itdata, void *menudata){
 	il_add_to_history(INPUTLN(aband), sw->callunder);
 	sw_unset_focus();
     il_set_focus(INPUTLN(aband));
+#ifdef HAVE_HAMLIB
+	trigs_set_qrg(gtrigs, sw->qrg * 1000.0);
+#endif
 }
 /*
 0     6          17       26           39                             70  
@@ -1203,6 +1251,7 @@ void dxc_qrg_changed(freq_t qrg){
 		
 	spot = (struct spot *)z_ptr_array_index(sband->freq, sband->current);
 	min = Z_ABS(spot->qrg * 1000 - qrg);
+	if (min == 0.0) return;		// already on correc frequency. Maybe different call which allows Ctrl+Arrows
 		
 	for (cur = sband->current; cur < sband->freq->len; cur++){
 		sp = (struct spot *)z_ptr_array_index(sband->freq, cur);
@@ -1265,4 +1314,4 @@ int dxc_new_mult(const char *callsign){
    }
 
    return 0;
-}
\ No newline at end of file
+}
