@@ -1083,6 +1083,7 @@ destroy:;
         if (!ctest) {
             init_ctest();
             load_ctest_from_mem(ctest, datedir, hash);
+			update_hw();
             maps_reload();
             chart_reload();
         }
@@ -1251,9 +1252,6 @@ destroy:;
         }
     }
 
-/*#ifdef Z_HAVE_SDL		
-    maps_reload(); dela se pri activate_band 
-#endif*/
 	progress(VTEXT(T_UPDATING_WKD));
     qrv_recalc_wkd(qrv);
 	progress(VTEXT(T_UPDATING_QRBQTF));
