@@ -229,29 +229,35 @@ void read_exc_files(struct excdb *excdb, enum exctype exctype, gchar *excname){
         dbg("disk loading failed, try to load memory defaults '%s'\n", ename);
         if (strcasecmp(ename, "agcw")==0) {
             ret = load_vexc_from_mem(excdb, txt_tucnakvexcagcw, sizeof(txt_tucnakvexcagcw));
-            dbg("    %s %d\n", ename, ret);
         }
         if (strcasecmp(ename, "okres")==0) {
             ret = load_vexc_from_mem(excdb, txt_tucnakvexcokres, sizeof(txt_tucnakvexcokres));
-            dbg("    %s %d\n", ename, ret);
         }
         if (strcasecmp(ename, "rsgbdc")==0) {
             ret = load_vexc_from_mem(excdb, txt_tucnakvexcrsgbdc, sizeof(txt_tucnakvexcrsgbdc));
-            dbg("    %s %d\n", ename, ret);
         }
         if (strcasecmp(ename, "usaca")==0) {
             ret = load_vexc_from_mem(excdb, txt_tucnakvexcusaca, sizeof(txt_tucnakvexcusaca));
-            dbg("    %s %d\n", ename, ret);
         }
         if (strcasecmp(ename, "kac")==0) {
             ret = load_vexc_from_mem(excdb, txt_tucnakvexckac, sizeof(txt_tucnakvexckac));
-            dbg("    %s %d\n", ename, ret);
         }
+        if (strcasecmp(ename, "dok")==0) {
+            ret = load_vexc_from_mem(excdb, txt_tucnakvexcdok, sizeof(txt_tucnakvexcdok));
+        }
+        if (strcasecmp(ename, "wna")==0) {
+            ret = load_vexc_from_mem(excdb, txt_tucnakvexcwna, sizeof(txt_tucnakvexcwna));
+        }
+        if (strcasecmp(ename, "wsa")==0) {
+            ret = load_vexc_from_mem(excdb, txt_tucnakvexcwsa, sizeof(txt_tucnakvexcwsa));
+        }
+        dbg("    %s %d\n", ename, ret);
     }
     g_free(ename);
 
     // set vexcia
     switch(excdb->exctype){
+        case EXC_MULTIPLIED:
         case EXC_VERIFIED:
             g_hash_table_foreach(excdb->vexc, exc_ia_verified, (gpointer) excdb->vexcia);
             break;
@@ -482,6 +488,7 @@ int is_valid_vexc(struct excdb *excdb, gchar *vexc){
  
     switch (ctest->exctype){
         case EXC_FREE:
+        case EXC_MULTIPLIED:
             return 1;
             
         case EXC_VERIFIED:
