@@ -297,7 +297,10 @@ struct fft *init_fft(void){
 #endif
 
 	int i, j;
-    if (!sdl) return NULL;
+#ifdef Z_HAVE_SDL	
+    if (!sdl) 
+#endif
+		return NULL;
 
 	progress(VTEXT(T_INIT_FFT));		
 
