@@ -43,7 +43,8 @@
 
 #include <zconfig.h>
 
-#if defined(Z_HAVE_SDL) && defined(HAVE_LIBFFTW3)
+//#if defined(Z_HAVE_SDL) && defined(HAVE_LIBFFTW3)
+#if defined(HAVE_LIBFFTW3)
 #define USE_FFT
 #define USE_SDR
 #endif
@@ -260,7 +261,7 @@ typedef long long vint;
 #error You have no integer type of size void *
 #endif*/
 
-#ifdef TERMIOS_H
+#ifdef HAVE_TERMIOS_H
 #include <termios.h>
 #endif
 
