@@ -1,6 +1,6 @@
 /*
     hf.c - functions (mostly statistics) for HF contests
-    Copyright (C) 2010-2012 Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2010-2015 Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -99,9 +99,18 @@ int sw_hf_mouse_func(struct subwin *sw, struct event *ev, int fw){
     return 0;
 }
 
+// uses aband
+void sw_hf_notworked_dxcc(gpointer key, gpointer value, gpointer user_data)
+{
+	ZPtrArray *notworked_dxccs = (ZPtrArray *)user_data;
+	char *dxcc = (char*)key;
+
+	if (g_hash_table_lookup(aband->stats->dxcs, key) == NULL) z_ptr_array_add(notworked_dxccs, g_strdup(dxcc));
+}
+
 
 void sw_hf_redraw(struct subwin *sw, struct band *band, int flags){
-    int i,j,x,y,col,idx, x0, y0;
+    int i,j,x,y=0,col,idx, x0, y0;
     int bi;
     struct band *bands[BANDS];
     char s[256], t[256];
@@ -120,11 +129,9 @@ void sw_hf_redraw(struct subwin *sw, struct band *band, int flags){
     if (QSOBOUND < sw->h) set_char(sw->x-1, sw->y + QSOBOUND, ATTR_FRAME|COL_NORM|195);
     clip_char(sw, RATEBOUND, QSOBOUND, ATTR_FRAME|COL_NORM|180);
 
-    // excs
+    // excs multiplier
     if (aband && excdb){
         int l, maxl = 0;
-//        struct miss_struct ms;
-//        miss_compute_ms(&ms, aband);
         x = 0;
         y = - sw->offset2;
         for (i=0; i<excdb->vexcia->len; i++){
@@ -145,8 +152,24 @@ void sw_hf_redraw(struct subwin *sw, struct band *band, int flags){
             x += maxl + 1;
 
         }
-//        z_ptr_array_free_all(ms.ia);
     }
+	// dxcc multiplier
+	if (aband && ctest->dxcmult){
+		ZPtrArray *notworked_dxccs = z_ptr_array_new();
+		g_hash_table_foreach(dw->dw, sw_hf_notworked_dxcc, notworked_dxccs);
+		z_ptr_array_qsort(notworked_dxccs, z_compare_string);
+		for (i = 0; i < notworked_dxccs->len; i++){
+			c = (gchar *)z_ptr_array_index(notworked_dxccs, i);
+            if (x + strlen(c) > RATEBOUND) {
+                x = 0;
+                y++;
+                if (y==QSOBOUND) break;
+            }  
+            clip_printf(sw, x, y, COL_NORM, c);
+            x += strlen(c) + 1;
+		}
+		z_ptr_array_free_all(notworked_dxccs);
+	}
 
 
     // bands prepare
@@ -301,7 +324,7 @@ void sw_hf_redraw(struct subwin *sw, struct band *band, int flags){
 #ifdef Z_HAVE_SDL        
         if (sdl){
             int xx = sw->x + LASTBOUND+2+strlen(s);
-            if (xx < sw->x + sw->w) print_text(xx, sw->y + 9, -1, "", COL_NORM); 
+            if (xx < sw->x + sw->w) print_text(xx, sw->y + 9, -1, "\xb0" /*""*/, COL_NORM); 
         }
         else
 #endif            
