@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -9,18 +9,23 @@
 */
 
 #include "header.h"
+#include "cwdaemon.h"
+#include "fifo.h"
+#include "inpout.h"
+#include "language2.h"
+#include "rc.h"
 
-#ifdef __CYGWIN__
+#if defined(Z_MSC_MINGW_CYGWIN)
 
 int parport_init(struct cwdaemon *cwda){
 	
 	cwda->hInpOut = LoadLibrary("inpout32.dll");
 	if (!cwda->hInpOut) {
-		log_addf("Can't load inpout32.dll, error %d", GetLastError());
+		log_addf(VTEXT(T_CANT_LOAD_INPOUT), GetLastError());
 		return -1;
 	}
 
-	cwda->inp32 = (void*)GetProcAddress(cwda->hInpOut, "Inp32");
+	cwda->inp32 = (short (_stdcall *)(short port))GetProcAddress(cwda->hInpOut, "Inp32");
 	if (!cwda->inp32){
 		log_addf("Can't get entry point for Inp32, error %d", GetLastError());
 		FreeLibrary(cwda->hInpOut);
@@ -28,7 +33,7 @@ int parport_init(struct cwdaemon *cwda){
 		return -2;
 	}
 
-	cwda->out32 = (void*)GetProcAddress(cwda->hInpOut, "Out32");
+	cwda->out32 = (void (_stdcall *)(short port, short data))GetProcAddress(cwda->hInpOut, "Out32");
 	if (!cwda->inp32){
 		log_addf("Can't get entry point for Out32, error %d", GetLastError());
 		FreeLibrary(cwda->hInpOut);
@@ -36,8 +41,8 @@ int parport_init(struct cwdaemon *cwda){
 		return -3;
 	}
 
-	cwda->hInpOut_mutex = g_mutex_new();
-	cwda->baseport = 0x378;
+	MUTEX_INIT(cwda->hInpOut);
+	cwda->baseport = strtol(cfg->cwda_device, NULL, 0);
 	cwda->ctrl = cwda->inp32(cwda->baseport+2);
 	parport_reset(cwda);
     return 0;
@@ -50,13 +55,13 @@ int parport_free(struct cwdaemon *cwda){
 	
 	MUTEX_LOCK(cwda->hInpOut);
 	cwda->ctrl &= 0xf0;
-    cwda->ctrl |= 0x0c; /* /CW=L=1 PTT=L=0 /SSBW=L=1 /ENABLE=H=0 */
+    cwda->ctrl |= 0x0b; /* /CW=L=1 PTT=L=0 /SSBW=L=1 /ENABLE=H=0 */
 	cwda->out32(cwda->baseport+2, cwda->ctrl);
 	MUTEX_UNLOCK(cwda->hInpOut);
 	
 	FreeLibrary(cwda->hInpOut);
 	cwda->hInpOut = NULL;
-    g_mutex_free(cwda->hInpOut_mutex);
+    MUTEX_FREE(cwda->hInpOut);
 	return 0;
 }
 
@@ -66,7 +71,8 @@ int parport_reset(struct cwdaemon *cwda){
 
 	MUTEX_LOCK(cwda->hInpOut);
 	cwda->ctrl &= 0xf0;
-    cwda->ctrl |= 0x0d; /* /CW=L=1 PTT=L=0 /SSBW=L=1 /ENABLE=L=1 */
+	cwda->ctrl &= ~0x04;
+    cwda->ctrl |= 0x0b; /* /CW=L=1 PTT=L=0 /SSBW=L=1 /ENABLE=L=1 */
 	cwda->out32(cwda->baseport+2, cwda->ctrl);
 	MUTEX_UNLOCK(cwda->hInpOut);
 	return 0;
@@ -131,11 +137,22 @@ int parport_band(struct cwdaemon *cwda, int bandsw){
 }
 
 
-void parport_info(){
+void parport_info(GString *gs){
+    HMODULE hInpOut;
     
-    printf("\n  parport_info:\n");
-	printf("Using inpout32.dll\n");
-    
-    printf("\n");
+    g_string_sprintfa(gs, "\n  parport_info:\n");
+	g_string_sprintfa(gs, "Using inpout32.dll");
+
+	hInpOut = LoadLibrary("inpout32.dll");
+	if (!hInpOut) {
+		g_string_sprintfa(gs, ", can't load library\nError %d ", GetLastError());
+        z_lasterror(gs);
+        g_string_sprintfa(gs, "\n");
+	}else{
+        g_string_sprintfa(gs, ", library loaded\n");
+	    FreeLibrary(hInpOut);
+    } 
+
+    g_string_sprintfa(gs, "\n");
 }
 #endif
