@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
     This program is free software; you can redistribute it and/or
@@ -11,6 +11,12 @@
 
 #include "header.h"
 
+#include "kbd.h"
+#include "language2.h"
+#include "main.h"
+#include "tsdl.h"
+#include "terminal.h"
+
 #define OUT_BUF_SIZE    16384
 #define IN_BUF_SIZE 16
 
@@ -26,7 +32,9 @@ struct itrm {
     int sock_out;
     int ctl_in;
     int blocked;
+#ifndef Z_MSC_MINGW
     struct termios t;
+#endif
     int flags;
     char kqueue[IN_BUF_SIZE];
     int qlen;
@@ -37,9 +45,9 @@ struct itrm {
     char *orig_title;
 };
 
-void free_trm(cba_t);
-void in_kbd(cba_t);
-void in_sock(cba_t);
+void free_trm(void *);
+void in_kbd(void *);
+void in_sock(void *);
 
 static struct itrm *ditrm = NULL;
 
@@ -50,18 +58,19 @@ int is_blocked(void)
 
 void free_all_itrms(void)
 {
-    if (ditrm) free_trm((cba_t)ditrm);
+	//progress(VTEXT(T_FREE_TERM));		
+    if (ditrm) free_trm(ditrm);
 }
 
 static void write_ev_queue(struct itrm *itrm)
 {
     int l;
-    if (!itrm->eqlen) internal_("event queue empty");
-    if ((l = write(itrm->sock_out, itrm->ev_queue, itrm->eqlen > 128 ? 128 : itrm->eqlen)) == -1) {
-        free_trm((cba_t)itrm);
+    if (!itrm->eqlen) zinternal("event queue empty");
+    if ((l = z_pipe_write(itrm->sock_out, itrm->ev_queue, itrm->eqlen > 128 ? 128 : itrm->eqlen)) == -1) {
+        free_trm(itrm);
         return;
     }
-#ifdef HAVE_SDL    
+#ifdef Z_HAVE_SDL    
     if (sdl){
         MUTEX_LOCK(sdl->eventpipestate);
         sdl->eventpipestate+=l;
@@ -71,7 +80,14 @@ static void write_ev_queue(struct itrm *itrm)
 #endif
     
     memmove(itrm->ev_queue, itrm->ev_queue + l, itrm->eqlen -= l);
-    if (!itrm->eqlen) set_handlers(itrm->sock_out, get_handler(itrm->sock_out, H_READ), NULL, get_handler(itrm->sock_out, H_ERROR), (cba_t)(void*)get_handler(itrm->sock_out, H_DATA));
+    if (!itrm->eqlen) 
+        zselect_set(zsel,
+            itrm->sock_out, 
+            (ZSELECT_HANDLER)zselect_get(zsel, itrm->sock_out, H_READ), 
+            NULL, 
+            (ZSELECT_HANDLER)zselect_get(zsel, itrm->sock_out, H_ERROR), 
+            zselect_get(zsel, itrm->sock_out, H_DATA)
+        );
 }
 
 static void queue_event(struct itrm *itrm, char *data, int len)
@@ -79,19 +95,19 @@ static void queue_event(struct itrm *itrm, char *data, int len)
     int w = 0;
     if (!len) return;
     if (!itrm->eqlen && can_write(itrm->sock_out)){
-#if defined(_MSC_VER) || defined(__MINGW32__)
+#ifdef Z_MSC_MINGW
 		w = send(itrm->sock_out, data, len, 0);
 #else
 		w = write(itrm->sock_out, data, len);
 #endif
 		if (w <= 0) {
 			/*free_trm(itrm);*/
-			register_bottom_half((void (*)(cba_t))free_trm, (cba_t)itrm);
+			zselect_bh_new(zsel, (void (*)(void *))free_trm, (void *)itrm);
 			return;
 		}
     }
     
-#ifdef HAVE_SDL    
+#ifdef Z_HAVE_SDL    
     if (sdl){
         MUTEX_LOCK(sdl->eventpipestate);
         sdl->eventpipestate+=w;
@@ -102,18 +118,18 @@ static void queue_event(struct itrm *itrm, char *data, int len)
         
     if (w < len) {
         char *c;
-        if (!(c = mem_realloc(itrm->ev_queue, itrm->eqlen + len - w))) {
-            free_trm((cba_t)itrm);
+        if (!(c = (char *)g_realloc(itrm->ev_queue, itrm->eqlen + len - w))) {
+            free_trm(itrm);
             return;
         }
         itrm->ev_queue = c;
         memcpy(itrm->ev_queue + itrm->eqlen, data + w, len - w);
         itrm->eqlen += len - w;
-        set_handlers(itrm->sock_out, 
-                (void (*)(cba_t))get_handler(itrm->sock_out, H_READ), 
-                (void (*)(cba_t))write_ev_queue, 
-                (void (*)(cba_t))free_trm, 
-                (cba_t)itrm);
+        zselect_set(zsel, itrm->sock_out, 
+                (void (*)(void *))zselect_get(zsel, itrm->sock_out, H_READ), 
+                (void (*)(void *))write_ev_queue, 
+                (void (*)(void *))free_trm, 
+                itrm);
     }
 }
 
@@ -158,24 +174,27 @@ static void send_term_sequence(int h,int flags)
     }
 }
 
-void resize_terminal(cba_t cba)
+void resize_terminal(void *arg)
 {
-    struct event ev = { EV_RESIZE, 0, 0, 0 };
+	struct event ev = { EV_RESIZE, 0, 0, GPOINTER_TO_INT(arg) };
     int x, y;
-#ifdef HAVE_SDL    
+#ifdef Z_HAVE_SDL    
     if (sdl){
         if (sdl_get_terminal_size(ditrm->std_out, &x, &y)) return;
     }
     else
 #endif
     {
+#ifndef Z_MSC_MINGW
         if (term_get_terminal_size(ditrm->std_out, &x, &y)) return;
+#endif
     }
     ev.x = x;
     ev.y = y;
     queue_event(ditrm, (char *)&ev, sizeof(struct event));
 }
 
+#ifndef Z_MSC_MINGW
 static int setraw(int fd, struct termios *p)
 {
     struct termios t;
@@ -192,6 +211,7 @@ static int setraw(int fd, struct termios *p)
     return 0;
 }
 
+#endif
 void handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in)
 {
     int x, y;
@@ -200,20 +220,22 @@ void handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in)
     char *ts;
     int xwin, ret;
 
-#ifdef HAVE_SDL    
+#ifdef Z_HAVE_SDL    
     if (sdl){
         ret=sdl_get_terminal_size(ctl_in, &x, &y);
     }
     else
 #endif
     {
+#ifndef Z_MSC_MINGW
         ret=term_get_terminal_size(ctl_in, &x, &y);
+#endif
     }
     if (ret){
         error("ERROR: could not get terminal size");
         return;
     }
-    if (!(itrm = mem_alloc(sizeof(struct itrm)))) return;
+    if (!(itrm = (struct itrm*)g_malloc(sizeof(struct itrm)))) return;
     ditrm = itrm;
     itrm->std_in = std_in;
     itrm->std_out = std_out;
@@ -223,23 +245,28 @@ void handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in)
     itrm->blocked = 0;
     itrm->qlen = 0;
     itrm->tm = -1;
-    itrm->ev_queue = DUMMY;
+    itrm->ev_queue = NULL;
     itrm->eqlen = 0;
     if (ctl_in >= 0) {
+#ifndef Z_MSC_MINGW
         setraw(ctl_in, &itrm->t);
-        set_handlers(std_in, in_kbd, NULL, free_trm, (cba_t)itrm);
+        zselect_set(zsel, std_in, in_kbd, NULL, free_trm, itrm);
+#endif
     }
-    if (sock_in != std_out) set_handlers(sock_in, in_sock, NULL, free_trm, (cba_t)itrm);
+    // condidion always false
+	// if (sock_in != std_out) zselect_set(zsel, sock_in, in_sock, NULL, free_trm, itrm);
     ev.x = x;
     ev.y = y;
+#ifndef Z_MSC_MINGW
     handle_terminal_resize(ctl_in, resize_terminal);
+#endif
     queue_event(itrm, (char *)&ev, sizeof(struct event));
-    xwin = is_xterm() * ENV_XWIN + can_twterm() * ENV_TWIN + (!!getenv("STY")) * ENV_SCREEN + get_system_env();
+    xwin = is_xterm() * ENV_XWIN + can_twterm() * ENV_TWIN + (!!getenv("STY")) * ENV_SCREEN;
     itrm->flags = 0;
     if (!(ts = getenv("TERM"))) ts = "";
     if ((xwin & ENV_TWIN) && !strcmp(ts,"linux")) itrm->flags |= USE_TWIN_MOUSE;
     
-#ifdef HAVE_SDL    
+#ifdef Z_HAVE_SDL    
     if (!sdl){ 
         itrm->mouse_h = handle_mouse(0, (void (*)(void *, char *, int))queue_event, itrm);
     }
@@ -253,15 +280,16 @@ void handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in)
         save_screen();
         send_init_sequence(std_out,itrm->flags);
     }
-    set_window_title(PACKAGE_NAME " " PACKAGE_VERSION);
+    set_window_title("Tucnak "PACKAGE_VERSION);
 }
+#ifndef Z_MSC_MINGW
 
-void unblock_itrm_x(cba_t cba)
+void unblock_itrm_x(void *arg)
 {
-    close_handle(cba);
+    close_handle(arg);
     if (!ditrm) return;
     unblock_itrm(0);
-    resize_terminal(CBA0);
+    resize_terminal(NULL);
 }
 
 int unblock_itrm(int fd)
@@ -269,15 +297,15 @@ int unblock_itrm(int fd)
     struct itrm *itrm = ditrm;
     if (!itrm) return -1;
     /*if (ditrm->sock_out != fd) {
-        internal_("unblock_itrm: bad fd: %d", fd);
+        zinternal("unblock_itrm: bad fd: %d", fd);
         return -1;
     }*/
     if (itrm->ctl_in >= 0 && setraw(itrm->ctl_in, NULL)) return -1;
     itrm->blocked = 0;
     send_init_sequence(itrm->std_out,itrm->flags);
-    set_handlers(itrm->std_in, in_kbd, NULL, free_trm, (cba_t)itrm);
+    zselect_set(zsel, itrm->std_in, in_kbd, NULL, free_trm, itrm);
     handle_terminal_resize(itrm->ctl_in, resize_terminal);
-    unblock_stdin();
+    //unblock_stdin();
     return 0;
 }
 
@@ -286,46 +314,53 @@ void block_itrm(int fd)
     struct itrm *itrm = ditrm;
     if (!itrm) return;
     /*if (ditrm->sock_out != fd) {
-        internal_("block_itrm: bad fd: %d", fd);
+        zinternal("block_itrm: bad fd: %d", fd);
         return;
     }*/
     itrm->blocked = 1;
-    block_stdin();
+    //block_stdin();
     unhandle_terminal_resize(itrm->ctl_in);
     send_term_sequence(itrm->std_out,itrm->flags);
     tcsetattr(itrm->ctl_in, TCSANOW, &itrm->t);
-    set_handlers(itrm->std_in, NULL, NULL, free_trm, (cba_t)itrm);
+    zselect_set(zsel, itrm->std_in, NULL, NULL, free_trm, itrm);
 }
+#endif
 
-void free_trm(cba_t cba)
+void free_trm(void *arg)
 {
     struct itrm *itrm;
 
-    itrm=(struct itrm*)GETCBA(cba, itrm);
+    itrm=(struct itrm*)arg;
     if (!itrm) return;
     set_window_title(itrm->orig_title);
-    if (itrm->orig_title) mem_free(itrm->orig_title), itrm->orig_title = NULL;
+    if (itrm->orig_title) g_free(itrm->orig_title), itrm->orig_title = NULL;
+#ifndef Z_MSC_MINGW
     unhandle_terminal_resize(itrm->ctl_in);
+#endif
     if (itrm->std_out>=0)  send_term_sequence(itrm->std_out,itrm->flags);
+#ifndef Z_MSC_MINGW
     tcsetattr(itrm->ctl_in, TCSANOW, &itrm->t);
+#endif
     if (itrm->std_out>=0) restore_screen();
     if (itrm->mouse_h) unhandle_mouse(itrm->mouse_h);
-    if (itrm->std_in>=0)   set_handlers(itrm->std_in, NULL, NULL, NULL, CBA0);
-    if (itrm->sock_in>=0)  set_handlers(itrm->sock_in, NULL, NULL, NULL, CBA0);
-    if (itrm->std_out>=0)  set_handlers(itrm->std_out, NULL, NULL, NULL, CBA0);
-    if (itrm->sock_out>=0) set_handlers(itrm->sock_out, NULL, NULL, NULL, CBA0);
-    if (itrm->tm != -1) kill_timer(itrm->tm);
-    mem_free(itrm->ev_queue);
-    mem_free(itrm);
+    if (itrm->std_in>=0)   zselect_set(zsel, itrm->std_in, NULL, NULL, NULL, NULL);
+    if (itrm->sock_in>=0)  zselect_set(zsel, itrm->sock_in, NULL, NULL, NULL, NULL);
+    if (itrm->std_out>=0)  zselect_set(zsel, itrm->std_out, NULL, NULL, NULL, NULL);
+    if (itrm->sock_out>=0) zselect_set(zsel, itrm->sock_out, NULL, NULL, NULL, NULL);
+    if (itrm->tm != -1) zselect_timer_kill(zsel, itrm->tm);
+    g_free(itrm->ev_queue);
+    g_free(itrm);
     if (itrm == ditrm) ditrm = NULL;
-     itrm=NULL;
+    itrm=NULL;
 }
 
+#ifndef Z_MSC_MINGW
 void itrm_safe_abort(void){
     struct itrm *itrm = ditrm;
     if (!itrm) return;
     tcsetattr(itrm->ctl_in, TCSANOW, &itrm->t);
 }
+#endif
 
 static void resize_terminal_x(char *text)
 {
@@ -336,7 +371,7 @@ static void resize_terminal_x(char *text)
     x = atoi(text);
     y = atoi(p);
     resize_window(x, y);
-    resize_terminal(CBA0);
+    resize_terminal(NULL);
 }
 
 void dispatch_special(char *text)
@@ -351,25 +386,25 @@ void dispatch_special(char *text)
     }
 }
 
-static char buf[OUT_BUF_SIZE];
+//static char buf[OUT_BUF_SIZE];
 
 /*#define RD ({ char cc; if (p < c) cc = buf[p++]; else if ((dl = hard_read(itrm->sock_in, &cc, 1)) <= 0) {debug("%d %d", dl, errno);goto fr;} cc; })*/
 /*udefine RD ({ char cc; if (p < c) cc = buf[p++]; else if ((hard_read(itrm->sock_in, &cc, 1)) <= 0) goto fr; cc; })*/
 #define RD(xx) { char cc; if (p < c) cc = buf[p++]; else if ((hard_read(itrm->sock_in, &cc, 1)) <= 0) goto fr; xx = cc; }
 
-void in_sock(cba_t cba)
+#if 0  // seems uncalled
+void in_sock(void *arg)
 {
-    char *path, *delete;
-    int pl, dl;
     char ch;
     int fg;
     int c, i, p;
     struct itrm *itrm;
+    GString *path, *delete1;
 
-    itrm=(struct itrm *)GETCBA(cba, itrm);
-    if ((c = read(itrm->sock_in, buf, OUT_BUF_SIZE)) <= 0) {
+    itrm=(struct itrm *)arg;
+    if ((c = z_pipe_read(itrm->sock_in, buf, OUT_BUF_SIZE)) <= 0) {
         fr:
-        free_trm((cba_t)itrm);
+        free_trm(itrm);
         return;
     }
     qwerty:
@@ -392,53 +427,53 @@ void in_sock(cba_t cba)
     p = 0;
     /*fg = RD;*/
     RD(fg);
-    path = init_str();
-    delete = init_str();
-    pl = dl = 0;
+    path = g_string_new("");
+    delete1 = g_string_new("");
     while (1) {
         RD(ch);
         if (!ch) break;
-        add_chr_to_str(&path, &pl, ch);
+        g_string_append_c(path, ch);
     }
     while (1) {
         RD(ch);
         if (!ch) break;
-        add_chr_to_str(&delete, &dl, ch);
+        g_string_append_c(delete1, ch);
     }
-    if (!*path) {
-        dispatch_special(delete);
+    if (!*path->str) {
+        dispatch_special(delete1->str);
     } else {
         int blockh;
         char *param;
         if (is_blocked() && fg) {
-            if (*delete) unlink(delete);
+            if (*delete1->str) unlink(delete1->str);
             goto to_je_ale_hnus;
         }
-        if (!(param = mem_alloc(strlen(path) + strlen(delete) + 3))) goto to_je_ale_hnus;
+        if (!(param = g_malloc(strlen(path->str) + strlen(delete1->str) + 3))) goto to_je_ale_hnus;
         param[0] = fg;
-        strcpy(param + 1, path);
-        strcpy(param + 1 + strlen(path) + 1, delete);
+        strcpy(param + 1, path->str);
+        strcpy(param + 1 + strlen(path->str) + 1, delete1->str);
         if (fg == 1) block_itrm(itrm->ctl_in);
-        if ((blockh = start_thread((void (*)(void *, int))exec_thread, param, strlen(path) + strlen(delete) + 3)) == -1) {
+        if ((blockh = start_thread((void (*)(void *, int))exec_thread, param, strlen(path->str) + strlen(delete1->str) + 3)) == -1) {
             if (fg == 1) unblock_itrm(itrm->ctl_in);
-            mem_free(param);
+            g_free(param);
             goto to_je_ale_hnus;
         }
-        mem_free(param);
+        g_free(param);
         if (fg == 1) {
-            set_handlers(blockh, unblock_itrm_x, NULL, unblock_itrm_x, (cba_t)blockh);
+            zselect_set(zsel, blockh, unblock_itrm_x, NULL, unblock_itrm_x, GINT_TO_POINTER(blockh));
             /*block_itrm(itrm->ctl_in);*/
         } else {
-            set_handlers(blockh, close_handle, NULL, close_handle, (cba_t)blockh);
+            zselect_set(zsel, blockh, close_handle, NULL, close_handle, GINT_TO_POINTER(blockh));
         }
     }
     to_je_ale_hnus:
-    mem_free(path);
-    mem_free(delete);
+    g_string_free(path, TRUE);
+    g_string_free(delete1, TRUE);
     memmove(buf, buf + p, OUT_BUF_SIZE - p);
     c -= p;
     goto qwerty;
 }
+#endif
 
 int process_queue(struct itrm *);
 
@@ -451,7 +486,7 @@ static void kbd_timeout(struct itrm *itrm)
         return;
     }*/
     if (!itrm->qlen) {
-        internal_("timeout on empty queue");
+        zinternal("timeout on empty queue");
         return;
     }
     queue_event(itrm, (char *)&ev, sizeof(struct event));
@@ -535,6 +570,8 @@ static struct key os2xtd[256] = {
 };
 
 static int xterm_button = -1;
+static int xterm_mouse_x = -1;
+static int xterm_mouse_y = -1;
 
 int process_queue(struct itrm *itrm)
 {
@@ -631,10 +668,11 @@ int process_queue(struct itrm *itrm)
                         xterm_button = itrm->kqueue[el] - ' ';
                         el += 5;
                     } else {
-//                        dbg("process_queue: %d\n", itrm->kqueue[el]-' ');
-                        ev.x = itrm->kqueue[el+1] - ' ' - 1;
-                        ev.y = itrm->kqueue[el+2] - ' ' - 1;
-                        ev.b = B_DOWN;
+                        //dbg("process_queue: %d\n", (unsigned char)itrm->kqueue[el+1]);
+                        dbg("process_queue: %02x %02x %02x\n", (unsigned char)itrm->kqueue[el], (unsigned char)itrm->kqueue[el+1], (unsigned char)itrm->kqueue[el+3]);
+                        ev.x = (unsigned char)(itrm->kqueue[el+1]) - ' ' - 1;
+                        ev.y = (unsigned char)(itrm->kqueue[el+2]) - ' ' - 1;
+/*                        ev.b = B_DOWN;
                         if (itrm->kqueue[el] & 4) ev.b = B_DRAG;
                         ev.b |= (itrm->kqueue[el] & BM_BUTT) | B_DOWN;
                         if (itrm->kqueue[el] & 64) ev.b |= 4;
@@ -642,9 +680,40 @@ int process_queue(struct itrm *itrm)
                             ev.b = B_UP;
                             if (xterm_button != -1) ev.b |= xterm_button;
                         }  
-                        /*if ((itrm->kqueue[el] & 4) && ev.b != B_UP) ev.b |= B_DRAG;*/
                         xterm_button = -1;
-                        if ((ev.b & BM_ACT) == B_DOWN) xterm_button = ev.b & BM_BUTT;
+                        if ((ev.b & BM_ACT) == B_DOWN) xterm_button = ev.b & BM_BUTT;*/
+                        ev.b = 0;
+                        switch(itrm->kqueue[el]){
+                            case 0x20: 
+                                ev.b = B_LEFT | B_DOWN;
+                                xterm_button = B_LEFT;
+                                break;
+                            case 0x21: 
+                                ev.b = B_MIDDLE | B_DOWN; 
+                                xterm_button = B_MIDDLE;
+                                break;
+                            case 0x22:
+                                ev.b = B_RIGHT | B_DOWN;
+                                xterm_button = B_RIGHT;
+                                break;
+                            case 0x23:
+                                ev.b = B_UP;
+                                if (xterm_button != -1) ev.b |= xterm_button;
+                                if (ev.x == xterm_mouse_x && ev.y == xterm_mouse_y) ev.b |= B_CLICK;
+                                xterm_button = -1;
+                                break;
+                            case 0x60:
+                                ev.b = B_WHUP | B_DOWN;
+                                xterm_button = B_WHUP;
+                                break;
+                            case 0x61:
+                                ev.b = B_WHDOWN | B_DOWN;
+                                xterm_button = B_WHDOWN;
+                                break;
+
+                        }
+                        xterm_mouse_x = ev.x;
+                        xterm_mouse_y = ev.y;
                         el += 3;
                     }
                     ev.ev = EV_MOUSE;
@@ -709,7 +778,7 @@ int process_queue(struct itrm *itrm)
     }
     l1:
     if (itrm->qlen < el) {
-        internal_("event queue underflow");
+        zinternal("event queue underflow");
         itrm->qlen = el;
     }
     if (ev.x != -1) {
@@ -720,27 +789,30 @@ int process_queue(struct itrm *itrm)
         memmove(itrm->kqueue, itrm->kqueue + el, itrm->qlen -= el);
     }
     end:
-    if (itrm->qlen < IN_BUF_SIZE) set_handlers(itrm->std_in, in_kbd, NULL, free_trm, (cba_t)itrm);
+    if (itrm->qlen < IN_BUF_SIZE) zselect_set(zsel, itrm->std_in, in_kbd, NULL, free_trm, itrm);
     return el;
     ret:
-    itrm->tm = install_timer(ESC_TIMEOUT, (void (*)(cba_t))kbd_timeout, (cba_t)itrm);
+    itrm->tm = zselect_timer_new(zsel, ESC_TIMEOUT, (void (*)(void *))kbd_timeout, itrm);
     return 0;
 }
 
-void in_kbd(cba_t cba)
+void in_kbd(void *arg)
 {
+#ifdef Z_MSC_MINGW
+	zinternal("in_kbd unimplemented");
+#else
     int r;
     struct itrm *itrm;
 
-    itrm=(struct itrm *)GETCBA(cba, itrm);
-    if (itrm->tm != -1) kill_timer(itrm->tm), itrm->tm = -1;
+    itrm=(struct itrm *)arg;
+    if (itrm->tm != -1) zselect_timer_kill(zsel, itrm->tm), itrm->tm = -1;
     if (itrm->qlen >= IN_BUF_SIZE) {
-        set_handlers(itrm->std_in, NULL, NULL, free_trm, (cba_t)itrm);
+        zselect_set(zsel, itrm->std_in, NULL, NULL, free_trm, itrm);
         while (process_queue(itrm));
         return;
     }
     if ((r = read(itrm->std_in, itrm->kqueue + itrm->qlen, IN_BUF_SIZE - itrm->qlen)) <= 0) {
-        free_trm((cba_t)itrm);
+        free_trm(itrm);
         return;
     }
     if ((itrm->qlen += r) > IN_BUF_SIZE) {
@@ -748,4 +820,5 @@ void in_kbd(cba_t cba)
         itrm->qlen = IN_BUF_SIZE;
     }
     while (process_queue(itrm));
+#endif
 }
