@@ -40,6 +40,7 @@
 #include "ppdev.h"
 #include "qrvdb.h"
 #include "qsodb.h"
+#include "rain.h"
 #include "rc.h"
 #include "rotar.h"
 #include "sdev.h"
@@ -702,6 +703,7 @@ static void init(void)
 #endif
 #ifdef Z_HAVE_SDL
 	gbat = zbat_init();
+	grain = init_rain();
 #endif
 
     
@@ -777,6 +779,7 @@ static void terminate_all_subsystems(void)
 
 #ifdef Z_HAVE_SDL    
     sdl_stop_event_thread();
+	free_rain(grain);
 	zbat_free(gbat);
 	free_acs(gacs); gacs = NULL;
 	progress(VTEXT(T_FREE_MAP));		
