@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2014  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2016  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -13,18 +13,21 @@
 #include "ac.h"
 #include "bfu.h"
 #include "cor.h"
+#include "fifo.h"
 #include "html.h"
 #include "icons.h"
 #include "kbdbind.h"
 #include "kbd.h"
 #include "map.h"
 #include "qrvdb.h"
+#include "rain.h"
 #include "rotar.h"
 #include "tsdl.h"
 #include "session.h"
 #include "stats.h"
 #include "subwin.h"
 #include "terminal.h"
+#include "zpng.h"
 
 #define sqr(x) ((x)*(x))       
 
@@ -127,7 +130,7 @@ void zoom(struct subwin *sw, double factor, int centerx, int centery){
 	}
 	sw->gdirty=1;
     redraw_later();
-    //dbg("zoom=%d\n", sw->zoom);
+    dbg("zoom=%d\n", sw->zoom);
 }
 
 int sw_map_mouse_func(struct subwin *sw, struct event *ev, int fw){
@@ -303,7 +306,7 @@ void sw_map_redraw_rect(struct subwin *sw, SDL_Rect *area, struct band *band, in
     SDL_FillRect(sw->screen, &sw->info, sdl->termcol[2]);*/
     
     plot_cor(sw, sw->l1map, area, flags);
-	plot_qth(sw, area);
+	//plot_qth(sw, area);
 //        dumpbitmap(sw->l1map);
     if (ctest && qrv && !fp) {
         plot_qrvs(sw, sw->l1map, band, area);
@@ -316,8 +319,9 @@ void sw_map_redraw_rect(struct subwin *sw, SDL_Rect *area, struct band *band, in
     
 //    sw_map_update_rotar(sw, area);
 //  dbg("                    %dx%d%+d%+d\n", area->x, area->y, area->w, area->h);
-    
+  
     sw_map_redraw_rotar(sw, area, flags);
+	plot_qth(sw, area);
 }					  
 
 
@@ -325,22 +329,15 @@ void sw_map_redraw_rotar(struct subwin *sw, SDL_Rect *area, int flags){
     SDL_Rect rotarea;
     SDL_Rect r;
     
-//    SDL_FillRect(sw->l2rot, area, z_makecol(0, 0, 0));
-//    dbg("rotarea= %dx%d%+d%+d\n", rotarea.x, rotarea.y, rotarea.w, rotarea.h);
-    
-    //rect2(sw->l2rot, &rotarea, z_makecol(250, 0, 0));
     
     SDL_SetClipRect(sw->screen, area);
     memcpy(&r, area, sizeof(r));
     SDL_BlitSurface(sw->l1map, &r, sw->screen, &r);
+	if (sw->type == SWT_MAP){ // no rain for KST A/C info
+	    sw_map_redraw_rain(sw, area, flags);
+	}
     if ((flags & HTML_FOR_PHOTO) == 0) plot_rotars(sw, sw->screen, area, &rotarea);
     
-   // SDL_SetAlpha(sw->l2rot, SDL_SRCALPHA|SDL_RLEACCEL, 128);
-//    memcpy(&r, rotarea, sizeof(r));
-    //dbg("r= %dx%d%+d%+d\n", r.x, r.y, r.w, r.h);
-  //  SDL_BlitSurface(sw->l2rot, &r, sw->screen, &r);
-   // dbg("r= %dx%d%+d%+d\n", r.x, r.y, r.w, r.h);
-    //dbg("                    %dx%d%+d%+d\n", area->x, area->y, area->w, area->h);
 }
 
 void sw_map_update_rotar(struct subwin *sw){
@@ -373,10 +370,173 @@ void sw_map_redraw_icons(struct subwin *sw, SDL_Rect *area, int flags){
 	r.y += sw->map.h - sdl->zoomout->h;
     SDL_BlitSurface(sdl->zoomout, NULL, sdl->screen, &r);
 
+    memcpy(&r, area, sizeof(r));
+	plot_rain_titles(sdl->screen, &r, flags);
 //    if ((flags & HTML_FOR_PHOTO) == 0) plot_rotars(sw, sw->screen, area, &rotarea);
 }
 
 
+void sw_map_redraw_rain(struct subwin *sw, SDL_Rect *aarea, int flags){
+	int ci;
+	if (grain == NULL) return;
+	if ((flags & HTML_FOR_PHOTO) != 0) return;
+
+	for (ci = 0; ci < RAIN_COLORS; ci++){
+		if (grain->wetteronline != NULL) plot_rain_provider(sw, aarea, grain->wetteronline, ci);
+		if (grain->meteox != NULL) plot_rain_provider(sw, aarea, grain->meteox, ci);
+		if (grain->chmi != NULL) plot_rain_provider(sw, aarea, grain->chmi, ci);
+		if (grain->debug) break; 
+	}
+}
+
+void plot_rain_provider(struct subwin *sw, SDL_Rect *aarea, struct rain_provider *provider, int ci)
+{
+	SDL_Rect area;
+    int ix, iy, c, i;
+	int minkx, minky, maxkx, maxky;
+	//double hmult, wmult;
+    double hh1, hh2, ww1, ww2;
+	int kx, ky;
+    int px1, px2, px3 = 0, px4 = 0, py1, py2, py3 = 0, py4 = 0;
+
+    if (provider->img == NULL) return;
+	if (provider->w <= 0) return;
+    
+
+	area.x = aarea->x - 16; // maximal "pixel" size is about 16x16
+	area.y = aarea->y - 16;
+	area.w = aarea->w + 32;
+	area.h = aarea->h + 32;
+
+    //hmult = (provider->h2 - provider->h1) / (provider->hpx2 - provider->hpx1);
+    //wmult = (provider->w2 - provider->w1) / (provider->wpx2 - provider->wpx1);
+
+
+	if (provider->rain->debug){
+		// debug mode - only pixels
+		for (iy = 0; iy < provider->img->h; iy++){
+			for (ix = 0; ix < provider->img->w; ix++){
+				c = z_getpixel_fmt(provider->img, ix, iy, sdl->screen->format);
+				hh1 = provider->h1 + (ix - provider->hpx1) * provider->hmult;
+				ww1 = provider->w1 + (iy - provider->wpx1) * provider->wmult;
+				hw2km(sw->myh, sw->myw, hh1, ww1, &kx, &ky);
+				km2px(sw, kx, ky, &px1, &py1);
+        
+				if (!z_overlapped_pixel(aarea, px1, py1)) continue;
+				z_putpixel(sw->screen, px1, py1, c);
+			}
+		}
+	}else{
+		// not debug
+		SDL_SetClipRect(sw->screen, aarea); // not area
+
+		px2km(sw, area.x, area.y, &minkx, &minky);
+		px2km(sw, area.x + area.w - 1, area.y + area.h - 1, &maxkx, &maxky);
+    
+		minkx &= provider->mask;
+		minky &= provider->mask;
+		maxkx &= provider->mask;
+		maxky &= provider->mask;
+		if (minkx < -32768) minkx = -32768;
+		if (maxkx > 32767) maxkx = 32767;
+		if (minky < -32768) minky = -32768;
+		if (maxky > 32767) maxky = 32767;
+
+		c = provider->pal[ci];
+		for (ky = minky; ky <= maxky; ky += provider->step){
+			for (kx = minkx; kx <= maxkx; kx += provider->step){
+
+				gpointer key = provider->hash_func(kx, ky, ci);
+				GHashTable *hash = provider->hashes[ci];
+				struct zbinbuf *zbb = (struct zbinbuf *)g_hash_table_lookup(hash, key);
+				//dbg("kx=%d   \tky=%d\n", kx, ky);
+				if (zbb == NULL) continue;
+
+				for (i = 0; i <= zbb->len - (int)sizeof(struct rain_km); i += sizeof(struct rain_km)){
+					struct rain_km *km = (struct rain_km *)(zbb->buf + i);
+					km2px_d(sw, km->kx1, km->ky1, &px1, &py1);
+					km2px_d(sw, km->kx2, km->ky2, &px2, &py2);
+					km2px_d(sw, km->kx3, km->ky3, &px3, &py3);
+					km2px_d(sw, km->kx4, km->ky4, &px4, &py4);
+
+					z_triangle(sw->screen, px1, py1, px3, py3, px2, py2, c);
+					z_triangle(sw->screen, px3, py3, px2, py2, px4, py4, c);
+				}
+			}
+		}
+
+		if (ci == 0){
+			// borders
+			for (iy = provider->top; iy <= provider->img->h && iy <= provider->bottom + 1; iy++){
+				ww1 = provider->w1 + (iy - provider->wpx1) * provider->wmult;
+
+				hh1 = provider->h1 + (provider->left - provider->hpx1) * provider->hmult;
+				hh2 = provider->h1 + (provider->right + 1 - provider->hpx1) * provider->hmult;
+
+				hw2km(sw->myh, sw->myw, hh1, ww1, &kx, &ky);
+				km2px(sw, kx, ky, &px1, &py1);
+
+				hw2km(sw->myh, sw->myw, hh2, ww1, &kx, &ky);
+				km2px(sw, kx, ky, &px2, &py2);
+
+				if (iy > provider->top){
+					z_line(sw->screen, px3, py3, px1, py1, provider->pal[2]);
+					z_line(sw->screen, px4, py4, px2, py2, provider->pal[2]);
+				}
+				px3 = px1;
+				py3 = py1;
+				px4 = px2;
+				py4 = py2;
+			} 
+
+			for (ix = provider->left; ix <= provider->img->w && ix <= provider->right + 1; ix++){
+				hh1 = provider->h1 + (ix - provider->hpx1) * provider->hmult;
+
+				ww1 = provider->w1 + (provider->top - provider->wpx1) * provider->wmult;
+				ww2 = provider->w1 + (provider->bottom + 1 - provider->wpx1) * provider->wmult;
+
+				hw2km(sw->myh, sw->myw, hh1, ww1, &kx, &ky);
+				km2px(sw, kx, ky, &px1, &py1);
+
+				hw2km(sw->myh, sw->myw, hh1, ww2, &kx, &ky);
+				km2px(sw, kx, ky, &px2, &py2);
+
+				if (ix > provider->left){
+					z_line(sw->screen, px3, py3, px1, py1, provider->pal[2]);
+					z_line(sw->screen, px4, py4, px2, py2, provider->pal[2]);
+				}
+				px3 = px1;
+				py3 = py1;
+				px4 = px2;
+				py4 = py2;
+			}
+		}
+	}
+}
+
+void plot_rain_title(SDL_Surface *surface, SDL_Rect *area, int flags, struct rain_provider *provider, int *y){
+	int x = area->x + 2, c;
+
+	if (!provider || !provider->title) return;
+
+    if (provider->pal == provider->rain->colors)
+       c = sdl->gr[11];
+    else
+       c = z_makecol(252, 0, 0);
+
+	zsdl_printf(surface, x, *y, c, 0, ZFONT_TRANSP | ZFONT_OUTLINE, provider->title);
+	*y -= zsdl->font_h;
+}
+
+void plot_rain_titles(SDL_Surface *surface, SDL_Rect *area, int flags){
+	int y = area->y + area->h - zsdl->font_h - 2;
+	if (!grain) return;
+
+    plot_rain_title(surface, area, flags, grain->chmi, &y);
+    plot_rain_title(surface, area, flags, grain->wetteronline, &y);
+    plot_rain_title(surface, area, flags, grain->meteox, &y);
+}
+
 void sw_map_redraw(struct subwin *sw, struct band *band, int flags){
     struct qso *odx;
 	if (!sdl || !sw->screen) return;
@@ -473,6 +633,7 @@ void hw2km(double h1, double w1, double h2, double w2, int *kx, int *ky){
     *kx = (int)(  qrb*sin(qtf));
     *ky = (int)(- qrb*cos(qtf));
 }
+
 #endif
 
 void hw2km_d(double h1, double w1, double h2, double w2, double *kx, double *ky){
@@ -485,6 +646,19 @@ void hw2km_d(double h1, double w1, double h2, double w2, double *kx, double *ky)
     *ky = - qrb*cos(qtf);
 }
 
+void hw2km_f(double h1, double w1, double h2, double w2, float *fkx, float *fky){
+	double kx, ky;
+	hw2km_d(h1, w1, h2, w2, &kx, &ky);
+
+	/*if (ikx < -32768) ikx = -32768;
+	if (ikx > 32767) ikx = 32767;
+	if (iky < -32768) iky = -32768;
+	if (iky > 32767) iky = 32767;*/
+
+	*fkx = kx;
+	*fky = ky;
+}
+
 void km2qrbqtf(int kx, int ky, double *qrb, double *qtf){
     int dx, dy;
     
@@ -668,7 +842,7 @@ void plot_qth(struct subwin *sw, SDL_Rect *area){
 	r.h = sdl->mast->h;
 	/*z_line(sw->l1map, px - 100, py, px + 100, py, sdl->gr[15]);
 	z_line(sw->l1map, px, py - 100, px, py + 100, sdl->gr[15]);*/
-	SDL_BlitSurface(sdl->mast, NULL, sw->l1map, &r);
+	SDL_BlitSurface(sdl->mast, NULL, sw->screen, &r);
 }
 
 
