@@ -15,6 +15,7 @@
 #include "qsodb.h"
 struct qso;
 struct band;
+struct rain_provider;
 
 #define MAP_AC_INFO 1
 
@@ -31,6 +32,9 @@ void sw_map_redraw_rect(struct subwin *sw, SDL_Rect *area, struct band *band, in
 void sw_map_compose(struct subwin *sw, SDL_Rect *area);
 void sw_map_redraw_rotar(struct subwin *sw, SDL_Rect *area, int flags);
 void sw_map_redraw_icons(struct subwin *sw, SDL_Rect *area, int flags);
+void sw_map_redraw_rain(struct subwin *sw, SDL_Rect *area, int flags);
+void plot_rain_provider(struct subwin *sw, SDL_Rect *aarea, struct rain_provider *provider, int ci);
+void plot_rain_titles(SDL_Surface *surface, SDL_Rect *area, int flags);
 void sw_map_update_rotar(struct subwin *sw);
 void km2px(struct subwin *sw, int kx, int ky, int *px, int *py);
 void km2px_d(struct subwin *sw, double kx, double ky, int *px, int *py);
@@ -38,6 +42,7 @@ void px2km(struct subwin *sw, int px, int py, int *kx, int *ky);
 void pxkm2o(struct subwin *sw, int px, int py, int kx, int ky, int *ox, int *oy);
 void hw2km(double h1, double w1, double h2, double w2, int *kx, int *ky);
 void hw2km_d(double h1, double w1, double h2, double w2, double *kx, double *ky);
+void hw2km_f(double h1, double w1, double h2, double w2, float *kx, float *ky);
 
 void km2qrbqtf(int kx, int ky, double *qrb, double *qtf);
 
