@@ -648,6 +648,8 @@ void internal_test(void *a, void *b){
 
 static struct menu_item no_bands_qrv_menu[] = {
     {CTEXT(T_NO_BANDS_QRV), "", M_BAR, NULL, NULL, 0, 0},
+    {"", "", M_BAR, NULL, NULL, 0, 0},
+	{TRANSLATE("Set mode"), "M", "M", menu_set_mode, NULL, 0, 0},
     {NULL, NULL, 0, NULL, NULL, 0, 0}
 };
 
@@ -705,6 +707,7 @@ void menu_bands(void *arg)
 
     add_to_menu(&mi, g_strdup(""), "", M_BAR, NULL, NULL, 0);
     add_to_menu(&mi, g_strdup(VTEXT(T_SET_READONLY)), "S", "S", MENU_FUNC robands, NULL, 0);
+    add_to_menu(&mi, g_strdup(TRANSLATE("Set mode")), "M", "M", menu_set_mode, NULL, 0);
     do_menu_selected(mi, NULL, sel);
 }
 
@@ -1614,6 +1617,7 @@ static struct menu_item setup_menu[] = {
 #if defined(HAVE_ALSA) || defined (HAVE_PORTAUDIO)
     {CTEXT(T_SDR_OPTIONS)/*CTEXT(T_TERMINAL_OPTIONS)*/, "", CTEXT(T_HK_SDR_OPTIONS)/*CTEXT(T_HK_TERMINAL_OPTIONS)*/, MENU_FUNC menu_sdr_opts, NULL, 0, 0},
 #endif
+    {CTEXT(T_RAIN_OPTIONS), "", CTEXT(T_HK_RAIN_OPTIONS), menu_rain_opts, NULL, 0, 0},
     {CTEXT(T_LANGUAGE), ">", CTEXT(T_HK_LANGUAGE), MENU_FUNC menu_language_list, NULL, 1, 0},
     //{CTEXT(T_CHARACTER_SET), ">", CTEXT(T_HK_CHARACTER_SET), MENU_FUNC charset_list, GINT_TO_POINTER(1), 1, 0},
     {"", "", M_BAR, NULL, NULL, 0, 0},
