@@ -661,7 +661,7 @@ void menu_responsible_op(void *arg){
 
 /******************** QSOP METHOD, TOTAL METHOD  ******************/
 
-#define MAX_QSOP_METHOD 18
+#define MAX_QSOP_METHOD 19
 #define MAX_TOTAL_METHOD 4
 
 int qsop_method=1, total_method=1;
@@ -687,6 +687,7 @@ char *qsop_method_msg[]={
     CTEXT(T_QP16),
     CTEXT(T_QP17),
     CTEXT(T_QP18),
+    CTEXT(T_QP19),
 };
 
 char *total_method_msg[]={
@@ -772,7 +773,7 @@ void tttype_func(void *arg){
     log_addf("tttype_func: active=%d", active);
     if (active<0 || active>=MAX_TTTYPE) return;
     tttype  = active;
-    safe_strncpy0(tttype_str, tttype_msg[tttype], MAX_STR_LEN);
+    safe_strncpy0(tttype_str, get_text_translation(tttype_msg[tttype]), MAX_STR_LEN);
     redraw_later();
 }
 
@@ -832,7 +833,7 @@ int dlg_wwltype(struct dialog_data *dlg, struct dialog_item_data *di){
 
 /****************** EXCTYPE *******************/
 
-#define MAX_EXCTYPE 5
+#define MAX_EXCTYPE 6
 
 char *exctype_msg[]={
     CTEXT(T_UNUSED),
@@ -840,6 +841,7 @@ char *exctype_msg[]={
     CTEXT(T_VERIFIED),
     CTEXT(T_WAZ),
     CTEXT(T_ITU),
+    CTEXT(T_MULTIPLIED)
 };
 
 void exctype_func(void *arg){
@@ -865,16 +867,15 @@ void exctype_func(void *arg){
 }
 
 int dlg_exctype(struct dialog_data *dlg, struct dialog_item_data *di){
-    int i, sel;
+    int i, sel = 0;
     struct menu_item *mi;
+    int x[] = { 0, 1, 5, 2, 3, 4,  -1 };
     
     if (!(mi = new_menu(1))) return 0;
-    for (i = 0; i < MAX_EXCTYPE; i++) {
-        add_to_menu(&mi, exctype_msg[i], "", "", MENU_FUNC exctype_func, GINT_TO_POINTER(i), 0);
+    for (i = 0; x[i] >= 0; i++){
+        add_to_menu(&mi, exctype_msg[x[i]], "", "", MENU_FUNC exctype_func, GINT_TO_POINTER(x[i]), 0);
+        if (x[i] == exctype) sel = i;
     }
-    sel = exctype;
-    if (sel < 0) sel = 1;
-    if (sel >= MAX_EXCTYPE) sel=0;
     do_menu_selected(mi, GINT_TO_POINTER(exctype), sel);
     return 0;
 }                
@@ -931,7 +932,7 @@ void wizz_func (void *arg){
     safe_strncpy(default_rst, cfg->default_rst, 5);  z_str_uc(default_rst);
     defrstr=wi->defrstr;
     tttype=wi->tttype;
-    safe_strncpy0(tttype_str,tttype_msg[tttype],MAX_STR_LEN);
+    safe_strncpy0(tttype_str, get_text_translation(tttype_msg[tttype]), MAX_STR_LEN);
     expmode=0;
 
     
@@ -1127,6 +1128,7 @@ void init_ctest2(int from_dlg, int updloc, struct zstring *zs2, struct wizz_item
         
         recalc_all_stats(ctest);
         if (updloc){
+			update_hw();
             qrv_recalc_qrbqtf(qrv);
             qrv_recalc_gst(qrv);
 #ifdef Z_HAVE_SDL
@@ -1618,7 +1620,7 @@ void contest_options1_from_menu(void *arg)
     safe_strncpy0(total_method_str,_(CTEXT(T_TP0+total_method)),MAX_STR_LEN);
     /*safe_strncpy0(total_method_str,total_method_msg[total_method],MAX_STR_LEN);*/
     tttype=TT_RSTS;
-    safe_strncpy0(tttype_str,tttype_msg[tttype],MAX_STR_LEN);
+    safe_strncpy0(tttype_str, get_text_translation(tttype_msg[tttype]), MAX_STR_LEN);
     expmode=0;
    
     contest_options1(VTEXT(T_NEW_CTEST), 0, NULL);
@@ -1666,13 +1668,10 @@ void contest_options1_from_ctest(void *arg)
     safe_strncpy0(total_method_str,_(CTEXT(T_TP0+total_method)),MAX_STR_LEN);
     defrstr=ctest->defrstr;
     tttype=ctest->tttype;
-    safe_strncpy0(tttype_str,tttype_msg[tttype],MAX_STR_LEN);
+    safe_strncpy0(tttype_str,get_text_translation(tttype_msg[tttype]),MAX_STR_LEN);
     expmode=ctest->expmode;
     
     contest_options1(VTEXT(T_CONTEST_OP), 1, NULL);
-/*#ifdef Z_HAVE_SDL 
-    maps_reload();
-#endif  */
 }
                                
 
@@ -1722,6 +1721,8 @@ void refresh_contest_def(void *xxx)
 	}
 
 
+	update_hw();
+
 #ifdef Z_HAVE_SDL
     if (updloc) maps_reload();
 #endif
