@@ -37,7 +37,7 @@ void mode_func (void *arg){
     active=GPOINTER_TO_INT(arg);
     if (active<0 || active>MAX_MODES) return;
     mode = active;
-    safe_strncpy0(mode_str,mode_msg[mode],EQSO_LEN);
+    safe_strncpy0(mode_str, get_text_translation(mode_msg[mode]),EQSO_LEN);
     /*safe_strncpy0(mode_str,_(CTEXT(T_QP0+qsop_method),term),MAX_STR_LEN);*/
     redraw_later();
 }
@@ -297,7 +297,7 @@ void edit_qso(struct qso *qso)
     safe_strncpy0(exc,       qso->exc,       EQSO_LEN); 
     safe_strncpy0(locator,   qso->locator,   EQSO_LEN); z_str_uc(locator);
     mode=qso->mode;
-    safe_strncpy0(mode_str,  mode_msg[mode], EQSO_LEN);
+    safe_strncpy0(mode_str,  get_text_translation(mode_msg[mode]), EQSO_LEN);
     safe_strncpy0(rsts,      qso->rsts,      EQSO_LEN);
     safe_strncpy0(qsonrs,    qso->qsonrs,    EQSO_LEN);
     safe_strncpy0(date_str,  qso->date_str,  EQSO_LEN); 
