@@ -1033,7 +1033,7 @@ void do_unfinished(struct session *ses, char *remark){
             
     g_string_sprintfa(gs, "%5s%4s %5s%4s",
         TMPQ.rsts?TMPQ.rsts:"---", 
-        TMPQ.qsonrs, 
+        TMPQ.qsonrs?TMPQ.qsonrs: "---",
         TMPQ.rstr?TMPQ.rstr:"---", 
         TMPQ.qsonrr?TMPQ.qsonrr:"---");
 
