@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -10,12 +10,26 @@
 
 #include "header.h"
 
+#include "alsa.h"
+#include "bfu.h"
+#include "cwdaemon.h"
+#include "dsp.h"
+#include "menu.h"
+#include "net.h"
+#include "oss.h"
+#include "pa.h"
+#include "rc.h"
+#include "scope.h"
+#include "ssbd.h"
+#include "subwin.h"
+#include "terminal.h"
+#include "tsdl.h"
 
 char *keys[MAX_CQ]={"F5", "F6", "F7", "F8", "F11", "F12"};
 
 struct menu_item no_cq_defined_menu[] = {
-    {CTEXT(T_NO_CQ), "", M_BAR, NULL, {NULL}, 0, 0},
-    {NULL, NULL, 0, NULL, {NULL}, 0, 0},
+    {CTEXT(T_NO_CQ), "", M_BAR, NULL, NULL, 0, 0},
+    {NULL, NULL, 0, NULL, NULL, 0, 0},
 };
 
 
@@ -107,7 +121,7 @@ void cq_cw_fn(struct dialog_data *dlg)
     
 }
 
-void cq_cw(cba_t cba){
+void cq_cw(void *arg){
 
     struct dialog *d;
     int i;
@@ -115,7 +129,7 @@ void cq_cw(cba_t cba){
     struct cq *cq;
 
 
-    no = (int)GETCBA(cba, int_);
+    no = GPOINTER_TO_INT(arg);
     cq = g_ptr_array_index(cfg->cqs, no);
     
     safe_strncpy0(cw_str, cq->cw_str, MAX_STR_LEN);
@@ -125,14 +139,14 @@ void cq_cw(cba_t cba){
     cw_allowifundef = cq->cw_allowifundef;
 
     
-    if (!(d = mem_alloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
+    if (!(d = g_malloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 55 * sizeof(struct dialog_item));
     d->title = VTEXT(T_CW_CQ);
     d->fn = cq_cw_fn;
     d->refresh = (void (*)(void *))refresh_cq_cw;
     d->refresh_data = (void *)cq;
     
-    d->items[i=0].type = D_FIELD;     /* 0 */
+    d->items[i=0].type = D_FIELD;     
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = cw_str;
     d->items[i].maxl = 60;
@@ -164,7 +178,7 @@ void cq_cw(cba_t cba){
     d->items[i].data = (void *)&cw_allowifundef;
     
 
-    d->items[++i].type = D_BUTTON; /* 5 */
+    d->items[++i].type = D_BUTTON; 
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
     d->items[i].text = VTEXT(T_OK);
@@ -180,7 +194,7 @@ void cq_cw(cba_t cba){
                                
 }
 
-void menu_cq_cw(cba_t cba){
+void menu_cq_cw(void *arg){
     int i;
     char *c;
     struct menu_item *mi = NULL;
@@ -195,15 +209,15 @@ void menu_cq_cw(cba_t cba){
         c = g_strdup_printf(VTEXT(T_CW_DSCS), i, keys[i], cq->cw_repeat?'R':' ', example);
         if (!mi) if (!(mi = new_menu(3))) return;
         add_to_menu(&mi, 
-                stracpy(c), 
+                g_strdup(c), 
                 "", "", 
-                MENU_FUNC cq_cw, (cba_t)i, 0);    
+                MENU_FUNC cq_cw, GINT_TO_POINTER(i), 0);    
         g_free(c);
     }
     if (i) 
-        do_menu(mi, CBA0);
+        do_menu(mi, NULL);
     else
-        do_menu(no_cq_defined_menu, CBA0);
+        do_menu(no_cq_defined_menu, NULL);
 }
 
 
@@ -281,7 +295,7 @@ void cq_ssb_fn(struct dialog_data *dlg)
     
 }
 
-void cq_ssb(cba_t cba){
+void cq_ssb(void *arg){
 
     struct dialog *d;
     int i;
@@ -289,7 +303,7 @@ void cq_ssb(cba_t cba){
     struct cq *cq;
 
 
-    no = (int)GETCBA(cba, int_);            
+    no = GPOINTER_TO_INT(arg);            
     cq = g_ptr_array_index(cfg->cqs, no);
     
     safe_strncpy0(ssb_file, cq->ssb_file, MAX_STR_LEN);
@@ -297,14 +311,14 @@ void cq_ssb(cba_t cba){
     g_snprintf(ssb_ts_str, EQSO_LEN, "%d", cq->ssb_ts);
 
     
-    if (!(d = mem_alloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
+    if (!(d = g_malloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 55 * sizeof(struct dialog_item));
     d->title = VTEXT(T_SSB_CQ);
     d->fn = cq_ssb_fn;
     d->refresh = (void (*)(void *))refresh_cq_ssb;
     d->refresh_data = (void *)cq;
     
-    d->items[i=0].type = D_FIELD;     /* 0 */
+    d->items[i=0].type = D_FIELD;     
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = ssb_file;
     d->items[i].maxl = 60;
@@ -322,7 +336,7 @@ void cq_ssb(cba_t cba){
     d->items[i].gid  = 0;
     d->items[i].gnum = 300;
     
-    d->items[++i].type = D_BUTTON; /* 3 */
+    d->items[++i].type = D_BUTTON; 
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
     d->items[i].text = VTEXT(T_OK);
@@ -339,7 +353,7 @@ void cq_ssb(cba_t cba){
 }
 
 
-void menu_cq_ssb(cba_t cba){
+void menu_cq_ssb(void *arg){
     int i;
     char *c;
     struct menu_item *mi = NULL;
@@ -354,15 +368,15 @@ void menu_cq_ssb(cba_t cba){
         c = g_strdup_printf(VTEXT(T_SSB_DSCS), i, keys[i], cq->ssb_repeat?'R':' ', example);
         if (!mi) if (!(mi = new_menu(3))) return;
         add_to_menu(&mi, 
-                stracpy(c), 
+                g_strdup(c), 
                 "", "", 
-                MENU_FUNC cq_ssb, (cba_t)i, 0);    
+                MENU_FUNC cq_ssb, GINT_TO_POINTER(i), 0);    
         g_free(c);
     }
     if (i) 
-        do_menu(mi, CBA0);
+        do_menu(mi, NULL);
     else
-        do_menu(no_cq_defined_menu, CBA0);
+        do_menu(no_cq_defined_menu, NULL);
 }
 
 
@@ -398,10 +412,11 @@ void refresh_cwda(void *xxx){
 	
 	free_cwdaemon(cwda);
 	cwda=init_cwdaemon();
+	progress(NULL);
 }
 
 
-void menu_cwda(cba_t cba){
+void menu_cwda(void *arg){
 
     struct dialog *d;
     int i;
@@ -420,7 +435,7 @@ void menu_cwda(cba_t cba){
     g_snprintf(autgive_str, EQSO_LEN, "%d", cfg->cwda_autgive);
 
     
-    if (!(d = mem_alloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
+    if (!(d = g_malloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 55 * sizeof(struct dialog_item));
     d->title = VTEXT(T_CW_DAEMON);
 //  d->fn = cwda_fn;
@@ -429,32 +444,32 @@ void menu_cwda(cba_t cba){
     d->refresh_data = NULL;
     d->y0 = 1;
     
-    d->items[i=0].type = D_CHECKBOX;  /* 0 */
+    d->items[i=0].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = CWD_NONE;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&cwda_type;
-    d->items[i].msg = "None";
+    d->items[i].data = (char *)&cwda_type;
+    d->items[i].msg = CTEXT(T_ROT_NONE);
 	
     d->items[++i].type = D_CHECKBOX; 
     d->items[i].gid = 1;
     d->items[i].gnum = CWD_PARPORT;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&cwda_type;
-    d->items[i].msg = "parport";
+    d->items[i].data = (char *)&cwda_type;
+    d->items[i].msg = CTEXT(T_PARPORT);
 	
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = CWD_TTYS;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&cwda_type;
-    d->items[i].msg = "ttys";
+    d->items[i].data = (char *)&cwda_type;
+    d->items[i].msg = CTEXT(T_TTYS);
 	
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = CWD_DAVAC4;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&cwda_type;
+    d->items[i].data = (char *)&cwda_type;
     d->items[i].msg = "Davac4";
     d->items[i].wrap = 1;
 	
@@ -462,22 +477,22 @@ void menu_cwda(cba_t cba){
     d->items[i].gid = 1;
     d->items[i].gnum = CWD_CWD;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&cwda_type;
+    d->items[i].data = (char *)&cwda_type;
     d->items[i].msg = "cwdaemon";
 	
-    d->items[++i].type = D_CHECKBOX;    /* 5 */
+    d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = CWD_WINKEY;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&cwda_type;
+    d->items[i].data = (char *)&cwda_type;
     d->items[i].msg = "winkey";
 
-    d->items[++i].type = D_CHECKBOX;    /* 6 */
+    d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
-    d->items[i].gnum = CWD_WINKEY4;
+    d->items[i].gnum = CWD_WINKEYTCP;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&cwda_type;
-    d->items[i].msg = "winkey+davac4";
+    d->items[i].data = (char *)&cwda_type;
+    d->items[i].msg = "winkey TCP";
     d->items[i].wrap = 1;
 	
     d->items[++i].type = D_FIELD;       
@@ -538,12 +553,15 @@ void menu_cwda(cba_t cba){
     d->items[i].msg = CTEXT(T_WPMTO);
     d->items[i].wrap = 1;
     
+#ifdef Z_UNIX
     d->items[++i].type = D_CHECKBOX;  
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&cwda_spk;
+    d->items[i].data = (char *)&cwda_spk;
     d->items[i].msg = CTEXT(T_USE_SPK);
+    d->items[i].wrap = 1;
+#endif
     
     d->items[++i].type = D_FIELD;  
     d->items[i].dlen = EQSO_LEN;
@@ -599,6 +617,10 @@ int record_format; /* index 0..MAX_RECORD_FORMAT-1 */
 extern char ssbd_format_str[MAX_STR_LEN];
 extern char ssbd_oss_src[MAX_STR_LEN];
 extern char ssbd_alsa_src[MAX_STR_LEN];
+#ifdef HAVE_PORTAUDIO
+char ssbd_pa_play_src[MAX_STR_LEN];
+char ssbd_pa_rec_src[MAX_STR_LEN];
+#endif
 
 char *record_format_msg[]={
     CTEXT(T_PCM8),
@@ -620,10 +642,10 @@ int record_formats[]={
     SF_FORMAT_WAV | SF_FORMAT_GSM610
 };
 
-void record_format_func (cba_t cba){
+void record_format_func (void *arg){
     int active;
     
-    active=cba.int_;
+    active=GPOINTER_TO_INT(arg);
     if (active<0 || active>=MAX_RECORD_FORMAT) return;
     record_format = active;
     safe_strncpy0(ssbd_format_str,_(CTEXT(T_PCM8+record_format)),MAX_STR_LEN);
@@ -637,27 +659,27 @@ int dlg_record_format(struct dialog_data *dlg, struct dialog_item_data *di){
     
     if (!(mi = new_menu(1))) return 0;
     for (i = 0; i < MAX_RECORD_FORMAT; i++) {
-        add_to_menu(&mi, record_format_msg[i], "", "", MENU_FUNC record_format_func, (cba_t)i, 0);
+        add_to_menu(&mi, record_format_msg[i], "", "", MENU_FUNC record_format_func, GINT_TO_POINTER(i), 0);
     }
     sel = record_format;
     if (sel < 0) sel = 0;
     if (sel>=MAX_RECORD_FORMAT) sel=0;
-    do_menu_selected(mi, (cba_t)record_format, sel);
+    do_menu_selected(mi, GINT_TO_POINTER(record_format), sel);
     return 0;
 }
 
 /* ------------------------ */    
 
-void alsa_src_func (cba_t cba){
+void alsa_src_func (void *arg){
     int i, active;
     GString *labels;
     gchar **items;
     
-    active=cba.int_;
+    active=GPOINTER_TO_INT(arg);
 
     labels=g_string_sized_new(100);
 #ifdef HAVE_ALSA
-    alsa_get_sources(labels);
+    alsa_get_sources(gssbd->dsp, labels);
 #endif    
     items=g_strsplit(labels->str, ";", 0);
     for (i=0;items[i]!=NULL;i++);/* dbg("items[%d]=%s\n", i, items[i]);*/
@@ -686,19 +708,19 @@ int dlg_alsa_src(struct dialog_data *dlg, struct dialog_item_data *di){
 
 
 #ifdef HAVE_ALSA
-    alsa_get_sources(labels);
+    alsa_get_sources(gssbd->dsp, labels);
 #endif    
     items=g_strsplit(labels->str, ";", 0);
     dbg("dlg_alsa_src() labels='%s'\n", labels->str);
     if (!(mi = new_menu(1))) goto x;
-    add_to_menu(&mi, stracpy(VTEXT(T_RECSRC_NONE)), "", "", MENU_FUNC alsa_src_func, (cba_t)0, 0);
+    add_to_menu(&mi, g_strdup(VTEXT(T_RECSRC_NONE)), "", "", MENU_FUNC alsa_src_func, GINT_TO_POINTER(0), 0);
     sel=0;
     for (i = 0; items[i]!=NULL; i++) {
-        add_to_menu(&mi, stracpy(items[i]), "", "", MENU_FUNC alsa_src_func, (cba_t)(i+1), 0);
+        add_to_menu(&mi, g_strdup(items[i]), "", "", MENU_FUNC alsa_src_func, GINT_TO_POINTER(i+1), 0);
         if (strcmp(items[i], ssbd_alsa_src)==0) sel=i+1;
     }
     dbg("  sel=%d\n", sel);
-    do_menu_selected(mi, CBA0, sel);
+    do_menu_selected(mi, NULL, sel);
 x:;    
     g_string_free(labels, 1);
     g_strfreev(items);
@@ -707,16 +729,16 @@ x:;
     
 /* ------------------------ */    
 
-void oss_src_func (cba_t cba){
+void oss_src_func (void *arg){
     int i, active;
     GString *labels;
     gchar **items;
     
-    active=cba.int_;
+    active=GPOINTER_TO_INT(arg);
 
     labels=g_string_sized_new(100);
 #ifdef HAVE_OSS
-    oss_get_sources(labels);
+    oss_get_sources(gssbd->dsp, labels);
 #endif
     items=g_strsplit(labels->str, ";", 0);
     for (i=0;items[i]!=NULL;i++);/* dbg("items[%d]=%s\n", i, items[i]);*/
@@ -742,24 +764,86 @@ int dlg_oss_src(struct dialog_data *dlg, struct dialog_item_data *di){
     labels=g_string_sized_new(100);
 
 #ifdef HAVE_OSS
-    oss_get_sources(labels);
+    oss_get_sources(gssbd->dsp, labels);
 #endif
     items=g_strsplit(labels->str, ";", 0);
     dbg("labels='%s'\n", labels->str);
     if (!(mi = new_menu(1))) goto x;
-    add_to_menu(&mi, stracpy(VTEXT(T_RECSRC_NONE)), "", "", MENU_FUNC oss_src_func, (cba_t)0, 0);
+    add_to_menu(&mi, g_strdup(VTEXT(T_RECSRC_NONE)), "", "", MENU_FUNC oss_src_func, GINT_TO_POINTER(0), 0);
     sel=0;
     for (i = 0; items[i]!=NULL; i++) {
-        add_to_menu(&mi, stracpy(items[i]), "", "", MENU_FUNC oss_src_func, (cba_t)(i+1), 0);
+        add_to_menu(&mi, g_strdup(items[i]), "", "", MENU_FUNC oss_src_func, GINT_TO_POINTER(i+1), 0);
         if (strcmp(items[i], ssbd_oss_src)==0) sel=i+1;
     }
     dbg("sel=%d\n", sel);
-    do_menu_selected(mi, CBA0, sel);
+    do_menu_selected(mi, NULL, sel);
 x:;    
     g_string_free(labels, 1);
     g_strfreev(items);
     return 0;
 }
+
+/* ------------------------ */    
+
+#ifdef HAVE_PORTAUDIO
+static void pa_play_src_func (void *arg){
+    int active = GPOINTER_TO_INT(arg);
+
+    const PaDeviceInfo *di = Pa_GetDeviceInfo(active);
+    safe_strncpy0(ssbd_pa_play_src, di->name, MAX_STR_LEN);
+    cfg->ssbd_pa_play = active;
+    redraw_later();
+}
+
+static int dlg_pa_play_src(struct dialog_data *dlg, struct dialog_item_data *di){
+    int i, max, sel = 0, n;
+    struct menu_item *mi;
+
+    max = Pa_GetDeviceCount();
+    mi = new_menu(1);
+    n = 0;
+
+	for (i = 0; i < max; i++){
+    	const PaDeviceInfo *di = Pa_GetDeviceInfo(i);
+		if (di->maxOutputChannels <= 0) continue;
+        add_to_menu(&mi, g_strdup_printf("%d: %s", i, di->name), "", "", MENU_FUNC pa_play_src_func, GINT_TO_POINTER(i), 0);
+		if (i == cfg->ssbd_pa_play) sel = n;
+        n++;
+    }
+    do_menu_selected(mi, NULL, sel);
+	return 0;
+}
+    
+/* ------------------------ */    
+
+static void pa_rec_src_func(void *arg){
+    int active = GPOINTER_TO_INT(arg);
+
+    const PaDeviceInfo *di = Pa_GetDeviceInfo(active);
+    safe_strncpy0(ssbd_pa_rec_src, di->name, MAX_STR_LEN);
+    cfg->ssbd_pa_rec = active;
+    redraw_later();
+}
+
+static int dlg_pa_rec_src(struct dialog_data *dlg, struct dialog_item_data *di){
+    int i, max, sel = 0, n;
+    struct menu_item *mi;
+
+    max = Pa_GetDeviceCount();
+    mi = new_menu(1);
+    n = 0;
+
+	for (i = 0; i < max; i++){
+    	const PaDeviceInfo *di = Pa_GetDeviceInfo(i);
+		if (di->maxInputChannels <= 0) continue;
+        add_to_menu(&mi, g_strdup_printf("%d: %s", i, di->name), "", "", MENU_FUNC pa_rec_src_func, GINT_TO_POINTER(i), 0);
+		if (i == cfg->ssbd_pa_rec) sel = n;
+        n++;
+    }
+    do_menu_selected(mi, NULL, sel);
+	return 0;
+}
+#endif
     
 /* ------------------------ */    
 int ssbd_type;
@@ -772,7 +856,7 @@ char ssbd_plev_str[EQSO_LEN], ssbd_rlev_str[EQSO_LEN];
 char ssbd_template[MAX_STR_LEN];
 
 char ssbd_pcm_play[EQSO_LEN], ssbd_pcm_rec[EQSO_LEN];
-char ssbd_period_time_str[EQSO_LEN], ssbd_buffer_time_str[EQSO_LEN];
+char ssbd_period_time_str[EQSO_LEN]/*, ssbd_buffer_time_str[EQSO_LEN]*/;
 char ssbd_alsa_mixer[EQSO_LEN];
 char ssbd_alsa_src[MAX_STR_LEN];
     
@@ -804,7 +888,7 @@ void refresh_ssbd(void *xxx){
     STORE_STR(cfg, ssbd_pcm_play);
     STORE_STR(cfg, ssbd_pcm_rec);
     STORE_SINT(cfg, ssbd_period_time);
-    STORE_SINT(cfg, ssbd_buffer_time);
+    //STORE_SINT(cfg, ssbd_buffer_time);
     STORE_STR(cfg, ssbd_alsa_mixer);
     if (strcmp(ssbd_alsa_src, VTEXT(T_RECSRC_NONE))==0){
         strcpy(ssbd_alsa_src, "");
@@ -826,135 +910,10 @@ void refresh_ssbd(void *xxx){
     STORE_STR(cfg, ssbd_hostname);
     STORE_SINT(cfg, ssbd_udp_port);
 
-    free_ssbd(ssbd);
-    free_dsp(dsp);
-    dsp=init_dsp();
-    ssbd=init_ssbd();
-}
-
-char *ssbd_msg[] = {
-    CTEXT(T_ALSA),          /* 0 */
-    CTEXT(T_OSS),
-    CTEXT(T_SSBD),
-    CTEXT(T_RECORD), 
-    CTEXT(T_MAXMIN), 
-    CTEXT(T_DISKFREE),/* 5 */
-    CTEXT(T_FORMAT),
-    CTEXT(T_CHANNELS),      
-    CTEXT(T_SAMPLERATE),
-    "Playback volume",
-    "Capture volume",   /* 10 */
-    CTEXT(T_TEMPLATE),  
-
-    CTEXT(T_PCM_PLAY),  
-    CTEXT(T_PCM_REC),
-    CTEXT(T_PERIOD),         
-    CTEXT(T_BUFFER),       /* 15 */
-    CTEXT(T_ALSA_MIXER),
-    CTEXT(T_CAPTURE),      
-
-    CTEXT(T_DSP), 
-    CTEXT(T_MAXFRAG),   
-    CTEXT(T_MIXER),         /* 20 */  
-    CTEXT(T_RECSRC),
-    
-    CTEXT(T_SSBD_HOSTNAME), 
-    CTEXT(T_UDPPORT),
-
-    "", /* OK */       /* 24 */ 
-    "", /* Cancel */
-};
-
-    /*CTEXT(T_PCM8),      
-    CTEXT(T_PCM16), 
-    CTEXT(T_ULAW), 
-    CTEXT(T_ALAW), 
-    CTEXT(T_IMAADPCM),  
-    CTEXT(T_MSADPCM),   
-    CTEXT(T_GSM610), 
-    CTEXT(T_RECSRC_NONE),   
-    CTEXT(T_RECSRC_MIC),   
-    CTEXT(T_RECSRC_LINE), */  
-
-void ssbd_fn(struct dialog_data *dlg)
-{
-    struct terminal *term = dlg->win->term;
-    int max = 0, min = 0;
-    int w, rw;
-    int y = -1;
-
-    max_group_width(term, ssbd_msg + 0, dlg->items + 0, 3, &max);
-    min_group_width(term, ssbd_msg + 0, dlg->items + 0, 3, &min);
-    max_group_width(term, ssbd_msg + 3, dlg->items + 3, 4, &max);
-    min_group_width(term, ssbd_msg + 3, dlg->items + 3, 4, &min);
-    max_group_width(term, ssbd_msg + 7, dlg->items + 7, 4, &max);
-    min_group_width(term, ssbd_msg + 7, dlg->items + 7, 4, &min);
-    max_group_width(term, ssbd_msg +11, dlg->items +11, 1, &max);
-    min_group_width(term, ssbd_msg +11, dlg->items +11, 1, &min);
-
-    max_group_width(term, ssbd_msg +12, dlg->items +12, 4, &max);
-    min_group_width(term, ssbd_msg +12, dlg->items +12, 4, &min);
-    max_group_width(term, ssbd_msg +16, dlg->items +16, 2, &max);
-    min_group_width(term, ssbd_msg +16, dlg->items +16, 2, &min);
-    
-    max_group_width(term, ssbd_msg +18, dlg->items +18, 2, &max);
-    min_group_width(term, ssbd_msg +18, dlg->items +18, 2, &min);
-    max_group_width(term, ssbd_msg +20, dlg->items +20, 2, &max);
-    min_group_width(term, ssbd_msg +20, dlg->items +20, 2, &min);
-    
-    max_group_width(term, ssbd_msg +22, dlg->items +22, 2, &max);
-    min_group_width(term, ssbd_msg +22, dlg->items +22, 2, &min);
-
-    max_buttons_width(term, dlg->items +24, 2, &max);
-    min_buttons_width(term, dlg->items +24, 2, &min);
-    
-    w = dlg->win->term->x * 9 / 10 - 2 * DIALOG_LB;
-    if (w > max) w = max;
-    if (w < min) w = min;
-    if (w > dlg->win->term->x - 2 * DIALOG_LB - 8 ) w = dlg->win->term->x - 2 * DIALOG_LB - 8;
-    if (w < 1) w = 1;
-    
-    rw = 0;
-    y ++;
-    dlg_format_group1(NULL, term, ssbd_msg + 0, dlg->items + 0, 3, 0, &y, w, &rw);
-    dlg_format_group1(NULL, term, ssbd_msg + 3, dlg->items + 3, 4, 0, &y, w, &rw);
-    dlg_format_group1(NULL, term, ssbd_msg + 7, dlg->items + 7, 4, 0, &y, w, &rw);
-    dlg_format_group1(NULL, term, ssbd_msg +11, dlg->items +11, 1, 0, &y, w, &rw);
-    y++;
-    dlg_format_group1(NULL, term, ssbd_msg +12, dlg->items +12, 4, 0, &y, w, &rw);
-    dlg_format_group1(NULL, term, ssbd_msg +16, dlg->items +16, 2, 0, &y, w, &rw);
-    y++;
-    dlg_format_group1(NULL, term, ssbd_msg +18, dlg->items +18, 2, 0, &y, w, &rw);
-    dlg_format_group1(NULL, term, ssbd_msg +20, dlg->items +20, 2, 0, &y, w, &rw);
-    y++;
-    dlg_format_group1(NULL, term, ssbd_msg +22, dlg->items +22, 2, 0, &y, w, &rw);
-    y++;
-    dlg_format_buttons(NULL, term, dlg->items +24, 2, 0, &y, w, &rw, AL_LEFT);
-    
-    
-    w = rw;
-    dlg->xw = w + 2 * DIALOG_LB;
-    dlg->yw = y + 2 * DIALOG_TB;
-
-    
-    center_dlg(dlg);
-    draw_dlg(dlg);
-    y = dlg->y + DIALOG_TB;
-    y++;
-    dlg_format_group1(term, term, ssbd_msg + 0, dlg->items + 0, 3, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group1(term, term, ssbd_msg + 3, dlg->items + 3, 4, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group1(term, term, ssbd_msg + 7, dlg->items + 7, 4, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group1(term, term, ssbd_msg +11, dlg->items +11, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    y++;
-    dlg_format_group1(term, term, ssbd_msg +12, dlg->items +12, 4, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group1(term, term, ssbd_msg +16, dlg->items +16, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    y++;
-    dlg_format_group1(term, term, ssbd_msg +18, dlg->items +18, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group1(term, term, ssbd_msg +20, dlg->items +20, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    y++;
-    dlg_format_group1(term, term, ssbd_msg +22, dlg->items +22, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    y++;
-    dlg_format_buttons(term, term, dlg->items + 24, 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
+    free_ssbd(gssbd);
+    sw_scope_clear_scopes();
+    gssbd=init_ssbd();
+	progress(NULL);
 }
 
 void do_ssbd(void){
@@ -962,7 +921,16 @@ void do_ssbd(void){
     int i;
     GString *labels;
     gchar *c;
-    
+
+#ifdef HAVE_PORTAUDIO
+    const PaDeviceInfo *di;
+
+	if (!pa_initialised) {
+		Pa_Initialize();
+		pa_initialised = 1;
+	}
+#endif
+
     labels=g_string_sized_new(100);
 
     ssbd_type = cfg->ssbd_type;
@@ -983,7 +951,7 @@ void do_ssbd(void){
     safe_strncpy0(ssbd_pcm_play, cfg->ssbd_pcm_play, EQSO_LEN);
     safe_strncpy0(ssbd_pcm_rec,  cfg->ssbd_pcm_rec,  EQSO_LEN);
     g_snprintf(ssbd_period_time_str, EQSO_LEN, "%d", cfg->ssbd_period_time);
-    g_snprintf(ssbd_buffer_time_str, EQSO_LEN, "%d", cfg->ssbd_buffer_time);
+    //g_snprintf(ssbd_buffer_time_str, EQSO_LEN, "%d", cfg->ssbd_buffer_time);
     safe_strncpy0(ssbd_alsa_mixer, cfg->ssbd_alsa_mixer, EQSO_LEN);
 /*    safe_strncpy0(ssbd_alsa_src,   cfg->ssbd_alsa_src, MAX_STR_LEN);*/
     
@@ -1015,6 +983,22 @@ void do_ssbd(void){
     }else{
         safe_strncpy0(ssbd_oss_src, VTEXT(T_RECSRC_NONE), MAX_STR_LEN);
     }
+
+#ifdef HAVE_PORTAUDIO
+	if (cfg->ssbd_pa_play < 0) cfg->ssbd_pa_play = Pa_GetDefaultOutputDevice();
+    di = Pa_GetDeviceInfo(cfg->ssbd_pa_play);
+    if (di)
+        safe_strncpy(ssbd_pa_play_src, di->name, MAX_STR_LEN);
+    else
+        safe_strncpy0(ssbd_pa_play_src, VTEXT(T_RECSRC_NONE), MAX_STR_LEN);
+    
+	if (cfg->ssbd_pa_rec < 0) cfg->ssbd_pa_rec = Pa_GetDefaultInputDevice();
+    di = Pa_GetDeviceInfo(cfg->ssbd_pa_rec);
+    if (di)
+        safe_strncpy(ssbd_pa_rec_src, di->name, MAX_STR_LEN);
+    else
+        safe_strncpy0(ssbd_pa_rec_src, VTEXT(T_RECSRC_NONE), MAX_STR_LEN);
+#endif
     
 
     safe_strncpy0(ssbd_hostname, cfg->ssbd_hostname, MAX_STR_LEN);
@@ -1022,35 +1006,59 @@ void do_ssbd(void){
     
     g_string_free(labels, 0);
 
-    if (!(d = mem_alloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
+    if (!(d = (struct dialog *)g_malloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 55 * sizeof(struct dialog_item));
     d->title = VTEXT(T_AUDIO);
-    d->fn = ssbd_fn;
+    d->fn = dlg_pf_fn;
     d->refresh = (void (*)(void *))refresh_ssbd;
+    d->y0 = 1;
     
-    d->items[i=0].type = D_CHECKBOX;  /* 0 */
+	d->items[i = 0].type = D_TEXT; 
+	d->items[i].msg = CTEXT(T_AUDIO_SUBSYSTEM);
+#ifdef HAVE_ALSA
+    d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = DSPT_ALSA;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&ssbd_type;
+    d->items[i].data = (char *)&ssbd_type;
+    d->items[i].msg = CTEXT(T_ALSA);
+#endif
     
+#ifdef HAVE_OSS
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = DSPT_OSS;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&ssbd_type;
-    
+    d->items[i].data = (char *)&ssbd_type;
+    d->items[i].msg = CTEXT(T_OSS);
+#endif
+
+#ifdef HAVE_PORTAUDIO
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
-    d->items[i].gnum = DSPT_SSBD;
+    d->items[i].gnum = DSPT_PORTAUDIO;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&ssbd_type;
+    d->items[i].data = (char *)&ssbd_type;
+    d->items[i].msg = CTEXT(T_PORTAUDIO);
+#endif
+	
+#ifdef USE_SDR
+	d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+	d->items[i].gnum = DSPT_SNDPIPE;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char *)&ssbd_type;
+    d->items[i].msg = CTEXT(T_SDR);
+#endif
+
+	d->items[i].wrap = 1;
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&ssbd_record;
+    d->items[i].data = (char *)&ssbd_record;
+    d->items[i].msg = CTEXT(T_RECORD);
     
     d->items[++i].type = D_FIELD;     
     d->items[i].dlen = EQSO_LEN;
@@ -1059,19 +1067,23 @@ void do_ssbd(void){
     d->items[i].fn   = check_number;
     d->items[i].gid  = 0;
     d->items[i].gnum = 10000;
+    d->items[i].msg = CTEXT(T_MAXMIN);
 
-    d->items[++i].type = D_FIELD;      /* 5 */
+    d->items[++i].type = D_FIELD;     
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_diskfree_str;
     d->items[i].maxl = 6;
     d->items[i].fn   = check_number;
     d->items[i].gid  = 0;
     d->items[i].gnum = 10000;
+    d->items[i].msg = CTEXT(T_DISKFREE);
  
     d->items[++i].type = D_BUTTON;     
     d->items[i].gid  = 0;
     d->items[i].fn   = dlg_record_format;
     d->items[i].text = ssbd_format_str;
+    d->items[i].msg = CTEXT(T_FORMAT); 
+	d->items[i].wrap = 1;
  
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
@@ -1080,15 +1092,27 @@ void do_ssbd(void){
     d->items[i].fn   = check_number;
     d->items[i].gid  = 1;
     d->items[i].gnum = 8;
+    d->items[i].msg = CTEXT(T_CHANNELS); 
     
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_samplerate_str;
-    d->items[i].maxl = 6;
+    d->items[i].maxl = 7;
     d->items[i].fn   = check_number;
     d->items[i].gid  = 4000;
-    d->items[i].gnum = 180000;
+    d->items[i].gnum = 192000;
+    d->items[i].msg = CTEXT(T_SAMPLERATE); 
+
+	d->items[++i].type = D_FIELD;
+    d->items[i].dlen = EQSO_LEN;
+    d->items[i].data = ssbd_period_time_str;
+    d->items[i].maxl = 5;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 0;
+    d->items[i].gnum = 1000;
+    d->items[i].msg = CTEXT(T_PERIOD); 
     
+#if defined(HAVE_SYS_SOUNDCARD_H) || defined(HAVE_ALSA)
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_plev_str;
@@ -1096,60 +1120,72 @@ void do_ssbd(void){
     d->items[i].fn   = check_number;
     d->items[i].gid  = -1;
     d->items[i].gnum = 100;
+    d->items[i].msg = CTEXT(T_PLAYBACK_VOLUME);
     
-    d->items[++i].type = D_FIELD;    /* 10 */
+    d->items[++i].type = D_FIELD;    
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_rlev_str;
     d->items[i].maxl = 4;
     d->items[i].fn   = check_number;
     d->items[i].gid  = -1;
     d->items[i].gnum = 100;
+    d->items[i].msg = CTEXT(T_CAPTURE_VOLUME);
+#endif
+	d->items[i].wrap = 1;
     
     d->items[++i].type = D_FIELD;  
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = ssbd_template;
     d->items[i].maxl = 30;
+    d->items[i].msg = CTEXT(T_TEMPLATE);
+	d->items[i].wrap = 2;
     
+#ifdef HAVE_ALSA
     d->items[++i].type = D_FIELD;    
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_pcm_play;
     d->items[i].maxl = 15;
+    d->items[i].msg = CTEXT(T_PCM_PLAY);
     
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_pcm_rec;
     d->items[i].maxl = 15;
+    d->items[i].msg = CTEXT(T_PCM_REC);
+	d->items[i].wrap = 1;
     
-    d->items[++i].type = D_FIELD;
-    d->items[i].dlen = EQSO_LEN;
-    d->items[i].data = ssbd_period_time_str;
-    d->items[i].maxl = 5;
-    d->items[i].fn   = check_number;
-    d->items[i].gid  = 0;
-    d->items[i].gnum = 1000;
+
     
-    d->items[++i].type = D_FIELD;   /* 15 */   
+    /*d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_buffer_time_str;
     d->items[i].maxl = 5;
     d->items[i].fn   = check_number;
     d->items[i].gid  = 0;
     d->items[i].gnum = 1000;
+    d->items[i].msg = CTEXT(T_BUFFER);
+	d->items[i].wrap = 1;*/
     
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = ssbd_alsa_mixer;
     d->items[i].maxl = 15;
+    d->items[i].msg = CTEXT(T_ALSA_MIXER);
     
     d->items[++i].type = D_BUTTON;  
     d->items[i].gid  = 0;
     d->items[i].fn   = dlg_alsa_src;
     d->items[i].text = ssbd_alsa_src;
+    d->items[i].msg = CTEXT(T_CAPTURE);
+	d->items[i].wrap = 2;
+#endif
 
+#ifdef HAVE_SYS_SOUNDCARD_H
     d->items[++i].type = D_FIELD;     
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = ssbd_dsp;
     d->items[i].maxl = 20;
+    d->items[i].msg = CTEXT(T_DSP);
     
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
@@ -1158,31 +1194,42 @@ void do_ssbd(void){
     d->items[i].fn   = check_number;
     d->items[i].gid  = 0;
     d->items[i].gnum = 20;
+    d->items[i].msg = CTEXT(T_MAXFRAG);
+	d->items[i].wrap = 1;
     
-    d->items[++i].type = D_FIELD;    /* 20 */     
+    d->items[++i].type = D_FIELD;        
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = ssbd_mixer;
     d->items[i].maxl = 20;
+    d->items[i].msg = CTEXT(T_MIXER);
     
     d->items[++i].type = D_BUTTON; 
     d->items[i].gid  = 0;
     d->items[i].fn   = dlg_oss_src;
     d->items[i].text = ssbd_oss_src;
+    d->items[i].msg = CTEXT(T_RECSRC);
+    d->items[i].wrap = 2;
+#endif
 
-    d->items[++i].type = D_FIELD;    
-    d->items[i].dlen = MAX_STR_LEN;
-    d->items[i].data = ssbd_hostname;
-    d->items[i].maxl = 20;
+
+#ifdef HAVE_PORTAUDIO
+    d->items[++i].type = D_BUTTON;  
+    d->items[i].gid  = 0;
+    d->items[i].fn   = dlg_pa_play_src;
+    d->items[i].text = ssbd_pa_play_src;
+    d->items[i].msg = CTEXT(T_PA_PLAY);
+    d->items[i].wrap = 1;
     
-    d->items[++i].type = D_FIELD;
-    d->items[i].dlen = EQSO_LEN;
-    d->items[i].data = ssbd_udp_port_str;
-    d->items[i].maxl = 6;
-    d->items[i].fn   = check_number;
-    d->items[i].gid  = 1;
-    d->items[i].gnum = 65535;
+    d->items[++i].type = D_BUTTON;  
+    d->items[i].gid  = 0;
+    d->items[i].fn   = dlg_pa_rec_src;
+    d->items[i].text = ssbd_pa_rec_src;
+    d->items[i].msg = CTEXT(T_PA_REC);
+    d->items[i].wrap = 2;
+#endif
     
-    d->items[++i].type = D_BUTTON; /* 24 */
+    
+    d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
     d->items[i].text = VTEXT(T_OK);
@@ -1191,6 +1238,8 @@ void do_ssbd(void){
     d->items[i].gid = B_ESC;
     d->items[i].fn = cancel_dialog;
     d->items[i].text = VTEXT(T_CANCEL);
+    d->items[i].align = AL_BUTTONS;
+    d->items[i].wrap = 2;
     
     
     d->items[++i].type = D_END;
@@ -1198,7 +1247,7 @@ void do_ssbd(void){
 }
 
 
-void menu_ssbd(cba_t cba){
+void menu_ssbd(void *arg){
     
     do_ssbd();
 }
@@ -1208,21 +1257,45 @@ void menu_ssbd(cba_t cba){
 /*********************** NETWORK ****************************/
 
 char net_if_ignore[MAX_STR_LEN],net_ip_ignore[MAX_STR_LEN],net_ip_announce[MAX_STR_LEN];
-int trace_bcast, trace_sock, trace_recv, trace_send, trace_qsos, trace_sdev;
+int trace_bcast, trace_sock, trace_recv, trace_send, trace_qsos, trace_rig, trace_sdev, trace_keys;
+int net_remote_enable;
+char net_remote_host[MAX_STR_LEN], net_remote_port_str[EQSO_LEN], net_remote_pass[MAX_STR_LEN];
 
 void refresh_net(void *xxx){
+	int raise = 0;
+	if (trace_bcast && !cfg->trace_bcast) raise++;
+	if (trace_sock && !cfg->trace_sock) raise++;
+	if (trace_recv && !cfg->trace_recv) raise++;
+	if (trace_send && !cfg->trace_send) raise++;
+	if (trace_rig  && !cfg->trace_rig)  raise++;
+	if (trace_qsos && !cfg->trace_qsos) raise++;
+	if (trace_sdev && !cfg->trace_sdev) raise++;
+	if (trace_keys && !cfg->trace_keys) raise++;
     
     STORE_STR(cfg, net_if_ignore);
     STORE_STR(cfg, net_ip_ignore);
     STORE_STR(cfg, net_ip_announce);
+
+	STORE_INT(cfg, net_remote_enable);
+    STORE_STR(cfg, net_remote_host);
+    STORE_SINT(cfg, net_remote_port);
+    STORE_STR(cfg, net_remote_pass);
+
     STORE_INT(cfg, trace_bcast);
     STORE_INT(cfg, trace_sock);
     STORE_INT(cfg, trace_recv);
     STORE_INT(cfg, trace_send);
     STORE_INT(cfg, trace_qsos);
+    STORE_INT(cfg, trace_rig);
     STORE_INT(cfg, trace_sdev);
-    free_net_ifaces(net);
-    init_net_ifaces(net);
+    STORE_INT(cfg, trace_keys);
+    
+    free_net_ifaces(gnet);
+    init_net_ifaces(gnet, 1);
+	net_connect_remote(gnet);
+	if (raise){
+		sw_raise_or_new(SWT_LOG);
+	}
 }
 
 
@@ -1234,21 +1307,61 @@ void do_net(void){
     safe_strncpy0(net_if_ignore, cfg->net_if_ignore, MAX_STR_LEN);
     safe_strncpy0(net_ip_ignore, cfg->net_ip_ignore, MAX_STR_LEN);
     safe_strncpy0(net_ip_announce, cfg->net_ip_announce, MAX_STR_LEN);
+	net_remote_enable = cfg->net_remote_enable;
+    safe_strncpy0(net_remote_host, cfg->net_remote_host, MAX_STR_LEN);
+    g_snprintf(net_remote_port_str, EQSO_LEN, "%d", cfg->net_remote_port);
+    safe_strncpy0(net_remote_pass, cfg->net_remote_pass, MAX_STR_LEN);
     trace_bcast=cfg->trace_bcast;
     trace_sock=cfg->trace_sock;
     trace_recv=cfg->trace_recv;
     trace_send=cfg->trace_send;
     trace_qsos=cfg->trace_qsos;
+	trace_rig=cfg->trace_rig;
+	trace_sdev=cfg->trace_sdev;
+    trace_keys=cfg->trace_keys;
 
     
-    if (!(d = mem_alloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
+    if (!(d = (struct dialog *) g_malloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 55 * sizeof(struct dialog_item));
     d->title = VTEXT(T_NETWORK);
     d->fn = dlg_pf_fn;
     d->refresh = (void (*)(void *))refresh_net;
     d->y0 = 1;
+	i = -1;
     
-    d->items[i=0].type = D_FIELD;     /* 0 */
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 0;
+    d->items[i].gnum = 1;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char *)&net_remote_enable;
+    d->items[i].msg = CTEXT(T_ENABLE_REMOTE_CONNECT);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_FIELD;     
+    d->items[i].dlen = MAX_STR_LEN;
+    d->items[i].data = net_remote_host;
+    d->items[i].maxl = 24;
+    d->items[i].msg = CTEXT(T_REMOTE_HOST);
+    d->items[i].wrap = 1;
+
+    d->items[++i].type = D_FIELD;     
+    d->items[i].dlen = EQSO_LEN;
+    d->items[i].data = net_remote_port_str;
+    d->items[i].maxl = 24;
+	d->items[i].gid  = 0;
+	d->items[i].gnum = 65535;
+    d->items[i].msg = CTEXT(T_REMOTE_PORT);
+    d->items[i].wrap = 1;
+
+    d->items[++i].type = D_FIELD;     
+    d->items[i].dlen = MAX_STR_LEN;
+    d->items[i].data = net_remote_pass;
+    d->items[i].maxl = 24;
+    d->items[i].msg = CTEXT(T_REMOTE_PASS);
+    d->items[i].wrap = 2;
+
+   
+    d->items[++i].type = D_FIELD;     
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = net_if_ignore;
     d->items[i].maxl = 24;
@@ -1268,34 +1381,35 @@ void do_net(void){
     d->items[i].maxl = 24;
     d->items[i].msg = CTEXT(T_IP_ANNOUNCE);
     d->items[i].wrap = 2;
-    
+
+
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&trace_bcast;
+    d->items[i].data = (char *)&trace_bcast;
     d->items[i].msg = CTEXT(T_TRACE_BCAST);
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&trace_sock;
+    d->items[i].data = (char *)&trace_sock;
     d->items[i].msg = CTEXT(T_TRACE_SOCK);
     d->items[i].wrap = 1;
     
-    d->items[++i].type = D_CHECKBOX;   /* 5 */
+    d->items[++i].type = D_CHECKBOX;  
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&trace_recv;
+    d->items[i].data = (char *)&trace_recv;
     d->items[i].msg = CTEXT(T_TRACE_RECV);
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&trace_send;
+    d->items[i].data = (char *)&trace_send;
     d->items[i].msg = CTEXT(T_TRACE_SEND);
     d->items[i].wrap = 1;
     
@@ -1303,18 +1417,34 @@ void do_net(void){
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&trace_qsos;
+    d->items[i].data = (char *)&trace_qsos;
     d->items[i].msg = CTEXT(T_TRACE_QSOS);
     
+	d->items[++i].type = D_CHECKBOX3;
+    d->items[i].gid = 0;
+    d->items[i].gnum = 1;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char *)&trace_rig;
+    d->items[i].msg = CTEXT(T_TRACE_RIG);//CTEXT(T_TRACE_SDEV);
+    d->items[i].wrap = 1;
+
+	d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 0;
+    d->items[i].gnum = 1;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char *)&trace_sdev;
+    d->items[i].msg = CTEXT(T_TRACE_SDEV);
+    d->items[i].wrap = 0;
+
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&trace_sdev;
-    d->items[i].msg = CTEXT(T_TRACE_SDEV);
+    d->items[i].data = (char *)&trace_keys;
+    d->items[i].msg = CTEXT(T_TRACE_KEYS);//CTEXT(T_TRACE_SDEV);
     d->items[i].wrap = 2;
     
-    d->items[++i].type = D_BUTTON; /* 8 */
+	d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
     d->items[i].text = VTEXT(T_OK);
@@ -1333,7 +1463,7 @@ void do_net(void){
 }
 
 
-void menu_network(cba_t cba){
+void menu_network(void *arg){
     
     do_net();
 }
