@@ -27,7 +27,7 @@
 char loglines_str[MAX_STR_LEN],skedcount_str[MAX_STR_LEN],startbandchar_str[MAX_STR_LEN];
 char gfx_x_ch_str[MAX_STR_LEN],gfx_y_ch_str[MAX_STR_LEN];
 char gfx_x_px_str[MAX_STR_LEN],gfx_y_px_str[MAX_STR_LEN];
-int fullscreen, usetouch, altsyms, nolocks, portrait, reverse;
+int fullscreen, usetouch, altsyms, nolocks, portrait, reverse, inverse;
 char fontheight_str[EQSO_LEN];
 char slashkey[3];
 int dssaver;
@@ -65,6 +65,8 @@ void refresh_misc_opts(void *xxx)
 	    STORE_INT(cfg, portrait);
 	    STORE_INT(cfg, reverse);
     }
+
+	STORE_INT(cfg, inverse);
     
     new_w=800;
     new_h=600;
@@ -101,6 +103,9 @@ void refresh_misc_opts(void *xxx)
                 resize++;
             }
         }
+		if (inverse != zsdl->inverse){
+			resize++;
+		}
 
         if (resize && new_w <= 2048 && new_h <= 2048) {
             cfg->gfx_x = new_w;
@@ -160,6 +165,7 @@ void misc_opts(void *arg)
 	nolocks = cfg->nolocks;
 	portrait = cfg->portrait;
 	reverse = cfg->reverse;
+	inverse = cfg->inverse;
 
 
     d = g_new0(struct dialog, 20);
@@ -303,7 +309,15 @@ void misc_opts(void *arg)
 	d->items[i].msg = CTEXT(T_REVERSE);
 #endif
 	d->items[i].wrap = 1;
-    
+
+	d->items[++i].type = D_CHECKBOX;
+	d->items[i].gid  = 0;
+	d->items[i].gnum = 1;
+	d->items[i].dlen = sizeof(int);
+	d->items[i].data = (char *)&inverse;
+	d->items[i].msg = TRANSLATE("Inverse display");
+	d->items[i].wrap = 1;
+
 
 	d->items[i].wrap++;
     d->items[++i].type = D_BUTTON;   
@@ -477,7 +491,7 @@ void winkey_opts(void *arg)
     wk_swap = cfg->wk_swap;
 
 
-    if (!(d = g_malloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item)))) return;
+    if (!(d = (struct dialog *)g_malloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 20 * sizeof(struct dialog_item));
     d->title = VTEXT(T_WINKEYOPTS); 
     d->fn = winkey_opts_fn;
@@ -560,6 +574,7 @@ char rig_filename[MAX_STR_LEN];
 char rig_model_str[EQSO_LEN];
 int rig_model;
 char rig_speed_str[EQSO_LEN];
+int rig_handshake_none;
 /* choose manufacturer */
 char rig_civaddr_str[EQSO_LEN];
 char rig_ssbcw_shift_str[EQSO_LEN];
@@ -567,8 +582,17 @@ char rig_lo_str[MAX_STR_LEN];
 //int rig_poll_ms;
 char rig_poll_ms_str[MAX_STR_LEN];
 int rig_qrg_r2t, rig_qrg_t2r, rig_mode_t2r, rig_clr_rit, rig_ptt_t2r, rig_verbose;
+char rig_ptt_t2r_str[EQSO_LEN];
 
-char new_model_str[EQSO_LEN];
+int new_model = 0;
+char new_model_name[EQSO_LEN];
+
+char *rig_ptt_types[] = {
+    "Unused", 
+    "PTT", 
+    "PTT MIC",
+    "PTT DATA"
+};
 
 void set_rig (void *arg){
     struct rig_caps *mcaps;
@@ -579,8 +603,9 @@ void set_rig (void *arg){
     mcaps = (struct rig_caps *)z_ptr_array_index(riglist, active);
     
     dbg("%4d %4d %-20s %s\n", active, mcaps->rig_model, mcaps->mfg_name, mcaps->model_name);
-    g_snprintf(new_model_str, EQSO_LEN, "%d", mcaps->rig_model);
-    dbg("set_rig: new_model_str = '%s'\n", new_model_str);
+	new_model = mcaps->rig_model;
+    g_snprintf(new_model_name, EQSO_LEN, "%s", mcaps->model_name);
+    dbg("set_rig: new_model = %d (%s)\n", new_model, new_model_name);
     redraw_later();
 }
     
@@ -641,6 +666,23 @@ int choose_manufacturer(struct dialog_data *dlg, struct dialog_item_data *di){
     return 0;
 }
 
+void set_ptt_type(void *arg, void *arg2){
+    rig_ptt_t2r = GPOINTER_TO_INT(arg);
+    strcpy(rig_ptt_t2r_str, rig_ptt_types[rig_ptt_t2r]);
+    redraw_later();
+}
+
+int choose_ptt_type(struct dialog_data *dlg, struct dialog_item_data *di){
+    struct menu_item *mi;
+    int i;
+    
+    if (!(mi = new_menu(1))) return 0;
+
+    for (i = 0; i < 4; i++) add_to_menu(&mi, rig_ptt_types[i], "", "", set_ptt_type, GINT_TO_POINTER(i), 0);
+    do_menu_selected(mi, di->cdata, rig_ptt_t2r);
+    return 0;
+}
+
 void refresh_rig_opts(void *xxx){
     struct config_rig *crig = (struct config_rig *)xxx;
 
@@ -655,6 +697,7 @@ void refresh_rig_opts(void *xxx){
     STORE_STR(crig, rig_filename);
     STORE_SINT(crig, rig_model);
     STORE_SINT(crig, rig_speed);
+    STORE_INT(crig, rig_handshake_none);
 	STORE_SHEX(crig, rig_civaddr);
     STORE_SINT(crig, rig_ssbcw_shift);
     STORE_SDBL(crig, rig_lo);
@@ -667,16 +710,17 @@ void refresh_rig_opts(void *xxx){
     STORE_INT(crig, rig_verbose);
 
     free_trigs(gtrigs);
-    gtrigs = init_trigs();        
+    init_trigs();        
 	progress(NULL);
 }
 
 void rig_opts_fn(struct dialog_data *dlg)
 {
-    if (strlen(new_model_str)>0){
-        dbg("new_model_str=%s\n", new_model_str);
-        safe_strncpy0(dlg->items[3].cdata, new_model_str, EQSO_LEN);
-        strcpy(new_model_str, "");
+    if (new_model > 0){
+        dbg("new_model_str=%d (%s)\n", new_model, new_model_name);
+        if (strlen(dlg->items[1].cdata) == 0) safe_strncpy0(dlg->items[1].cdata, new_model_name, EQSO_LEN);
+		g_snprintf(dlg->items[3].cdata, EQSO_LEN, "%d", new_model);
+        new_model = 0;
     }
 
 }
@@ -694,10 +738,11 @@ void menu_rig_opts(void *arg)
     if (rigi < 0){
         crig = NULL;
 		rig_enabled = 1;
-		strcpy(rig_desc, VTEXT(T_NEW_RIG));
+		strcpy(rig_desc, ""/*VTEXT(T_NEW_RIG)*/);
         strcpy(rig_filename, "");       
         strcpy(rig_model_str, "1");         
         strcpy(rig_speed_str, "0");     
+		rig_handshake_none = 0;
         strcpy(rig_civaddr_str, "0x58");   
         strcpy(rig_ssbcw_shift_str, "0");
         strcpy(rig_lo_str, "0");        
@@ -716,6 +761,7 @@ void menu_rig_opts(void *arg)
         LOAD_STR(crig, rig_filename);
         LOAD_SINT(crig, rig_model);
         LOAD_SINT(crig, rig_speed);
+        LOAD_INT(crig, rig_handshake_none);
 		LOAD_SHEX2(crig, rig_civaddr);
         LOAD_SINT(crig, rig_ssbcw_shift);
         LOAD_SDBL(crig, rig_lo);
@@ -729,7 +775,8 @@ void menu_rig_opts(void *arg)
     }
 
 
-    strcpy(new_model_str, "");
+	new_model = 0;
+    strcpy(new_model_name, "");
     
 
     if (!(d = (struct dialog*)g_malloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item)))) return;
@@ -789,12 +836,19 @@ void menu_rig_opts(void *arg)
     d->items[i].gid  = 0;
     d->items[i].msg  = CTEXT(T_RIG_SPEED);
     d->items[i].gnum = 999999;
-    d->items[i].wrap = 1;
 
+	d->items[++i].type = D_CHECKBOX;      
+    d->items[i].gid = 0;
+    d->items[i].gnum = 1;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data=(char *)&rig_handshake_none;
+    d->items[i].msg = TRANSLATE("Force handshake NONE");
+    d->items[i].wrap = 1;
+		
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = rig_civaddr_str;
-    d->items[i].maxl = 4;
+    d->items[i].maxl = 5;
     d->items[i].fn   = check_hex;
     d->items[i].gid  = 0;
     d->items[i].msg  = CTEXT(T_RIG_CIV);
@@ -854,12 +908,13 @@ void menu_rig_opts(void *arg)
     d->items[i].msg = CTEXT(T_RIG_MODE_T2R);
     d->items[i].wrap = 1;
 
-    d->items[++i].type = D_CHECKBOX;      
+    strcpy(rig_ptt_t2r_str, rig_ptt_types[rig_ptt_t2r]);
+    d->items[++i].type = D_BUTTON;      
     d->items[i].gid = 0;
-    d->items[i].gnum = 1;
-    d->items[i].dlen = sizeof(int);
+    d->items[i].fn = choose_ptt_type;
     d->items[i].data=(char *)&rig_ptt_t2r;
-    d->items[i].msg = CTEXT(T_SET_PTT_ON_RADIO);//CTEXT(T_RIG_CLR_RIT);
+    d->items[i].msg = CTEXT(T_SET_PTT_ON_RADIO);
+    d->items[i].text = rig_ptt_t2r_str;
     d->items[i].wrap = 1;
 
     d->items[++i].type = D_CHECKBOX;      
@@ -907,8 +962,7 @@ void menu_rig_delete(void *arg){
     g_ptr_array_remove_index(cfg->crigs, rigi);
 	
     free_trigs(gtrigs);
-    gtrigs = init_trigs();        
-
+    init_trigs();        
 }
 
 
