@@ -278,6 +278,41 @@ void robands(void *arg){
     do_dialog(d, getml(d, NULL));
 }
 
+//-----------------------------------------------------------------------------
+void choose_mode(void *itdata, void *menudata)
+{
+    enum modes m = (enum modes)GPOINTER_TO_INT(itdata);
+    set_mode(m);
+}
+
+void menu_set_mode(void *itdata, void *menudata)
+{
+    struct menu_item *mi = NULL;
+	int sel = 0;
+
+    if (!(mi = new_menu(0))) return;
+
+	add_to_menu(&mi, CTEXT(T_SSB), "S", "S", MENU_FUNC choose_mode, GINT_TO_POINTER(MOD_SSB_SSB), 0);
+	if (gses->mode == MOD_SSB_SSB) sel = 0;
+
+	add_to_menu(&mi, CTEXT(T_CW), "C", "C", MENU_FUNC choose_mode, GINT_TO_POINTER(MOD_CW_CW), 0);
+	if (gses->mode == MOD_CW_CW) sel = 1;
+
+	add_to_menu(&mi, CTEXT(T_AM), "A", "A", MENU_FUNC choose_mode, GINT_TO_POINTER(MOD_AM_AM), 0);
+	if (gses->mode == MOD_AM_AM) sel = 2;
+
+	add_to_menu(&mi, CTEXT(T_FM), "F", "F", MENU_FUNC choose_mode, GINT_TO_POINTER(MOD_FM_FM), 0);
+	if (gses->mode == MOD_FM_FM) sel = 3;
+
+	add_to_menu(&mi, CTEXT(T_RTTY), "R", "R", MENU_FUNC choose_mode, GINT_TO_POINTER(MOD_RTTY_RTTY), 0);
+	if (gses->mode == MOD_RTTY_RTTY) sel = 4;
+
+    set_window_ptr(gses->win, (term->x - 6 - 6) / 2, (term->y - 2 - 5) / 2);
+
+	do_menu_selected(mi, NULL, sel);
+}
+
+
 //---- A/C --------------------------------------------------------------------
 
 int ac_enable;       
