@@ -50,9 +50,9 @@ int nCodePage = GetACP();
 	g_string_sprintfa(gs, "libsndfile: yes %s\n", sf_version_string());
 	g_string_sprintfa(gs, "      alsa: no\n");
 #ifdef HAVE_HAMLIB
-	/*hlver = strchr(hamlib_version, ' ');
-	if (hlver) g_string_sprintfa(gs, " libhamlib: yes %s\n", hlver + 1);  */
-	g_string_sprintfa(gs, " libhamlib: yes 1.2.15.3-zia\n");  
+	hlver = strchr(hamlib_version, ' ');
+	if (hlver) g_string_sprintfa(gs, " libhamlib: yes %s\n", hlver + 1);  
+	//g_string_sprintfa(gs, " libhamlib: yes 3.0.1-zia\n");  
 #else
 	g_string_sprintfa(gs, " libhamlib: no\n");
 #endif
