@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -10,63 +10,79 @@
 
 #include "header.h"
 
-#if defined(HAVE_OSS) && defined(HAVE_SNDFILE)
+#include "fifo.h"
+#include "language2.h"
+#include "oss.h"
 
+#ifdef HAVE_OSS
 
 static const char *clabels[] = SOUND_DEVICE_LABELS;
 
+static int oss_get_bufsize_frames(struct dsp *dsp){
+	double period_time = 0.0;
+	double wanted = 0.050;
+	int frames = 64;
+
+	if (dsp->period_time > 0) wanted = dsp->period_time / 1000.0;
+
+	while (period_time < wanted){
+		frames <<= 1;
+		period_time = (double)frames / (double)dsp->speed;
+	} 
+	return frames;
+}
+
+
+
 int oss_open(struct dsp *dsp, int rec){
     int tmp;
 	char errbuf[1024];
+	int maxfrag = 2;
 
-    CONDGFREE(dsp->filename);
-    dsp->filename=g_strdup(cfg->ssbd_dsp);
-
-    dbg("open_dsp('%s',%s)\n", dsp->filename, rec?"record":"playback");
-    CONDGFREE(dsp->name);
-    dsp->name=g_strdup(cfg->ssbd_dsp);
-    dsp->fd = open (cfg->ssbd_dsp, rec?O_RDONLY:O_WRONLY, 0);
+    
+    dbg("open_dsp('%s',%s)\n", dsp->oss_filename, rec?"record":"playback");
+    
+    zg_free0(dsp->name);
+    dsp->name = g_strdup(dsp->oss_filename);
+    
+    dsp->frames = oss_get_bufsize_frames(dsp);
+	dsp->samples = dsp->frames * dsp->channels;
+	dsp->bytes = dsp->samples * sizeof(short);
+    
+    dsp->fd = open (dsp->oss_filename, rec?O_RDONLY:O_WRONLY, 0);
     if (dsp->fd < 0){
         log_addf("Can't open file '%s': %s", dsp->name, strerror_r(errno, errbuf, sizeof(errbuf)) );
         goto err;
     }
 
-    if (cfg->ssbd_maxfrag>=2 && cfg->ssbd_maxfrag<16384){
-        dsp->fragment = (cfg->ssbd_maxfrag<<16)|0x000a;
-        dbg("setting fragment to 0x%08x\n", dsp->fragment);
-        if (ioctl (dsp->fd, SNDCTL_DSP_SETFRAGMENT, &dsp->fragment)){
-            log_addf("Can't set fragment size to 0x%x: %s", dsp->fragment, strerror_r(errno, errbuf, sizeof(errbuf)) );
-            goto err;
-        }
-    }   
+    int a = -1, b;
+    for (b = dsp->bytes; b > 0; b >>= 1) a++; 
+    if (a < 6) a = 6;
+    dbg("bufsize=%df %ds %db a=%d\n", dsp->frames, dsp->samples, dsp->bytes, a);
+    dsp->fragment = (maxfrag<<16)|a;//0x000a;
+    dbg("setting fragment to 0x%08x\n", dsp->fragment);
+    if (ioctl (dsp->fd, SNDCTL_DSP_SETFRAGMENT, &dsp->fragment)){
+        log_addf("Can't set fragment size to 0x%x: %s", dsp->fragment, strerror_r(errno, errbuf, sizeof(errbuf)) );
+        goto err;
+    }
         
     if (ioctl(dsp->fd, SNDCTL_DSP_GETFMTS, &dsp->fmt_mask)){
-        log_addf("Can't get supported audio formats: %s", strerror_r(errno, errbuf, sizeof(errbuf)) );
+        log_addf(VTEXT(T_CANT_GET_FMTS), strerror_r(errno, errbuf, sizeof(errbuf)) );
         goto err;
     }
     
     tmp=dsp->oss_format;
     if (ioctl (dsp->fd, SNDCTL_DSP_SETFMT, &dsp->oss_format)){
-        log_addf("Can't set audio format %d, recommended %d: %s", tmp, dsp->oss_format, strerror_r(errno, errbuf, sizeof(errbuf)) );
+        log_addf(VTEXT(T_CANT_SET_FMT), tmp, dsp->oss_format, strerror_r(errno, errbuf, sizeof(errbuf)) );
         
         tmp=dsp->oss_format;
         if (ioctl (dsp->fd, SNDCTL_DSP_SETFMT, &dsp->oss_format)){
-            log_addf("Can't set recommended audio format %d, return %d: %s", tmp, dsp->oss_format, strerror_r(errno, errbuf, sizeof(errbuf)) );
+            log_addf(VTEXT(T_CANT_SET_FMT_RET), tmp, dsp->oss_format, strerror_r(errno, errbuf, sizeof(errbuf)) );
             
             tmp = dsp->oss_format = AFMT_S16_NE;
             if (ioctl (dsp->fd, SNDCTL_DSP_SETFMT, &dsp->oss_format)){
-                log_addf("Can't set fallback audio %d, return %d: %s", tmp, dsp->oss_format, strerror_r(errno, errbuf, sizeof(errbuf)) );
-                
-                dsp->oss_format = AFMT_S8;
-                if (ioctl (dsp->fd, SNDCTL_DSP_SETFMT, &dsp->oss_format)){
-                    log_addf("Can't set fallback audio format %d, return %d: %s", tmp, dsp->oss_format, strerror_r(errno, errbuf, sizeof(errbuf)) );
-                    
-                    dsp->oss_format = AFMT_U8;
-                    if (ioctl (dsp->fd, SNDCTL_DSP_SETFMT, &dsp->oss_format)){
-                        log_addf("Can't set fallback audio format %d, return %d: %s", tmp, dsp->oss_format, strerror_r(errno, errbuf, sizeof(errbuf)) );
-                        goto err;
-                    }
-                }
+                log_addf(VTEXT(T_CANT_SET_FALLBACK), tmp, dsp->oss_format, strerror_r(errno, errbuf, sizeof(errbuf)) );
+                goto err;
             }
         }
         /* fallback ok */
@@ -75,11 +91,11 @@ int oss_open(struct dsp *dsp, int rec){
 
     tmp=dsp->channels;
     if (ioctl (dsp->fd, SNDCTL_DSP_CHANNELS, &dsp->channels)){
-        log_addf("Can't set channels %d, recommended %d: %s", tmp, dsp->channels, strerror_r(errno, errbuf, sizeof(errbuf)) );
+        log_addf(VTEXT(T_CANT_SET_CHANNELS), tmp, dsp->channels, strerror_r(errno, errbuf, sizeof(errbuf)) );
         
         tmp=dsp->channels;
         if (ioctl (dsp->fd, SNDCTL_DSP_CHANNELS, &dsp->channels)){
-            log_addf("Can't set recommended channels %d, return %d: %s", tmp, dsp->channels, strerror_r(errno, errbuf, sizeof(errbuf)) );
+            log_addf(VTEXT(T_CANT_SET_CHANNELS_RET), tmp, dsp->channels, strerror_r(errno, errbuf, sizeof(errbuf)) );
             goto err;
         }
         /* fallback ok */
@@ -87,20 +103,22 @@ int oss_open(struct dsp *dsp, int rec){
 
     tmp=dsp->speed;
     if (ioctl (dsp->fd, SNDCTL_DSP_SPEED, &dsp->speed)){
-        log_addf("Can't set samplerate %d, recommended %d: %s", tmp, dsp->speed, strerror_r(errno, errbuf, sizeof(errbuf)) );
+        log_addf(VTEXT(T_CANT_SET_RATE), tmp, dsp->speed, strerror_r(errno, errbuf, sizeof(errbuf)) );
         
         tmp=dsp->speed;
         if (ioctl (dsp->fd, SNDCTL_DSP_SPEED, &dsp->speed)){
-            log_addf("Can't set recommended samplerate %d, return %d: %s", tmp, dsp->speed, strerror_r(errno, errbuf, sizeof(errbuf)) );
+            log_addf(VTEXT(T_CANT_SET_RATE_RET), tmp, dsp->speed, strerror_r(errno, errbuf, sizeof(errbuf)) );
             goto err;
         }
         /* fallback ok */
     }
 
     if (ioctl (dsp->fd, SNDCTL_DSP_GETBLKSIZE, &dsp->blksize)){
-        log_addf("Can't set block size %d: %s", tmp, dsp->blksize, strerror_r(errno, errbuf, sizeof(errbuf)) );
+        log_addf(VTEXT(T_CANT_SET_BLOCK_SIZE), tmp, dsp->blksize, strerror_r(errno, errbuf, sizeof(errbuf)) );
         goto err;
     }   
+    //log_addf("OSS opened, rate=%d, bufsize=%df %ds %db", dsp->speed, dsp->frames, dsp->samples, dsp->bytes); 
+    dbg("OSS opened, rate=%d, bufsize=%df %ds %db\n", dsp->speed, dsp->frames, dsp->samples, dsp->bytes); 
     goto x;
     
 err:;
@@ -116,7 +134,6 @@ int oss_close(struct dsp *dsp){
     
     /*ST_START;*/
     dsp->reset(dsp);
-    CONDGFREE(dsp->filename);
     if (dsp->fd >= 0) {
         /*dbg("close_dsp(fd=%d)\n",dsp->fd);*/
         close(dsp->fd);
@@ -127,13 +144,19 @@ int oss_close(struct dsp *dsp){
 }
 
 
-int oss_write(struct dsp *dsp, void *data, int len){
-    return write(dsp->fd, data, len);    
+int oss_write(struct dsp *dsp, void *data, int frames){
+	int bpf = dsp->frames / dsp->bytes;
+    int ret = write(dsp->fd, data, frames * bpf);    
+	if (ret > 0) ret /= bpf;
+	return ret;
 }
 
 
-int oss_read(struct dsp *dsp, void *data, int len){
-    return read(dsp->fd, data, len);    
+int oss_read(struct dsp *dsp, void *data, int frames){
+	int bpf = dsp->frames / dsp->bytes;
+    int ret = read(dsp->fd, data, frames);    
+	if (ret > 0) ret /= bpf;
+	return ret;
 }
 
 
@@ -149,9 +172,9 @@ int oss_sync(struct dsp *dsp){
 }
 
 
-int oss_set_format(struct dsp *dsp, SF_INFO *sfinfo){
+#ifdef HAVE_SNDFILE
+int oss_set_format(struct dsp *dsp, SF_INFO *sfinfo, int ret){
 
-    dsp->bpf=1;
     switch (sfinfo->format & SF_FORMAT_SUBMASK){
 /*        case SF_FORMAT_PCM_U8:
             format=AFMT_U8;
@@ -170,23 +193,36 @@ int oss_set_format(struct dsp *dsp, SF_INFO *sfinfo){
             break;            
         default:
             dsp->oss_format=AFMT_S16_NE;    
-            dsp->bpf=2;
     }
     dsp->channels=sfinfo->channels; 
     if (dsp->channels>MAX_CHANNELS) dsp->channels=MAX_CHANNELS;
     dsp->speed=sfinfo->samplerate;   
     return 0;
 }    
+#endif
 
+static char *stripr(char *s){
+    int i;
 
-int oss_get_sources(GString *labels){
+    i=strlen(s);
+    if (!i) return s;
+
+    for (i--;i>=0;i--){
+        if (s[i]!=' ') break;
+        s[i]='\0';
+    }
+
+    return s;
+}
+
+int oss_get_sources(struct dsp *dsp, GString *labels){
 	int fd, i, recmask;
     char s[16];
 	
     g_string_truncate(labels, 0);
-    if (!cfg->ssbd_mixer || strlen(cfg->ssbd_mixer)==0) return 0;
+    if (!dsp || !dsp->oss_mixer || strlen(dsp->oss_mixer)==0) return 0;
 
-    fd=open(cfg->ssbd_mixer, O_RDWR, 0);
+    fd=open(dsp->oss_mixer, O_RDWR, 0);
 	if (fd<0){
 		return -1;
 	}
@@ -210,14 +246,16 @@ int oss_get_sources(GString *labels){
 int oss_set_source(struct dsp *dsp){
 	int fd, i, val;
     char s[16];
-    
-    dbg("oss_set_source('%s')\n", dsp->source);
-    if (!dsp->source || !*dsp->source){
+
+
+	   
+    dbg("oss_set_dsp->source('%s')\n", dsp->source);
+    if (!dsp->source || !*dsp->source || !dsp->oss_mixer || !*dsp->oss_mixer){
         return 0;
     }
 
 	
-    fd=open(cfg->ssbd_mixer, O_RDWR, 0);
+    fd=open(dsp->oss_mixer, O_RDWR, 0);
 	if (fd<0){
 		return -1;
 	}
@@ -233,8 +271,8 @@ int oss_set_source(struct dsp *dsp){
             return -1;
         }
 
-        if (cfg->ssbd_rlev < 0) continue;
-        val = (cfg->ssbd_rlev << 8 ) | cfg->ssbd_rlev;
+        if (dsp->rlev < 0) continue;
+        val = (dsp->rlev << 8 ) | dsp->rlev;
         if (ioctl(fd, MIXER_WRITE(i), &val) < 0){
             close(fd);
             return -3;
@@ -254,10 +292,11 @@ int oss_set_plevel(struct dsp *dsp){
     
     dbg("oss_set_plevel()\n");
 
-    if (cfg->ssbd_plev < 0) return 0;
+	if (!dsp->oss_mixer || !*dsp->oss_mixer) return 0;
+    if (dsp->plev < 0) return 0;
         
 	
-    fd=open(cfg->ssbd_mixer, O_RDWR, 0);
+    fd=open(dsp->oss_mixer, O_RDWR, 0);
 	if (fd<0){
 		return -1;
 	}
@@ -269,7 +308,7 @@ int oss_set_plevel(struct dsp *dsp){
         dbg("a=%s\n",s);
         if (strcasecmp(s, "Vol")!=0 && strcasecmp(s, "Pcm")!=0) continue;
 
-        val = (cfg->ssbd_plev << 8 ) | cfg->ssbd_plev;
+        val = (dsp->plev << 8 ) | dsp->plev;
         if (ioctl(fd, MIXER_WRITE(i), &val) < 0){
             close(fd);
             return -3;
@@ -294,17 +333,4 @@ char *oss_recsrc2source(int recsrc){
     return s;
 }
 
-char *stripr(char *s){
-    int i;
-
-    i=strlen(s);
-    if (!i) return s;
-
-    for (i--;i>=0;i--){
-        if (s[i]!=' ') break;
-        s[i]='\0';
-    }
-
-    return s;
-}
 #endif
