@@ -168,6 +168,7 @@ int new_ctest(char *tdate){
 
     
     set_ctest_title();
+	update_hw();
 #ifdef Z_HAVE_SDL
     maps_reload();
 #endif  
@@ -236,10 +237,6 @@ int load_ctest_from_mem(struct contest *ctest, gchar *datedir, GHashTable *hash)
     STORE_HASH_INT(ctest, expmode);
     
     set_ctest_title();
-//    dbg("defrstr=%d\n", ctest->defrstr);
-/*#ifdef Z_HAVE_SDL
-    maps_reload();
-#endif  */
    
     return 0; 
 }
@@ -1477,19 +1474,29 @@ void set_mode(enum modes mode){
 		if (!trig->mode_t2r) return;
 		switch (mode){
 			case MOD_SSB_SSB:
-				//trig_set_mode(trig, RIG_MODE_USB);
-				//trig_set_qrg(trig, trig->qrg - cfg->rig_ssbcw_shift);
 				trig_set_mode_qrg(trig, RIG_MODE_USB, gtrigs->qrg - trig->ssbcw_shift);
 				break;
 			case MOD_CW_CW:
-				//trig_set_mode(trig, RIG_MODE_CW);
-				//trig_set_qrg(trig, trig->qrg + cfg->rig_ssbcw_shift);
 				trig_set_mode_qrg(trig, RIG_MODE_CW, gtrigs->qrg + trig->ssbcw_shift);
 				break;
+			case MOD_AM_AM:
+				trig_set_mode_qrg(trig, RIG_MODE_AM, gtrigs->qrg);
+				break;
+			case MOD_FM_FM:
+				trig_set_mode_qrg(trig, RIG_MODE_FM, gtrigs->qrg);
+				break;
+			case MOD_RTTY_RTTY:
+				trig_set_mode_qrg(trig, RIG_MODE_RTTY, gtrigs->qrg);
+				break;
+			case MOD_SSTV_SSTV:
+				trig_set_mode_qrg(trig, RIG_MODE_USB, gtrigs->qrg); // SSTV uses SSB
+				break;
+			case MOD_ATV_ATV:
+				trig_set_mode_qrg(trig, RIG_MODE_FM, gtrigs->qrg); // SSTV uses FMB
+				break;
 			default:
 				/* nic jineho se nanastavuje */
 				break;
-                    
 		}
 	}
 #endif    
