@@ -240,7 +240,8 @@ enum exctype {
     EXC_FREE = 1,
     EXC_VERIFIED = 2,
     EXC_WAZ = 3,
-    EXC_ITU = 4
+    EXC_ITU = 4,
+    EXC_MULTIPLIED = 5
 };
 
 enum tttype{
