@@ -295,7 +295,13 @@ enum {
 	SYMBOL_WIKI_CHART        = G_TOKEN_LAST + 295,
 	//SYMBOL_WIKI_OVERWRITE    = G_TOKEN_LAST + 296,
 
-    SYMBOL_MY_LAST           = G_TOKEN_LAST + 297       
+    SYMBOL_RAIN_ENABLE       = G_TOKEN_LAST + 296,
+    SYMBOL_RAIN_METEOX       = G_TOKEN_LAST + 297,
+    SYMBOL_RAIN_WETTERONLINE = G_TOKEN_LAST + 298,
+    SYMBOL_RAIN_CHMI         = G_TOKEN_LAST + 299,
+    SYMBOL_RAIN_DEBUG        = G_TOKEN_LAST + 300,
+
+    SYMBOL_MY_LAST           = G_TOKEN_LAST + 302       
 };
 
 void add_symbols(GScanner *scanner){
@@ -562,6 +568,12 @@ void add_symbols(GScanner *scanner){
 	g_scanner_add_symbol(scanner,"wiki_map",    GINT_TO_POINTER(SYMBOL_WIKI_MAP));
 	g_scanner_add_symbol(scanner,"wiki_chart",  GINT_TO_POINTER(SYMBOL_WIKI_CHART));
 
+    g_scanner_add_symbol(scanner,"rain_enable",        GINT_TO_POINTER(SYMBOL_RAIN_ENABLE));
+    g_scanner_add_symbol(scanner,"rain_meteox",        GINT_TO_POINTER(SYMBOL_RAIN_METEOX));
+    g_scanner_add_symbol(scanner,"rain_wetteronline",  GINT_TO_POINTER(SYMBOL_RAIN_WETTERONLINE));
+    g_scanner_add_symbol(scanner,"rain_chmi",          GINT_TO_POINTER(SYMBOL_RAIN_CHMI));
+    g_scanner_add_symbol(scanner,"rain_debug",         GINT_TO_POINTER(SYMBOL_RAIN_DEBUG));
+
     
     /* obsolete items */
     g_scanner_add_symbol(scanner,"cwda_device",      GINT_TO_POINTER(SYMBOL_OBSOLETE));
@@ -1239,6 +1251,12 @@ guint read_one_token(GScanner *scanner){
 		STORE_RC_STRING(cfg, SYMBOL_WIKI_PASS, wiki_pass);
 		STORE_RC_INT   (cfg, SYMBOL_WIKI_MAP, wiki_map);
 		STORE_RC_INT   (cfg, SYMBOL_WIKI_CHART, wiki_chart);
+		
+        STORE_RC_INT   (cfg, SYMBOL_RAIN_ENABLE, rain_enable);
+        STORE_RC_INT   (cfg, SYMBOL_RAIN_METEOX, rain_meteox);
+        STORE_RC_INT   (cfg, SYMBOL_RAIN_WETTERONLINE, rain_wetteronline);
+        STORE_RC_INT   (cfg, SYMBOL_RAIN_CHMI, rain_chmi);
+        STORE_RC_INT   (cfg, SYMBOL_RAIN_DEBUG, rain_debug);
 
     }
     /* force logging */
@@ -1765,6 +1783,13 @@ void save_rc_string(GString *gs){
     g_string_sprintfa(gs, "wiki_chart = %d\n", cfg->wiki_chart);
     g_string_sprintfa(gs, "\n");
 
+	g_string_sprintfa(gs, "rain_enable = %d\n", cfg->rain_enable);
+	g_string_sprintfa(gs, "rain_meteox = %d\n", cfg->rain_meteox);
+	g_string_sprintfa(gs, "rain_wetteronline = %d\n", cfg->rain_wetteronline);
+	g_string_sprintfa(gs, "rain_chmi = %d\n", cfg->rain_chmi);
+	g_string_sprintfa(gs, "rain_debug = %d\n", cfg->rain_debug);
+    g_string_sprintfa(gs, "\n");
+
 
     for (i=0; i<cfg->cqs->len; i++){
         struct cq *cq;
