@@ -29,6 +29,7 @@ struct hdkeyb *hdkeyb;
 
 GPtrArray *rotars;
 MUTEX_DEFINE(rotars);
+
 struct rotar *rotar;
 
 #ifdef HAVE_HAMLIB
@@ -87,8 +88,13 @@ struct rotar *init_rotar(struct config_rotar *crot, int rotchar){
 				return NULL;
 			}
 			if (crot->rot_model > 0){
-				safe_strncpy(rot->rot->state.rotport.pathname, crot->rot_filename, FILPATHLEN);
-				//rot->rot->state.rotport.timeout = 2000;
+				if (strchr(crot->rot_filename, '\\') != NULL){
+					safe_strncpy(rot->rot->state.rotport.pathname, crot->rot_filename, FILPATHLEN);
+				}else{
+					char *c = g_strdup_printf("\\\\.\\%s", crot->rot_filename);
+					safe_strncpy(rot->rot->state.rotport.pathname, c, FILPATHLEN);
+					g_free(c);
+				}
 			}
 
             rot->hl_azim = -1;
@@ -408,7 +414,7 @@ void rotar_remote_update(char *c){
     struct zstring *zs;
     struct rotar *rot;
 
-    dbg("rotar_remote_update('%s')\n", c);
+    //dbg("rotar_remote_update('%s')\n", c);
     zs = zstrdup(c);
 
     netid = ztokenize(zs, 1);
@@ -421,7 +427,7 @@ void rotar_remote_update(char *c){
     if (!elev) goto x;
 
     rot = get_rotar_by_netid_rotstr(netid, rotstr);
-    dbg("rotar_remote_update id='%s' rotstr='%s'   rot=%p\n", netid, rotstr, rot);
+    //dbg("rotar_remote_update id='%s' rotstr='%s'   rot=%p\n", netid, rotstr, rot);
 
     if (!rot) goto x;
 
@@ -613,7 +619,6 @@ gpointer rot_hamlib_main(gpointer xxx){
 
     ret = rot_open(rot->rot);
     if (ret != RIG_OK){
-        
         return NULL;
     }
 
