@@ -193,15 +193,15 @@ void sw_scope_clear_scopes(){
     for (i = 0; i < gses->subwins->len; i++){
         struct subwin *sw = (struct subwin*)g_ptr_array_index(gses->subwins, i);
         if (sw->type != SWT_SCOPE) continue;
+	if (!sw->screen) continue;
         SDL_FillRect(sw->screen, NULL, z_makecol(0, 0, 0));
     }
 #ifdef USE_FFT
-	if (gfft && gfft->screen){
+	if (gfft && gfft->screen && sdl){
 	    MUTEX_LOCK(gfft->screen);
-        SDL_FillRect(gfft->screen, NULL, z_makecol(0, 0, 0));
+            SDL_FillRect(gfft->screen, NULL, z_makecol(0, 0, 0));
 	    MUTEX_UNLOCK(gfft->screen);
 	}
-
 #endif
 }
 
