@@ -186,8 +186,9 @@ struct sdr *init_sdr(void){
 	int i, j = 0, r, f;
     struct sdr *sdr;
    
+    dbg("init_sdr  enable=%d  sdl=%p\n", cfg->sdr_enable, sdl);
 	if (!cfg->sdr_enable) return NULL;
-    if (!sdl) return NULL;
+    //if (!sdl) return NULL;
 
     sdr = g_new0(struct sdr, 1);
 
@@ -198,38 +199,41 @@ struct sdr *init_sdr(void){
 	
 	sdr->bfo_phase = 0.0;
 	sdr->load = -1;
-	sdr->butbg = z_makecol(0, 128, 128);
-	
+    if (zsdl){
+    	sdr->butbg = z_makecol(0, 128, 128);
+    }
 
     MUTEX_INIT(sdr->screen);
 	MUTEX_INIT(sdr->filter);
 
-	for (i = 128; i < 256; i++){
-		r = (i-128)*2;
-		f = 255 - r;
-		sdr->pal[j++] = z_makecol(0, 0, r);
-	} 
-	for (i = 256; i < 512; i++){
-		r = i - 256;
-		f = 255 - r;
-		sdr->pal[j++] = z_makecol(0, r, 255);
-	}
-	for (i = 512; i < 768; i++){
-		r = i - 512;
-		f = 255 - r;
-		sdr->pal[j++] = z_makecol(r, 255, f);
-	}
-	for (i = 768; i < 1024-32; i++){
-		r = i - 768;
-		f = 255 - r;
-		sdr->pal[j++] = z_makecol(255, f, 0);
-	}
+    if (sdl){
+        for (i = 128; i < 256; i++){
+            r = (i-128)*2;
+            f = 255 - r;
+            sdr->pal[j++] = z_makecol(0, 0, r);
+        } 
+        for (i = 256; i < 512; i++){
+            r = i - 256;
+            f = 255 - r;
+            sdr->pal[j++] = z_makecol(0, r, 255);
+        }
+        for (i = 512; i < 768; i++){
+            r = i - 512;
+            f = 255 - r;
+            sdr->pal[j++] = z_makecol(r, 255, f);
+        }
+        for (i = 768; i < 1024-32; i++){
+            r = i - 768;
+            f = 255 - r;
+            sdr->pal[j++] = z_makecol(255, f, 0);
+        }
 
 	
-	for (i = 0; i < FFT_COLORS; i++){
-		int val = (z_r(sdl->screen, sdr->pal[i]) * 30 + z_g(sdl->screen, sdr->pal[i]) * 59 + z_b(sdl->screen, sdr->pal[i]) * 11) / 100;
-		sdr->grpal[i] = z_makecol(val, val, val);
-	}
+    	for (i = 0; i < FFT_COLORS; i++){
+	        int val = (z_r(sdl->screen, sdr->pal[i]) * 30 + z_g(sdl->screen, sdr->pal[i]) * 59 + z_b(sdl->screen, sdr->pal[i]) * 11) / 100;
+		    sdr->grpal[i] = z_makecol(val, val, val);
+	    }
+    }
 
 	// initial filter, rewritten after source DSP open
 	sdr->samplerate = cfg->sdr_speed ? cfg->sdr_speed : 48000;
@@ -529,7 +533,7 @@ void sdr_resize(struct sdr *sdr, struct subwin *sw){
 	SDL_Rect r;
 	int minh;
 
-	if (!sdr/* || !sdr->screen to je asi pycovina */) return;
+	if (!sdr) return;
     minh = sw->h * FONT_H / 2;
     sw->fft_sp_h = 210;
     if (sw->fft_sp_h > minh) sw->fft_sp_h = minh;
@@ -792,16 +796,18 @@ void sw_sdr_redraw(struct subwin *sw, struct band *band, int flags){
 gpointer sdr_thread_func(gpointer data){
     int total = 0, i;
     struct sdr *sdr = (struct sdr *)data;
-	struct timeval tv1, tv2, stop;
+	struct timeval tv1, tv2, stop, tvstart;
 	long oldsec = 0;
 
-	double totsum = 0.0, cpusum = 0.0;
+	double totsum = 0.0, cpusum = 0.0, tot;
 	int loadn = 0;
 	short *buf, *b;
 	double *pd, avgi = 0.0, avgq = 0.0;
 	int playspeed, playfactor;
 	int use_pipe = 0;
 
+    dbg("sdr_thread_func\n");
+
 	zg_thread_set_name("Tucnak SDR");
 	zg_thread_set_priority(2);
 
@@ -844,6 +850,7 @@ gpointer sdr_thread_func(gpointer data){
 	buf = g_new0(short, sdr->frames * sdr->iqdsp->channels); // nem bt sdr->samples?
 
 	memset(sdr->iq, 0, sizeof(fftw_complex) * sdr->n);
+	gettimeofday(&tvstart, NULL);
 	gettimeofday(&tv1, NULL);
 	oldsec = tv1.tv_sec;
      
@@ -857,9 +864,10 @@ gpointer sdr_thread_func(gpointer data){
 
     while(!sdr->thread_break){
         int oldx, x, col, frames;
-        double d, e, tot, cpu, dmax, maxgain, mingain;
+        double d, e, cpu, dmax, maxgain, mingain;
 		short *sb;
 		fftw_complex *iq;
+        //dbg("sdr_thread_func while\n");
 
 		memmove(sdr->iq, sdr->iq + sdr->frames, sizeof(fftw_complex) * (sdr->n - sdr->frames)); 
 
@@ -867,6 +875,7 @@ gpointer sdr_thread_func(gpointer data){
 		frames = sdr->iqdsp->read(sdr->iqdsp, buf, sdr->frames);
 		//ST_STOP("dsp read");
 		if (frames != sdr->frames){
+            dbg("Can't read from SDR IQDSP %s (%d != %d)\n", sdr->iqdsp->name, frames, sdr->frames);
 			zselect_msg_send(zsel, "SDR;!;%s SDR IQDSP %s (%d != %d)", VTEXT(T_CANT_READ_FROM), sdr->iqdsp->name, frames, sdr->frames);
             break;
         }
@@ -897,71 +906,66 @@ gpointer sdr_thread_func(gpointer data){
 		fftw_execute(sdr->plan_f);
     
         MUTEX_LOCK(sdr->screen);
-        if (!sdr->screen) {    // no SDR window exists
-            MUTEX_UNLOCK(sdr->screen);
-            continue; 
+        if (sdr->screen){
+            oldx = -1;
+            dmax = 0.0;
+            maxgain = -1000000000000.0;
+            mingain = 100000000000000.0;
+            memset(sdr->wfvec, 0, sdr->wfveclen * sizeof(int));
+            
+            for (i = 0; i < sdr->n; i++){
+                int j = (i + sdr->n / 2) % sdr->n;
+                x = (i * (sdr->screen->w - 1)) / sdr->n;
+                if (x >= sdr->screen->w) break;
+                                
+                d = sqrt(sqr(sdr->bins[j][0])+sqr(sdr->bins[j][1]));
+                //d = sqr(sdr->bins[j][0]) + sqr(sdr->bins[j][1]);
+                d = d/sdr->n;
+                d /= M_SQRT2; 
+                if (d > dmax) dmax = d;
+                // d is epsilon to 1.0
+                e = log10(d);
+                e -= log10(1.0/130072.0);
+                e /= -log10(1.0/130072.0);
+                //e -= log10(1.0/32768.0);
+                //e /= -log10(1.0/32768.0);
+                if (e<0) e = 0;  // slabsi signaly nez 1 bit
+                if (i < sdr->amplen) {
+                    double gain = (1 - exp(-(0.2 * e)));
+                    if (maxgain < gain) maxgain = gain;
+                    if (mingain > gain) mingain = gain;
+                    if (gain < 0.03) gain = 0.03;
+                    if (i == 0) gain = 1.0;
+
+                    sdr->amp[i] = e;
+                    sdr->nea_amp[i] = sdr->nea_amp[i] * (1-gain) + e * gain;
+                }
+                col = (int)(e * (FFT_COLORS - 1.0));
+                //col = (int)(sdr->nea_amp[i] * (FFT_COLORS - 1.0));
+                if (oldx > x) if (col > sdr->wfvec[x]) sdr->wfvec[x] = col;
+            
+                for (; oldx <= x; oldx++){
+                    if (oldx >= sdr->wfveclen) break;
+                    if (col > sdr->wfvec[oldx]) sdr->wfvec[oldx] = col;
+                }
+
+                    //z_putpixel(sdr->screen, oldx, sdr->screeny, sdr->pal[col]);
+            }   
+            for (i = 0; i < sdr->wfveclen; i++) z_putpixel(sdr->screen, i, sdr->screeny, sdr->pal[sdr->wfvec[i]]);
+            
+            //dbg("maxgain=%f  mingain=%f\n", maxgain, mingain);
+            if (tv2.tv_sec != oldsec){ // draw time marks
+                int x2 = 3 + (tv2.tv_sec % 10 == 0) * 4;
+                oldsec = tv2.tv_sec;
+                for (i = 0; i <= x2; i++) {
+                    z_putpixel(sdr->screen, i, sdr->screeny, sdl->gr[14]);
+                    z_putpixel(sdr->screen, sdr->screen->w - 1 - i, sdr->screeny, sdl->gr[14]);
+                }
+            }
+
+            sdr->screeny++;
+            if (sdr->screeny == sdr->screen->h) sdr->screeny = 0;
         }
-		
-
-        oldx = -1;
-		dmax = 0.0;
-		maxgain = -1000000000000.0;
-		mingain = 100000000000000.0;
-		memset(sdr->wfvec, 0, sdr->wfveclen * sizeof(int));
-		
-
-        for (i = 0; i < sdr->n; i++){
-			int j = (i + sdr->n / 2) % sdr->n;
-            x = (i * (sdr->screen->w - 1)) / sdr->n;
-            if (x >= sdr->screen->w) break;
-                            
-            d = sqrt(sqr(sdr->bins[j][0])+sqr(sdr->bins[j][1]));
-            //d = sqr(sdr->bins[j][0]) + sqr(sdr->bins[j][1]);
-            d = d/sdr->n;
-			d /= M_SQRT2; 
-			if (d > dmax) dmax = d;
-            // d is epsilon to 1.0
-            e = log10(d);
-            e -= log10(1.0/130072.0);
-            e /= -log10(1.0/130072.0);
-            //e -= log10(1.0/32768.0);
-            //e /= -log10(1.0/32768.0);
-            if (e<0) e = 0;  // slabsi signaly nez 1 bit
-			if (i < sdr->amplen) {
-				double gain = (1 - exp(-(0.2 * e)));
-				if (maxgain < gain) maxgain = gain;
-				if (mingain > gain) mingain = gain;
-				if (gain < 0.03) gain = 0.03;
-				if (i == 0) gain = 1.0;
-
-				sdr->amp[i] = e;
-				sdr->nea_amp[i] = sdr->nea_amp[i] * (1-gain) + e * gain;
-			}
-            col = (int)(e * (FFT_COLORS - 1.0));
-            //col = (int)(sdr->nea_amp[i] * (FFT_COLORS - 1.0));
-			if (oldx > x) if (col > sdr->wfvec[x]) sdr->wfvec[x] = col;
-		
-            for (; oldx <= x; oldx++){
-				if (oldx >= sdr->wfveclen) break;
-				if (col > sdr->wfvec[oldx]) sdr->wfvec[oldx] = col;
-			}
-
-                //z_putpixel(sdr->screen, oldx, sdr->screeny, sdr->pal[col]);
-        }   
-		for (i = 0; i < sdr->wfveclen; i++) z_putpixel(sdr->screen, i, sdr->screeny, sdr->pal[sdr->wfvec[i]]);
-		
-		//dbg("maxgain=%f  mingain=%f\n", maxgain, mingain);
-		if (tv2.tv_sec != oldsec){ // draw time marks
-			int x2 = 3 + (tv2.tv_sec % 10 == 0) * 4;
-			oldsec = tv2.tv_sec;
-			for (i = 0; i <= x2; i++) {
-				z_putpixel(sdr->screen, i, sdr->screeny, sdl->gr[14]);
-				z_putpixel(sdr->screen, sdr->screen->w - 1 - i, sdr->screeny, sdl->gr[14]);
-			}
-		}
-
-        sdr->screeny++;
-        if (sdr->screeny == sdr->screen->h) sdr->screeny = 0;
         MUTEX_UNLOCK(sdr->screen);
 
 		MUTEX_LOCK(sdr->filter);
@@ -1081,6 +1085,7 @@ gpointer sdr_thread_func(gpointer data){
 		if (tot > 0.0){
 			if (++loadn == 40){
 				sdr->load = cpusum * 100.0 / totsum;
+	            dbg("sdr->load = %d%%\n", sdr->load);	
 				totsum = 0.0;
 				cpusum = 0.0;
 				loadn = 0;
@@ -1089,7 +1094,7 @@ gpointer sdr_thread_func(gpointer data){
 				cpusum += cpu;
 			}
 		}
-		
+
 		gettimeofday(&tv1, NULL);
 		sdr->afdsp->write(sdr->afdsp, sdr->sndbuf, sdr->frames / playfactor);
 #ifdef HAVE_SNDFILE
@@ -1099,6 +1104,9 @@ gpointer sdr_thread_func(gpointer data){
 #endif
 		zselect_msg_send(zsel, "SDR;D");
     }
+    gettimeofday(&stop, NULL);
+    tot = z_difftimeval_double(&stop, &tvstart);
+    dbg("sdr_thread_func done %5.2f seconds\n", tot);
 	sdr->iqdsp->close2(sdr->iqdsp);
 	sdr->afdsp->close2(sdr->afdsp);
 #ifdef HAVE_SNDFILE
