@@ -220,7 +220,7 @@ int ssbd_play_file(struct ssbd *ssbd, gchar *pfilename){
 	z_wokna(wfilename);										   
     if (! (ssbd->sndfile = sf_open (wfilename, SFM_READ, &sfinfo))){   
 		char *sferr = z_1250_to_8859_2(g_strdup(sf_strerror (NULL)));
-        log_addf("Can't play %s - %s", wfilename, sferr);
+        log_addf(TRANSLATE("Can't play %s - %s"), wfilename, sferr);
 		g_free(sferr);
 		g_free(wfilename);
         return -1;
@@ -454,7 +454,7 @@ void ssbd_play_read_handler(struct ssbd *ssbd, gchar *str){
             if (gses->last_cq->ssb_repeat)
                 cq_ssb_wait(gses->last_cq);
             else{
-                gses->last_cq->type=MOD_NONE;
+                gses->last_cq->mode=MOD_NONE;
                 cq_abort(ssbd->recording);
             }
             peer_tx(aband, 0);
