@@ -378,6 +378,21 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
 		case 18:
 			qsop = iaru_round(q->qrb) * ctest->qsomult * band->qsomultb;
 			break;
+        case 19: // WNA, WSA
+            switch (q->mode){ 
+                case MOD_CW_CW:
+                    qsop = 6;
+                    break;
+                case MOD_SSB_SSB:
+                case MOD_SSB_CW:
+                case MOD_CW_SSB:
+                    qsop = 4;
+                    break;
+                case MOD_FM_FM:
+                    qsop = 2;
+                    break;
+            }
+            break;
 
     }
     if (q->dupe) qsop=0;
@@ -475,6 +490,9 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
         if (strcasecmp(ctest->excname, "USACA")==0){
             usaca_multiple_exc(s);
         }
+        if (ctest->exctype == EXC_MULTIPLIED){
+            if (g_hash_table_lookup(excdb->vexc, s) == NULL) *s = '\0';
+        }
         if (*s){
             if (g_hash_table_lookup_extended(st->excs, s, &orig_key, (gpointer*) &orig_value)){
                 orig_value->count++; 
