@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2016  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
     This program is free software; you can redistribute it and/or
@@ -251,7 +251,7 @@ struct subwin *new_subwin(enum sw_type type, int where_){
 						sdl->screen->format->Rmask, sdl->screen->format->Gmask, sdl->screen->format->Bmask, 0);
 				MUTEX_INIT(sw->screen);
 				sw->zoom = 10000;
-				  /*  sw->zoom = 1000; */
+				    //sw->zoom = 1000; 
 				sw->gdirty = 1;
 				map_update_layout(sw);
 				map_update_qth(sw);
@@ -2505,3 +2505,12 @@ void sw_highlight(struct subwin *sw, const char *text, const char *needle, int c
     
 }
 
+void sw_set_gdirty(enum sw_type type){
+	int i;
+	for (i = 0; i < gses->subwins->len; i++){
+        struct subwin *sw = (struct subwin *)g_ptr_array_index(gses->subwins, i);
+        if (sw->type != type) continue;
+        sw->gdirty = 1;
+	}
+	redraw_later();
+}
