@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2016  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -77,14 +77,13 @@ ZPtrArray *riglist = NULL;
 
 
 // ---------------- trigs functions
-struct trigs *init_trigs(void){
+void init_trigs(void){
 	int i, verbose = 0;
-	struct trigs *p;
 	
-	p = g_new0(struct trigs, 1);
-    p->qrg = 0.0;
-    p->rit = 0;
-	p->trigs = g_ptr_array_new();
+	gtrigs = g_new0(struct trigs, 1);
+    gtrigs->qrg = 0.0;
+    gtrigs->rit = 0;
+	gtrigs->trigs = g_ptr_array_new();
 
 	for (i = 0; i < cfg->crigs->len; i++){
 		struct config_rig *crig = (struct config_rig *)g_ptr_array_index(cfg->crigs, i);
@@ -116,10 +115,9 @@ struct trigs *init_trigs(void){
 		if (!crig->rig_enabled) continue;
 
 		trig = init_trig(crig);
-		if (trig != NULL) g_ptr_array_add(p->trigs, trig);
+		if (trig != NULL) g_ptr_array_add(gtrigs->trigs, trig);
 	}
 
-	return p;
 }
 
 void free_trigs(struct trigs *trigs){
@@ -236,13 +234,17 @@ void trigs_resend_freq(struct trigs *trigs, int rignr){
 	}
 }
 
-void trigs_set_ptt(struct trigs *trigs, ptt_t ptt){
+void trigs_set_ptt(struct trigs *trigs, int ptt){
 	int i;
 
+	if (trigs == NULL) return;
+
 	for (i = 0; i < trigs->trigs->len; i++){
 		struct trig *trig;
 		trig = (struct trig *)g_ptr_array_index(trigs->trigs, i);
-		if (trig->ptt_t2r) trig_set_ptt(trig, ptt);
+		if (trig->ptt_t2r != 0) {
+            trig_set_ptt(trig, ptt ? trig->ptt_t2r : RIG_PTT_OFF);
+        }
 	}
 }
 
@@ -269,6 +271,7 @@ struct trig *init_trig(struct config_rig *crig){
     trig->model = crig->rig_model;
     trig->filename = g_strdup(crig->rig_filename);
     trig->speed = crig->rig_speed;
+	trig->handshake_none = crig->rig_handshake_none;
 	trig->qrg_t2r = crig->rig_qrg_t2r;
 	trig->civaddr = crig->rig_civaddr;
 	trig->mode_t2r = crig->rig_mode_t2r;
@@ -452,7 +455,7 @@ gpointer trig_main(gpointer xxx){
 
     safe_strncpy(trig->rig->state.rigport.pathname, trig->filename, FILPATHLEN);
     if (trig->speed > 0) trig->rig->state.rigport.parm.serial.rate = trig->speed;
-    trig->rig->state.rigport.parm.serial.handshake = RIG_HANDSHAKE_NONE;
+	if (trig->handshake_none) trig->rig->state.rigport.parm.serial.handshake = RIG_HANDSHAKE_NONE;
     trig->rig->state.rigport.timeout = 2000;
     civaddr_str = g_strdup_printf("%d", trig->civaddr);
 	rig_set_conf(trig->rig, rig_token_lookup(trig->rig, "civaddr"), civaddr_str);
@@ -463,7 +466,7 @@ gpointer trig_main(gpointer xxx){
     rigerr = rig_open(trig->rig);
 	TRTR(1, "rig_open(#%d, %d)", trig->nr, (int)trig->model);
     if (rigerr != RIG_OK){
-        zselect_msg_send(zsel, "TRIG;%d;!;Can't open rig %d (%s)", trig->nr, (int)trig->model, trig->rig->state.rigport.pathname);
+        zselect_msg_send(zsel, "TRIG;%d;!;Can't open rig %d (%s), error %d (%s)", trig->nr, (int)trig->model, trig->rig->state.rigport.pathname, rigerr, trig_short_errstr((enum rig_errcode_e)rigerr));
 		TRTR(1, "rig_open(#%d)", trig->nr);
 		gtrigs->rigerr = trig->rigerr = rigerr;
 		rig_cleanup(trig->rig);
