@@ -102,8 +102,7 @@ struct tsdl *init_sdl(){
     
     setenv("SDL_VIDEO_X11_WMCLASS", "Tucnak", 1);
     if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE)){
- /*       log_addf(VTEXT(T_CANT_SET_GR_S), "SDL_Init"); not initialised here */
-        log_addf(VTEXT(T_CANT_SET_GRAPHICS1));
+        log_adds("Can't set graphics mode (SDL_Init)");
         free_sdl();
         return NULL;
     };  
@@ -158,7 +157,7 @@ struct tsdl *init_sdl(){
 		h = sdl->screen_h;
 	}
     if (sdl_setvideomode(w, h, 1)){
-        log_addf(VTEXT(T_CANT_SET_GRAPHICS2));
+        log_adds("Can't set graphics mode");
         free_sdl(); 
         return NULL;
     }
