@@ -530,7 +530,7 @@ int zosk_send(struct zosk *zosk){
 		if (ev.key.keysym.sym == SDLK_BACKSPACE && zosk->gs->len > 0){
 			g_string_erase(zosk->gs, zosk->gs->len - 1, 1);
 		}
-		if (ev.key.keysym.sym == SDLK_RETURN && zosk->flags && ZOSK_CLEARENTER){
+		if (ev.key.keysym.sym == SDLK_RETURN && (zosk->flags & ZOSK_CLEARENTER)){
 			g_string_truncate(zosk->gs, 0);
 		}
 	}
